Type: Package
Package: gimap
Title: Calculate Genetic Interactions for Paired CRISPR Targets
Version: 1.1.2
Authors@R: c(
  person("Candace", "Savonen",
         email = c("cansav09@gmail.com","csavonen@fredhutch.org"), role = c("aut", "cre")),
  person("Phoebe", "Parrish",
         email = "pcrparrish@gmail.com", role = c("aut")),
  person("Kate", "Isaac",
         email = "kisaac@fredhutch.org", role = c("aut")),
  person("Howard", "Baek",
         email = "howardbaek.fh@gmail.com", role = c("aut")),
  person("Daniel", "Grosso", role = c("aut")),
  person("Siobhan", "O'Brien", role = c("aut")),
  person("Alice", "Berger", role = c("aut")))
Author: Candace Savonen [aut, cre],
  Phoebe Parrish [aut],
  Kate Isaac [aut],
  Howard Baek [aut],
  Daniel Grosso [aut],
  Siobhan O'Brien [aut],
  Alice Berger [aut]
Maintainer: Candace Savonen <cansav09@gmail.com>
Description: Helps find meaningful patterns in complex genetic experiments. First gimap takes data from paired CRISPR (Clustered regularly interspaced short palindromic repeats) screens that has been pre-processed to counts table of paired gRNA (guide Ribonucleic Acid) reads. The input data will have cell counts for how well cells grow (or don't grow) when different genes or pairs of genes are disabled. The output of the 'gimap' package is genetic interaction scores which are  the distance between the observed CRISPR score and the expected CRISPR score. The expected CRISPR scores are what we expect for the CRISPR values to be for two unrelated genes. The further away an observed CRISPR score is from its expected score the more we suspect genetic interaction. The work in this package is based off of original research from the Alice Berger lab at Fred Hutchinson Cancer Center (2021) <doi:10.1016/j.celrep.2021.109597>.
License: GPL-3
URL: https://github.com/FredHutch/gimap
BugReports: https://github.com/FredHutch/gimap/issues
Imports: readr, dplyr, tidyr, rmarkdown, vroom, ggplot2, magrittr,
        pheatmap, purrr, janitor, utils, stats, stringr, httr,
        jsonlite, openssl
Suggests: testthat (>= 3.0.0), roxygen2, kableExtra, knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-02-05 16:36:51 UTC; candace
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-02-05 22:10:02 UTC
Built: R 4.6.0; ; 2026-04-26 18:43:30 UTC; unix

Type: Package
Package: gimap
Title: Calculate Genetic Interactions for Paired CRISPR Targets
Version: 1.1.3
Authors@R: c(
  person("Candace", "Savonen",
         email = c("cansav09@gmail.com","csavonen@fredhutch.org"),
         role = c("aut", "cre"),
         comment = c(ORCID = "0000-0001-6331-7070")),
  person("Phoebe C.R.", "Parrish",
         email = "pcrparrish@gmail.com", role = c("aut"),
         comment = c(ORCID = "0000-0002-3530-0105")),
  person("Kathryn J.", "Isaac",
         email = "kisaac@fredhutch.org", role = c("aut"),
         comment = c(ORCID = "0000-0001-8701-4197")),
  person("Daniel", "Groso", role = c("aut")),
  person("Marissa", "Fujimoto", role = c("aut")),
  person("Siobhan", "O'Brien", role = c("aut"),
         comment = c(ORCID = "0000-0003-4958-1080")),
  person("Alice H.", "Berger", role = c("aut"),
         comment = c(ORCID = "0000-0002-6538-2658")))
Description: Helps find meaningful patterns in complex genetic experiments. First gimap takes data from paired CRISPR (Clustered regularly interspaced short palindromic repeats) screens that has been pre-processed to counts table of paired gRNA (guide Ribonucleic Acid) reads. The input data will have cell counts for how well cells grow (or don't grow) when different genes or pairs of genes are disabled. The output of the 'gimap' package is genetic interaction scores which are  the distance between the observed CRISPR score and the expected CRISPR score. The expected CRISPR scores are what we expect for the CRISPR values to be for two unrelated genes. The further away an observed CRISPR score is from its expected score the more we suspect genetic interaction. The work in this package is based off of original research from the Alice Berger lab at Fred Hutchinson Cancer Center (2021) <doi:10.1016/j.celrep.2021.109597>.
License: BSD_3_clause + file LICENSE
URL: https://github.com/FredHutch/gimap
BugReports: https://github.com/FredHutch/gimap/issues
Depends: R (>= 3.5.0)
Imports: readr, dplyr, tidyr, rmarkdown, vroom, ggplot2, magrittr,
        pheatmap, purrr, janitor, utils, stats, stringr, httr,
        jsonlite, openssl
Suggests: testthat (>= 3.0.0), roxygen2, kableExtra, knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-07-09 16:14:01 UTC; candace
Author: Candace Savonen [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6331-7070>),
  Phoebe C.R. Parrish [aut] (ORCID:
    <https://orcid.org/0000-0002-3530-0105>),
  Kathryn J. Isaac [aut] (ORCID: <https://orcid.org/0000-0001-8701-4197>),
  Daniel Groso [aut],
  Marissa Fujimoto [aut],
  Siobhan O'Brien [aut] (ORCID: <https://orcid.org/0000-0003-4958-1080>),
  Alice H. Berger [aut] (ORCID: <https://orcid.org/0000-0002-6538-2658>)
Maintainer: Candace Savonen <cansav09@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-09 22:10:02 UTC

# Generated by roxygen2: do not edit by hand

export("%>%")
export(calc_gi)
export(delete_annotation)
export(delete_example_data)
export(example_data_folder)
export(get_example_data)
export(get_figshare)
export(gimap_annotate)
export(gimap_filter)
export(gimap_normalize)
export(gimap_object)
export(plot_crispr)
export(plot_exp_v_obs_scatter)
export(plot_rank_scatter)
export(plot_targets)
export(plot_theme)
export(plot_volcano)
export(qc_cdf)
export(qc_constructs_countzero_bar)
export(qc_cor_heatmap)
export(qc_filter_plasmid)
export(qc_filter_zerocounts)
export(qc_plasmid_histogram)
export(qc_sample_hist)
export(qc_variance_hist)
export(run_qc)
export(save_example_timepoint_data)
export(save_example_treatment_data)
export(setup_data)
export(supported_cell_lines)
import(dplyr)
import(ggplot2)
import(httr)
importFrom(dplyr,across)
importFrom(dplyr,if_any)
importFrom(dplyr,mutate)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(janitor,clean_names)
importFrom(jsonlite,fromJSON)
importFrom(magrittr,"%>%")
importFrom(openssl,aes_cbc_decrypt)
importFrom(pheatmap,pheatmap)
importFrom(purrr,map)
importFrom(purrr,reduce)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(stringr,word)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unite)
importFrom(utils,browseURL)
importFrom(utils,download.file)
importFrom(utils,installed.packages)
importFrom(utils,menu)
importFrom(utils,unzip)

# gimap - Genetic Interaction MAPping for dual target CRISPR screens

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->
**Table of Contents**  *generated with [DocToc](https://github.com/thlorenz/doctoc)*

  - [Background on paired guide CRISPR](#background-on-paired-guide-crispr)
  - [What is gimap?](#what-is-gimap)
  - [Example output from this package:](#example-output-from-this-package)
  - [pgPEN library design](#pgpen-library-design)
  - [About Genetic Interaction Scores](#about-genetic-interaction-scores)
      - [Expected CRISPR scores](#expected-crispr-scores)
    - [Normalization](#normalization)
  - [Prerequisites](#prerequisites)
  - [Manual data download (Figshare)](#manual-data-download-figshare)
  - [Getting Started Tutorial](#getting-started-tutorial)
  - [Docker image](#docker-image)
  - [Citations:](#citations)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Background on paired guide CRISPR

Some genes have "backup copies" - these are called paralog genes. Think of it like having a spare tire in your car - if one fails, the other can often do a similar job. This redundancy makes it tricky to study what these genes do, because if you knock out just one gene, its partner might pick up the slack.

CRISPR allows genes to be knocked out so we can see what their function is. But because of paralog redundancy it can be hard to parse out what genes are actually involved. So instead of just targeting one gene at a time, paired guide CRISPR screening allows us to knock out two genes simultaneously.

It's particularly useful for understanding:
- What happens when you disable both backup copies of a gene
- How different genes might work together
- Which gene combinations are essential for cell survival

## What is gimap?

gimap - is a software tool that helps make sense of paired CRISPR screening data. Here's what it does:

1. Takes data from paired CRISPR screens that has been pre-processed by the pgmap software, or any counts table of paired gRNA reads
2. The input data will have cell counts for how well cells grow (or don't grow) when different genes or pairs of genes are disabled
3. gimap can take this data and helps identify interesting patterns, like:
   - When disabling two genes together is more devastating than you'd expect from disabling them individually (called synthetic lethality)
   - When genes work together cooperatively
   - Which gene combinations might be important in diseases like cancer
gimap can help find meaningful patterns in complex genetic experiments. It's particularly focused on analyzing data generated by screening cells with the Berger Lab paired gRNA CRISPR screening library, called pgPEN (paired guide RNAs for genetic interaction mapping).
<br>

## Example output from this package:

Here's an example Volcano plot with the genetic interaction scores and statistical tests from this package:
<br>
![Example results from this package](https://github.com/FredHutch/gimap/blob/main/vignettes/images/volcano_plot.png?raw=true)
<br>
<br>
And here is what the main summary calculations look like:
![Example results table](https://github.com/FredHutch/gimap/blob/main/vignettes/images/results_table.png?raw=true)
<br>
<br>
The gimap package is based off of the [code and research from the Berger Lab](https://github.com/FredHutch/GI_mapping)

[Go to **Quick Start** to get going!](https://fredhutch.github.io/gimap/articles/quick-start.html)

## pgPEN library design

The `gimap` package is based off of an original paired CRISPR knockout library design from the Berger lab.

There are four target types included in the custom Berger lab pgPEN library:

- `double_targeting` - these cells have two different genes that have been knocked out with pgRNAs. This is sometimes noted as `"gene_gene"`
- `single_targeting` - these cells have one gene that have been knocked out with pgRNA and another that is designed to not target any gene. This is includes `"gene_ctrl"` and `"ctrl_gene"`.
- `positive_control` - these cells have one **essential control** gene that has been knocked out with pgRNA and another that is designed to NOT target any genes. These are also noted as This is includes `"gene_ctrl"` and `"ctrl_gene"` but for when the gene is an essential gene, e.g. required in most cell lines for survival.
- `negative_control` - these cells have two pgRNAs designed to NOT target any genes. This is noted as `"ctrl_ctrl"`.

In the instance of a single gene pair: e.g. `geneA_geneB`, there are 32 different constructs related to it. `32 = 16 single targeting and 16 double targeting`

- There are 16 `double targeting`: `geneA_geneBpg1, geneA_geneBpg2, ... geneA_geneBpg16`.
- 4 unique targeting sequences for `gene A` * 4 unique targeting sequences for `gene B` = 16 unique combos of double targeting constructs.
- There are 16 `single targeting` in relation to this `geneA_geneB` construct. There are: `geneA_ctrl` and `ctrl_geneB` sequences.
- (4 unique targeting sequences (`"geneA"`) * 2 non targeting sequences (`"ctrl"`) = 8 constructs)
- (2 non targeting sequences (`"ctrl"`) * 4 unique targeting sequences (`"geneB"`) = 8 constructs)

## About Genetic Interaction Scores

The output of the `gimap` package is genetic interaction scores which are _the distance between the observed CRISPR score and the expected CRISPR score._ The expected CRISPR scores are what we expect for the CRISPR values to be for two unrelated genes. The further away an observed CRISPR score is from its expected score the more we suspect genetic interaction.

This can be true in a positive way (a CRISPR knockout pair caused more cell proliferation than expected -- called cooperativity) or in a negative way (a CRISPR knockout pair caused more cell lethality than expected -- called synthetic lethality).

The genetic interaction scores are based on a linear model calculated for each sample where `observed_crispr_single` is the outcome variable and `expected_crispr_single` is the predictor variable.

**For each sample:**
```
lm(observed_crispr_single ~ expected_crispr_single)
```
Using `y = mx+b`, we can fill in the following values:
* `y` = observed CRISPR score
* `x` = expected CRISPR score
* `m` = slope from linear model for this sample
* `b` = intercept from linear model for this sample

The intercept and slope from this linear model are used to adjust the CRISPR scores for each sample:
```
single_target_gi_score = observed single crispr - (intercept + slope * expected single crispr)
double_target_gi_score = double crispr score - (intercept + slope * expected double crispr)
```

These single and double target genetic interaction scores are calculated at the construct level and are then summarized using a t-test to see if the the distribution of the set of double targeting constructs is significantly different than the overall distribution of single targeting constructs. After multiple testing correction, FDR values are reported. Low FDR value for a double construct means high likelihood of paralog redundancy, also known as synthetic lethality.

#### Expected CRISPR scores

Expected CRISPR scores are calculated based on the pgRNAs included.

For a `double_targeting` construct we would expect the CRISPR score to be the sum of the single targeting CRISPR scores which use the same gRNA sequences present in the pgRNA.
```
expected crispr double = single target crispr 1 + single target crispr 2
```

So in the instance of the double construct `GSN_SCIN_pg1` we would expect its CRISPR score to be the same as `GSN_ctrl` + `ctrl_SCIN` where the pgRNA used to target GSN and SCIN respectively are the same.

For `single_targeting` constructs we would expect the CRISPR score to be the sum of a single target plus the mean of the control pgRNAs from the `negative control constructs` (or double non-targeting CRISPR constructs).

```
expected crispr single = single target crispr 1 + mean negative control for the pgRNA
```
So for `TRPC5_nt1` single target we would take its CRISPR score + the mean of the `nt1` sequence across the constructs where it is included in a double non targeting construct.

### t-tests

Genetic Interaction scores are used to calculate p values using a t-test.

For each gene pair...

A t-test compares:
1) The distribution of all single target GI scores
2) The distribution of the data from the 16 double targeting constructs for the particular gene pair

These t-tests can be performed with all samples combined or for each sample using the `stat_by_rep` argument in `calc_gi()`. False discovery rate is then applied after the p values are obtained to account for multiple testing correction.

### Normalization

gimap takes in a counts matrix that represents the number of cells that have each type of pgRNA. This data needs some normalization before CRISPR scores and Genetic Interaction scores can be calculated.

There are four steps of normalization.
1. `Calculate log2CPM` - First we account for different read depths across samples and transform data to log2 counts per million reads.
```
log2((counts / total counts for sample)) * 1 million) + 1)
```

2. `Calculate log2 fold change` - This is done by subtracting the log2CPM for the pre-treatment from each sample.  control is what is highlighted. The pretreatment is the day 0 of CRISPR treatment, before CRISPR pgRNAs have taken effect.
```
log2FC = log2CPM for each sample - pretreament log2CPM
```

Since the pgPEN library uses non-targeting controls, we adjust for the fact that single-targeting pgRNAs generate only two double-strand breaks (1 per allele), whereas the double-targeting pgRNAs generate four DSBs. To do this, we set the median LFC of each group to zero.

Calculate medians of based on single and double targeting and subtract these medians from `log2FC adjusted`

3. CRISPR scores: `Normalize by negative and positive controls` - Calculate a negative control median for each sample and a positive control median for each sample and divide each log2FC by this value. In this version then we are normalizing by the median difference of the negative and positive controls.
```
# FOR EACH SAMPLE:
log2FC adjusted =
(log2FC - log2FC median negative control) /
(log2FC median negative control - median log2FC positive control)
```

## Prerequisites

In order to run this pipeline you will need R and to install the `gimap` package and its dependencies. In R you can run this to install the package:
```
install.packages("remotes")
remotes::install_github("FredHutch/gimap")
```

## Manual data download (Figshare)

To keep the package size minimal for CRAN, example and annotation files are not bundled.
You can download the files manually from Figshare and place them in a writable data
directory. The package will also download these files automatically on first use.

### Where to put files

```
example_dir <- gimap::example_data_folder()
dir.create(example_dir, recursive = TRUE, showWarnings = FALSE)
```

### Example data (Figshare item 28264271)

These files are used by `get_example_data()`:

- `PP_pgPEN_HeLa_counts.txt`
- `counts_pgPEN_PC9_example.tsv`
- `pgRNA_ID_pgPEN_library_comp.csv`
- `pgPEN_annotations.txt`

Download with:

```
gimap::get_figshare(return_list = TRUE, item = "28264271")
gimap::get_figshare(file_name = "PP_pgPEN_HeLa_counts.txt", item = "28264271", output_dir = example_dir)
gimap::get_figshare(file_name = "counts_pgPEN_PC9_example.tsv", item = "28264271", output_dir = example_dir)
gimap::get_figshare(file_name = "pgRNA_ID_pgPEN_library_comp.csv", item = "28264271", output_dir = example_dir)
gimap::get_figshare(file_name = "pgPEN_annotations.txt", item = "28264271", output_dir = example_dir)
```

### DepMap/CCLE annotation data (Figshare item 19700056)

These files are used by `gimap_annotate()` when cell line annotation is enabled:

- `CCLE_expression.csv`
- `CCLE_gene_cn.csv`
- `Achilles_common_essentials.csv`

Download with:

```
gimap::get_figshare(return_list = TRUE, item = "19700056")
gimap::get_figshare(file_name = "CCLE_expression.csv", item = "19700056", output_dir = example_dir)
gimap::get_figshare(file_name = "CCLE_gene_cn.csv", item = "19700056", output_dir = example_dir)
gimap::get_figshare(file_name = "Achilles_common_essentials.csv", item = "19700056", output_dir = example_dir)
```

Note: The DepMap files can be large (hundreds of MB to several GB). Check the
Figshare listing for exact sizes before downloading.

## Getting Started Tutorial

Now you can [go to our quick start tutorial to get started!](https://fredhutch.github.io/gimap/articles/quick-start.html)

We also have tutorial examples that show how to run timepoint or treatment experimental set ups with gimap:

- [Timepoint example](https://fredhutch.github.io/gimap/articles/timepoint-example.html)
- [Treatment example](https://fredhutch.github.io/gimap/articles/treatment_example.html)

Follow the steps there that will walk you through the example data. Then you can tailor that tutorial to use your own data.

## Docker image

You can use gimap interactively using the docker image like this:
```
docker run -d -v $PWD:/home/rstudio -e PASSWORD=rstudio -p 8787:8787 cansav09/gimap
```
Where all files at your current working directory will be available to this container.

You can access this container's rstudio interactively and work from here by typing your browser:
```
http://localhost:8787/
```
And then putting "rstudio" for both the username and password.

## Citations:
- https://pubmed.ncbi.nlm.nih.gov/34469736/
- https://www.nature.com/articles/s43586-021-00093-4

![See metrics about this repository here](https://cauldron.io/project/8779/stats.svg)


\* This work is funded by NCI grant R01CA262556 as well as funding from the Translational Data Science IRC of Fred Hutchinson Cancer Center.

Email csavonen@fredhutch.org with any questions!

# gimap - Genetic Interaction MAPping for dual target CRISPR screens

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->
**Table of Contents**  *generated with [DocToc](https://github.com/thlorenz/doctoc)*

- [Background on paired guide CRISPR](#background-on-paired-guide-crispr)
- [What is gimap?](#what-is-gimap)
- [Example output from this package:](#example-output-from-this-package)
- [pgPEN library design](#pgpen-library-design)
- [About Genetic Interaction Scores](#about-genetic-interaction-scores)
    - [Expected CRISPR scores](#expected-crispr-scores)
  - [t-tests](#t-tests)
  - [Normalization](#normalization)
- [Prerequisites](#prerequisites)
  - [Release version (CRAN)](#release-version-cran)
  - [Development version (GitHub)](#development-version-github)
- [Manual data download (Figshare)](#manual-data-download-figshare)
  - [Where to put files](#where-to-put-files)
  - [Example data (Figshare item 28264271)](#example-data-figshare-item-28264271)
  - [DepMap/CCLE annotation data (Figshare item 19700056)](#depmapccle-annotation-data-figshare-item-19700056)
- [Getting Started Tutorial](#getting-started-tutorial)
- [Docker image](#docker-image)
- [Citations:](#citations)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Background on paired guide CRISPR

Some genes have "backup copies" - these are called paralog genes. Think of it like having a spare tire in your car - if one fails, the other can often do a similar job. This redundancy makes it tricky to study what these genes do, because if you knock out just one gene, its partner might pick up the slack.

CRISPR allows genes to be knocked out so we can see what their function is. But because of paralog redundancy it can be hard to parse out what genes are actually involved. So instead of just targeting one gene at a time, paired guide CRISPR screening allows us to knock out two genes simultaneously.

It's particularly useful for understanding:
- What happens when you disable both backup copies of a gene
- How different genes might work together
- Which gene combinations are essential for cell survival

## What is gimap?

gimap - is a software tool that helps make sense of paired CRISPR screening data. Here's what it does:

1. Takes data from paired CRISPR screens that has been pre-processed by the pgmap software, or any counts table of paired gRNA reads
2. The input data will have a count of paired gRNA reads which is one per cell, representing cell counts for how well cells grow (or don't grow) when different genes or pairs of genes are disabled
3. gimap can take this data and helps identify interesting patterns, like:
   - When disabling two genes together is more devastating than you'd expect from disabling them individually (called synthetic lethality)
   - When genes work together cooperatively
   - Which gene combinations might be important in diseases like cancer
gimap can help find meaningful patterns in complex genetic experiments. It's particularly focused on analyzing data generated by screening cells with the Berger Lab paired gRNA CRISPR screening library, called pgPEN (paired guide RNAs for genetic interaction mapping).
<br>

## Example output from this package:

Here's an example Volcano plot with the genetic interaction scores and statistical tests from this package:
<br>
![Example results from this package](https://github.com/FredHutch/gimap/blob/main/vignettes/images/volcano_plot.png?raw=true)
<br>
<br>
And here is what the main summary calculations look like:
![Example results table](https://github.com/FredHutch/gimap/blob/main/vignettes/images/results_table.png?raw=true)
<br>
<br>
The gimap package is based off of the [code and research from the Berger Lab](https://github.com/FredHutch/GI_mapping)

[Go to **Quick Start** to get going!](https://fredhutch.github.io/gimap/articles/quick-start.html)

## pgPEN library design

The `gimap` package is based off of an original paired CRISPR knockout library design from the Berger lab.

There are four target types included in the custom Berger lab pgPEN library:

- `double_targeting` - these cells have two different genes that have been knocked out with pgRNAs. This is sometimes noted as `"gene_gene"`
- `single_targeting` - these cells have one gene that have been knocked out with pgRNA and another that is designed to not target any gene. This is includes `"gene_ctrl"` and `"ctrl_gene"`.
- `positive_control` - these cells have one **essential control** gene that has been knocked out with pgRNA and another that is designed to NOT target any genes. These are also noted as This is includes `"gene_ctrl"` and `"ctrl_gene"` but for when the gene is an essential gene, e.g. required in most cell lines for survival.
- `negative_control` - these cells have two pgRNAs designed to NOT target any genes. This is noted as `"ctrl_ctrl"`.

In the instance of a single gene pair: e.g. `geneA_geneB`, there are 32 different constructs related to it. `32 = 16 single targeting and 16 double targeting`

- There are 16 `double targeting`: `geneA_geneBpg1, geneA_geneBpg2, ... geneA_geneBpg16`.
- 4 unique targeting sequences for `gene A` * 4 unique targeting sequences for `gene B` = 16 unique combos of double targeting constructs.
- There are 16 `single targeting` in relation to this `geneA_geneB` construct. There are: `geneA_ctrl` and `ctrl_geneB` sequences.
- (4 unique targeting sequences (`"geneA"`) * 2 non targeting sequences (`"ctrl"`) = 8 constructs)
- (2 non targeting sequences (`"ctrl"`) * 4 unique targeting sequences (`"geneB"`) = 8 constructs)

## About Genetic Interaction Scores

The output of the `gimap` package is genetic interaction scores which are _the distance between the observed CRISPR score and the expected CRISPR score._ The expected CRISPR scores are what we expect for the CRISPR values to be for two unrelated genes. The further away an observed CRISPR score is from its expected score the more we suspect genetic interaction.

This can be true in a positive way (a CRISPR knockout pair caused more cell proliferation than expected -- called cooperativity) or in a negative way (a CRISPR knockout pair caused more cell lethality than expected -- called synthetic lethality).

The genetic interaction scores are based on a linear model calculated for each sample where `observed_crispr_single` is the outcome variable and `expected_crispr_single` is the predictor variable.

**For each sample:**
```
lm(observed_crispr_single ~ expected_crispr_single)
```
Using `y = mx+b`, we can fill in the following values:
* `y` = observed CRISPR score
* `x` = expected CRISPR score
* `m` = slope from linear model for this sample
* `b` = intercept from linear model for this sample

The intercept and slope from this linear model are used to adjust the CRISPR scores for each sample:
```
single_target_gi_score = observed single crispr - (intercept + slope * expected single crispr)
double_target_gi_score = double crispr score - (intercept + slope * expected double crispr)
```

These single and double target genetic interaction scores are calculated at the construct level and are then summarized using a t-test to see if the the distribution of the set of double targeting constructs is significantly different than the overall distribution of single targeting constructs. After multiple testing correction, FDR values are reported. Low FDR value for a double construct means high likelihood of paralog redundancy, also known as synthetic lethality.

#### Expected CRISPR scores

Expected CRISPR scores are calculated based on the pgRNAs included.

For a `double_targeting` construct we would expect the CRISPR score to be the sum of the single targeting CRISPR scores which use the same gRNA sequences present in the pgRNA.
```
expected crispr double = single target crispr 1 + single target crispr 2
```

So in the instance of the double construct `GSN_SCIN_pg1` we would expect its CRISPR score to be the same as `GSN_ctrl` + `ctrl_SCIN` where the pgRNA used to target GSN and SCIN respectively are the same.

For `single_targeting` constructs we would expect the CRISPR score to be the sum of a single target plus the mean of the control pgRNAs from the `negative control constructs` (or double non-targeting CRISPR constructs).

```
expected crispr single = single target crispr 1 + mean negative control for the pgRNA
```
So for `TRPC5_nt1` single target we would take its CRISPR score + the mean of the `nt1` sequence across the constructs where it is included in a double non targeting construct.

### t-tests

Genetic Interaction scores are used to calculate p values using a t-test.

For each gene pair...

A t-test compares:
1) The distribution of all single target GI scores
2) The distribution of the data from the 16 double targeting constructs for the particular gene pair

These t-tests can be performed with all samples combined or for each sample using the `stat_by_rep` argument in `calc_gi()`. False discovery rate is then applied after the p values are obtained to account for multiple testing correction.

### Normalization

gimap takes in a counts matrix that represents the number of cells that have each type of pgRNA. This data needs some normalization before CRISPR scores and Genetic Interaction scores can be calculated.

There are four steps of normalization.
1. `Calculate log2CPM` - First we account for different read depths across samples and transform data to log2 counts per million reads.
```
log2((counts / total counts for sample)) * 1 million) + 1)
```

2. `Calculate log2 fold change` - This is done by subtracting the log2CPM for the pre-treatment from each sample.  control is what is highlighted. The pretreatment is the day 0 of CRISPR treatment, before CRISPR pgRNAs have taken effect.
```
log2FC = log2CPM for each sample - pretreament log2CPM
```

Since the pgPEN library uses non-targeting controls, we adjust for the fact that single-targeting pgRNAs generate only two double-strand breaks (1 per allele), whereas the double-targeting pgRNAs generate four DSBs. To do this, we set the median LFC of each group to zero.

Calculate medians of based on single and double targeting and subtract these medians from `log2FC adjusted`

3. CRISPR scores: `Normalize by negative and positive controls` - Calculate a negative control median for each sample and a positive control median for each sample and divide each log2FC by this value. In this version then we are normalizing by the median difference of the negative and positive controls.
```
# FOR EACH SAMPLE:
log2FC adjusted =
(log2FC - log2FC median negative control) /
(log2FC median negative control - median log2FC positive control)
```

## Prerequisites

In order to run this pipeline you will need R and to install the `gimap` package and its dependencies.

### Release version (CRAN)

The [latest release on CRAN](https://cran.r-project.org/package=gimap) is the recommended install for most users:

```
install.packages("gimap")
```

### Development version (GitHub)

The `main` branch on GitHub may contain changes that are not yet on CRAN. If you need the development build, install with the [remotes](https://cran.r-project.org/package=remotes) package:

```
install.packages("remotes")
remotes::install_github("FredHutch/gimap")
```

To install from a specific branch or tag, pass the `ref` argument (for example `ref = "main"` or a release tag from the [releases page](https://github.com/FredHutch/gimap/releases)).

## Manual data download (Figshare)

To keep the package size minimal for CRAN, example and annotation files are not bundled.
`get_example_data()` downloads Figshare examples into a session-specific folder under
`tempdir()` (see `example_data_folder()`). For a permanent copy, pass `data_dir` to
`get_example_data()` or download manually into any directory you choose.

### Where to put files (manual download)

If you use `get_figshare()` yourself, pick a writable directory (or use the same path
as `example_data_folder()` for consistency):

```
example_dir <- gimap::example_data_folder()
```

### Example data (Figshare item 28264271)

You can browse this item on Figshare here:
[gimap example data (Figshare item 28264271)](https://figshare.com/articles/dataset/gimap_example_data/28264271)
(DOI: [10.6084/m9.figshare.28264271](https://doi.org/10.6084/m9.figshare.28264271)).

These files are used by `get_example_data()`:

- `PP_pgPEN_HeLa_counts.txt`
- `counts_pgPEN_PC9_example.tsv`
- `pgRNA_ID_pgPEN_library_comp.csv`
- `pgPEN_annotations.txt`

Download with:

```
gimap::get_figshare(return_list = TRUE, item = "28264271")
gimap::get_figshare(file_name = "PP_pgPEN_HeLa_counts.txt", item = "28264271", output_dir = example_dir)
gimap::get_figshare(file_name = "counts_pgPEN_PC9_example.tsv", item = "28264271", output_dir = example_dir)
gimap::get_figshare(file_name = "pgRNA_ID_pgPEN_library_comp.csv", item = "28264271", output_dir = example_dir)
gimap::get_figshare(file_name = "pgPEN_annotations.txt", item = "28264271", output_dir = example_dir)
```

### DepMap/CCLE annotation data (Figshare item 19700056)

You can browse this item on Figshare here:
[DepMap 22Q2 Public (Figshare item 19700056)](https://figshare.com/articles/dataset/DepMap_22Q2_Public/19700056)
(DOI: [10.6084/m9.figshare.19700056](https://doi.org/10.6084/m9.figshare.19700056)).

These files are used by `gimap_annotate()` when cell line annotation is enabled:

- `CCLE_expression.csv`
- `CCLE_gene_cn.csv`
- `Achilles_common_essentials.csv`

Download with:

```
gimap::get_figshare(return_list = TRUE, item = "19700056")
gimap::get_figshare(file_name = "CCLE_expression.csv", item = "19700056", output_dir = example_dir)
gimap::get_figshare(file_name = "CCLE_gene_cn.csv", item = "19700056", output_dir = example_dir)
gimap::get_figshare(file_name = "Achilles_common_essentials.csv", item = "19700056", output_dir = example_dir)
```

Note: The DepMap files can be large (hundreds of MB to several GB). Check the
Figshare listing for exact sizes before downloading.

## Getting Started Tutorial

Now you can [go to our quick start tutorial to get started!](https://fredhutch.github.io/gimap/articles/quick-start.html)

We also have tutorial examples that show how to run timepoint or treatment experimental set ups with gimap:

- [Timepoint example](https://fredhutch.github.io/gimap/articles/timepoint-example.html)
- [Treatment example](https://fredhutch.github.io/gimap/articles/treatment_example.html)

Follow the steps there that will walk you through the example data. Then you can tailor that tutorial to use your own data.

## Docker image

You can use gimap interactively using the docker image like this:
```
docker run -d -v $PWD:/home/rstudio -e PASSWORD=rstudio -p 8787:8787 cansav09/gimap
```
Where all files at your current working directory will be available to this container.

You can access this container's rstudio interactively and work from here by typing your browser:
```
http://localhost:8787/
```
And then putting "rstudio" for both the username and password.

## Citations:
- https://pubmed.ncbi.nlm.nih.gov/34469736/
- https://www.nature.com/articles/s43586-021-00093-4

![See metrics about this repository here](https://cauldron.io/project/8779/stats.svg)


\* This work was funded in part by the Translational Data Science Integrated Research Center (TDS-IRC) of Fred Hutchinson Cancer Center, NIH/NCI R01CA262556 to A.H.B., the American Cancer Society to A.H.B., NIH/NHGRI R25HG012337 to M.F., and a Washington Research Foundation postdoctoral fellowship to S.O.

Email csavonen@fredhutch.org with any questions!

#' Create a CDF for the pgRNA normalized counts
#' @description This function uses pivot_longer to rearrange the data for plotting
#'  and then plots a CDF of the normalized counts
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function
#' which contains the transformed data
#' @param wide_ar aspect ratio, default is 0.75
#' @importFrom tidyr pivot_longer unite
#' @importFrom ggplot2 ggplot labs
#' @return counts_cdf a ggplot
#' @export
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap")
#' qc_cdf(gimap_dataset)
#' }
qc_cdf <- function(gimap_dataset, wide_ar = 0.75) {
  long_form <-
    tidyr::pivot_longer(data.frame(gimap_dataset$transformed_data$log2_cpm),
      everything(),
      names_to = "sample",
      values_to = "count_normalized"
    )

  counts_cdf <- ggplot(long_form, aes(x = count_normalized, color = sample)) +
    stat_ecdf() +
    labs(
      x = "-log10(count/total_count)",
      y = "Expected_pgRNAs",
      color = "Sample"
    ) +
    plot_options() +
    plot_theme() +
    theme(aspect.ratio = wide_ar)

  return(counts_cdf)
}

#' Create a histogram for the pgRNA log2 CPMs, faceted by sample
#' @description This function uses pivot_longer to rearrange the data for plotting and then
#' plots sample specific histograms of the pgRNA cpm's
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function
#' which contains the transformed data
#' @param wide_ar aspect ratio, default is 0.75
#' @importFrom tidyr pivot_longer
#' @import ggplot2
#' @return sample_cpm_histogram a ggplot
#' @export
#' @examples \dontrun{
#' gimap_dataset <- get_example_data("gimap")
#' qc_sample_hist(gimap_dataset)
#' }
qc_sample_hist <- function(gimap_dataset, wide_ar = 0.75) {
  long_form <-
    tidyr::pivot_longer(data.frame(gimap_dataset$transformed_data$log2_cpm),
      everything(),
      names_to = "sample",
      values_to = "log2_cpm"
    )

  sample_cpm_histogram <- ggplot(long_form, aes(x = log2_cpm, fill = sample)) +
    geom_histogram(color = "black", binwidth = 0.5) +
    plot_options() +
    plot_theme() +
    theme(
      aspect.ratio = wide_ar,
      legend.position = "none"
    ) +
    facet_wrap(~sample, scales = "free_y", ncol = ceiling(ncol(gimap_dataset$raw_counts) / 2))

  return(sample_cpm_histogram)
}

#' Create a histogram for the variance within replicates for each pgRNA
#' @description This function uses pivot_longer to rearrange the data for plotting,
#'  finds the variance for each pgRNA construct (using row number as a proxy) and then
#'  plots a histogram of these variances
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function
#' which contains the transformed data
#' @param filter_replicates_target_col default is NULL; Which sample columns are
#' replicates whose variation you'd like to analyze; If NULL, the last 3 sample columns are used
#' @param wide_ar aspect ratio, default is 0.75
#' @importFrom tidyr pivot_longer
#' @importFrom magrittr %>%
#' @importFrom stats var
#' @import ggplot2
#' @import dplyr
#' @return a ggplot histogram
#' @export
#' @examples \dontrun{
#' gimap_dataset <- get_example_data("gimap")
#' qc_variance_hist(gimap_dataset)
#' }
qc_variance_hist <- function(gimap_dataset, filter_replicates_target_col = NULL, wide_ar = 0.75) {
  if (is.null(filter_replicates_target_col)) {
    filter_replicates_target_col <- c(
      (ncol(gimap_dataset$transformed_data$log2_cpm) - 2):ncol(gimap_dataset$transformed_data$log2_cpm)
    )
  } # last 3 columns of the data

  return(
    gimap_dataset$transformed_data$log2_cpm[, filter_replicates_target_col] %>%
      as.data.frame() %>%
      mutate(row = row_number()) %>%
      tidyr::pivot_longer(-row) %>%
      group_by(row) %>%
      dplyr::summarize(var = var(value)) %>%
      ggplot(aes(x = var)) +
      geom_histogram(binwidth = 0.1) +
      theme(
        panel.background = element_blank(),
        panel.grid = element_blank(),
        aspect.ratio = wide_ar
      ) +
      xlab("variance") +
      ylab("pgRNA construct count")
  )
}

#' Create a bar graph that shows the number of replicates with a zero count for pgRNA constructs flagged by the zero count filter
#' @description This bar graph first uses the specified `filter_zerocount_target_col` columns to flag pgRNA constructs
#'  that have a raw count of 0 in any one of those columns/samples of interest.
#' Then, it looks at the specified columns for the final day/sample replicates
#' (`filter_replicates_target_col`) to see for pgRNAs that were flagged by the filter,
#' how many of those replicate samples had raw counts of zeros. And it produces a bar plot reporting on this.
#' Note, if you select samples/columns to check with the filter that don't have the
#'  replicate samples, this graph won't be informative. So you want there to be overlap
#'   between the columns for the two target_col parameters to have an informative graph
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function
#' which contains the transformed data
#' @param filter_zerocount_target_col default is NULL; Which sample column(s)
#' should be used to check for counts of 0? If NULL and not specified, downstream analysis will select all sample columns
#' @param filter_replicates_target_col default is NULL; Which sample columns are
#' replicates whose variation you'd like to analyze; If NULL, the last 3 sample columns are used
#' @param wide_ar aspect ratio, default is 0.75
#' @importFrom tidyr pivot_longer
#' @importFrom magrittr %>%
#' @import ggplot2
#' @import dplyr
#' @return a ggplot barplot
#' @export
#' @examples \dontrun{
#' gimap_dataset <- get_example_data("gimap")
#' qc_constructs_countzero_bar(gimap_dataset)
#'
#' # or if you want to select a specific column(s) for
#' # looking at where/which samples zero counts are present for
#' qc_constructs_countzero_bar(gimap_dataset, filter_zerocount_target_col = 3:5)
#'
#' # or if you want to select a specific column(s) for the final day/sample replicates
#' qc_constructs_countzero_bar(gimap_dataset, filter_replicates_target_col = 3:5)
#'
#' # or some combination of those
#' qc_constructs_countzero_bar(gimap_dataset,
#'   filter_zerocount_target_col = 3:5,
#'   filter_replicates_target_col = 3:5
#' )
#' }
qc_constructs_countzero_bar <- function(gimap_dataset,
                                        filter_zerocount_target_col = NULL,
                                        filter_replicates_target_col = NULL,
                                        wide_ar = 0.75) {
  if (is.null(filter_zerocount_target_col)) {
    filter_zerocount_target_col <- c(1:ncol(gimap_dataset$raw_counts))
  }

  if (!all(filter_zerocount_target_col %in% 1:ncol(gimap_dataset$raw_counts))) {
    stop("The columns selected do not exist. `filter_zerocount_target_col` needs to correspond to the index of the columns in `gimap_dataset$raw_counts` that you need to filter by")
  }

  qc_filter_output <- qc_filter_zerocounts(gimap_dataset, filter_zerocount_target_col = filter_zerocount_target_col)

  if (is.null(filter_replicates_target_col)) {
    filter_replicates_target_col <- c((ncol(gimap_dataset$transformed_data$log2_cpm) - 2):ncol(gimap_dataset$transformed_data$log2_cpm))
  } # last 3 columns of the data

  if (!all(filter_replicates_target_col %in% 1:ncol(gimap_dataset$transformed_data$log2_cpm))) {
    stop("The columns selected do not exist. `filter_replicates_target_col` needs
         to correspond to the index of the columns in `gimap_dataset$transformed_data$log2_cpm` that you need to filter by")
  }


  return(
    gimap_dataset$raw_counts[qc_filter_output$filter, filter_replicates_target_col] %>%
      as.data.frame() %>%
      mutate(row = row_number()) %>%
      tidyr::pivot_longer(colnames(gimap_dataset$raw_counts)[filter_replicates_target_col],
        values_to = "counts"
      ) %>%
      group_by(row) %>%
      summarize(numzero = sum(counts == 0)) %>%
      group_by(numzero) %>%
      summarize(count = n()) %>%
      ggplot(aes(x = numzero, y = count)) +
      geom_bar(stat = "identity") +
      theme_classic() +
      ylab("Number of pgRNAs") +
      xlab("Number of replicates with a zero") +
      geom_text(aes(label = count, group = numzero), vjust = -0.5, size = 2)
  )
}

#' Create a correlation heatmap for the pgRNA CPMs
#' @description This function uses the `cor` function to find correlations between the
#'  sample CPM's and then plots a heatmap of these
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function which contains the transformed data
#' @importFrom magrittr %>%
#' @importFrom pheatmap pheatmap
#' @importFrom stats cor
#' @return `sample_cor_heatmap` a pheatmap
#' @export
#' @examples \dontrun{
#' gimap_dataset <- get_example_data("gimap")
#' qc_cor_heatmap(gimap_dataset)
#' }
qc_cor_heatmap <- function(gimap_dataset) {
  cpm_cor <- gimap_dataset$transformed_data$cpm %>%
    cor() %>%
    round(2) %>%
    data.frame()

  sample_cor_heatmap <-
    pheatmap::pheatmap(cpm_cor,
      border_color = "white",
      cellwidth = 20,
      cellheight = 20,
      treeheight_row = 20,
      treeheight_col = 20
    )

  return(sample_cor_heatmap)
}

#' Create a histogram with plasmid log2 CPM values and ascertain a cutoff for low values
#' @description Find the distribution of plasmid (day0 data) pgRNA log2 CPM values, and ascertain a cutoff or filter for low log2 CPM values.
#' Assumes the first column of the dataset is the day0 data; do I need a better
#' method to tell, especially if there are reps?
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function which contains the transformed data
#' @param cutoff default is NULL, the cutoff for low log2 CPM values for the plasmid time period; if not specified, The lower outlier (defined by taking the difference of the lower quartile and 1.5 * interquartile range) is used
#' @param filter_plasmid_target_col default is NULL, and if NULL, will select the first column only; this parameter specifically should be used to specify the plasmid column(s) that will be selected
#' @param wide_ar aspect ratio, default is 0.75
#' @importFrom magrittr %>%
#' @importFrom janitor clean_names
#' @export
#' @import ggplot2
#' @return a ggplot histogram
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap")
#'
#' qc_plasmid_histogram(gimap_dataset)
#'
#' # or to specify a "cutoff" value that will be displayed as a dashed vertical line
#' qc_plasmid_histogram(gimap_dataset, cutoff = 1.75)
#'
#' # or to specify a different column (or set of columns) to select
#' qc_plasmid_histogram(gimap_dataset, filter_plasmid_target_col = 1:2)
#'
#' # or to specify a "cutoff" value that will be displayed as a dashed vertical
#' # line as well as to specify a different column (or set of columns) to select
#' qc_plasmid_histogram(gimap_dataset, cutoff = 2, filter_plasmid_target_col = 1:2)
#' }
qc_plasmid_histogram <- function(gimap_dataset,
                                 cutoff = NULL,
                                 filter_plasmid_target_col = NULL,
                                 wide_ar = 0.75) {
  if (is.null(filter_plasmid_target_col)) {
    filter_plasmid_target_col <- c(1)
  }

  if (!all(filter_plasmid_target_col %in% 1:ncol(gimap_dataset$transformed_data$log2_cpm))) {
    stop("The columns selected do not exist. `filter_plasmid_target_col` needs to correspond to the index of the columns in `gimap_dataset$transformed_data$log2_cpm` that you need to filter by")
  }

  to_plot <- data.frame(gimap_dataset$transformed_data$log2_cpm[, filter_plasmid_target_col]) %>%
    `colnames<-`(rep(c("plasmid_log2_cpm"), length(filter_plasmid_target_col))) %>%
    clean_names()

  if (length(filter_plasmid_target_col > 1)) { # if more than one column was selected, collapse all of the columns into the same vector and store in a df to plot
    to_plot <- data.frame(unlist(to_plot %>% select(starts_with("plasmid_log2_cpm")), use.names = FALSE)) %>% `colnames<-`(c("plasmid_log2_cpm"))
  }

  quantile_info <- quantile(to_plot$plasmid_log2_cpm)

  if (is.null(cutoff)) {
    cutoff <- quantile_info["25%"] - (1.5 * (quantile_info["75%"] - quantile_info["25%"]))
  }
  # if cutoff is null, suggest a cutoff and plot with suggested

  return(
    ggplot(to_plot, aes(x = plasmid_log2_cpm)) +
      geom_histogram(binwidth = 0.2, color = "black", fill = "gray60") +
      plot_options() +
      plot_theme() +
      theme(aspect.ratio = wide_ar) +
      geom_vline(xintercept = cutoff, linetype = "dashed")
  )
}


#' Standardized plot theme
#' @description this is a ggplot2 standardized plot theme for this package
#' @export
#' @return A ggplot2 theme that can be used on the plots.
plot_theme <- function() {
  theme(
    axis.text = element_text(colour = "black"),
    axis.ticks = element_line(color = "black")
  )
}

#' Calculate Genetic Interaction scores
#' @description Create results table that has CRISPR scores, Wilcoxon rank-sum
#' test and t tests.
#' The output of the `gimap` package is genetic interaction scores which _is the
#' distance between the observed CRISPR score and the expected CRISPR score._
#' The expected CRISPR scores are what we expect for the CRISPR values should
#' two genes be unrelated to each other. The further away an observed CRISPR
#' scoreis from its expected the more we suspect genetic interaction.
#' This can be true in a positive way (a CRISPR knockout pair caused more cell
#' proliferation than expected) or in a negative way (a CRISPR knockout pair
#' caused more cell lethality than expected).
#'
#' The genetic interaction scores are based on a linear model calculated for
#' each sample where `observed_crispr_single` is the outcome variable and
#' `expected_crispr_single` is the predictor variable.
#' For each sample: lm(observed_crispr_single ~ expected_crispr_single)
#'
#' Using `y = mx+b`, we can fill in the following values:
#' * `y` = observed CRISPR score
#' * `x` = expected CRISPR score
#' * `m` = slope from linear model for this sample
#' * `b` = intercept from linear model for this sample
#'
#' The intercept and slope from this linear model are used to adjust the CRISPR
#' scores for each sample:
#' single target gi score =
#'   observed single crispr - (intercept + slope * expected single crispr)
#' double_target_gi_score =
#'   double crispr score - (intercept + slope * expected double crispr)
#' These single and double target genetic interaction scores are calculated at
#' the construct level and are then summarized using a t-test to see if the the
#' distribution of the set of double targeting constructs is significantly
#' different than the overall distribution single targeting constructs.
#' After multiple testing correction, FDR values are reported.
#' Low FDR value for a double construct means high suspicion of paralogs.
#' @param .data Data can be piped in with tidyverse pipes from function to
#' function. But the data must still be a gimap_dataset
#' @param gimap_dataset A special dataset structure that is setup using the
#' `setup_data()` function.
#' @param use_lfc Should Log fold change be used to calculate GI scores instead
#' of CRISPR scores? If you do not have negative controls or CRISPR scores you
#' will need to set this to TRUE.
#' @param use_ntc If you do not have negative controls or do not want them to be
#' used set this to FALSE and then 0 will be used as the comparison value. Default
#' is that use_ntc = TRUE and looks for negative control genes.
#' @return A gimap dataset with statistics and genetic interaction scores
#' calculated. Overall results in the returned object can be obtained using
#' gimap_dataset$overall_results Whereas target level genetic interaction
#' scores can be retrieved using `gimap_dataset$gi_scores`.
#' @import dplyr
#' @importFrom stats lm
#' @export
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap",
#'   data_dir = tempdir()
#' ) %>%
#'   gimap_filter() %>%
#'   gimap_annotate(
#'     cell_line = "HELA",
#'     annot_dir = tempdir()
#'   ) %>%
#'   gimap_normalize(
#'     timepoints = "day",
#'     missing_ids_file = tempfile()
#'   ) %>%
#'   calc_gi()
#'
#' saveRDS(gimap_dataset, file.path(tempdir(), "gimap_dataset_final.RDS"))
#' }
calc_gi <- function(.data = NULL,
                    gimap_dataset,
                    use_lfc = FALSE,
                    use_ntc = TRUE) {
  # Summary the calculation
  # single_target_crispr_1 = geneA_nt1, geneA_nt2...
  # single_target_crispr_2 = nt1_geneB, nt2_geneB...
  # double_crispr_score = geneA_geneBpg1, geneA_geneBpg2...

  # mean_double_control_crispr = mean for the same control sequence
  # Unless use_ntc is set to FALSE

  # expected_crispr_double=single_target_crispr_1 + single_target_crispr_2
  # expected_crispr_single_1=single_target_crispr_1 + mean_double_control_crispr
  # expected_crispr_single_2=single_target_crispr_2 + mean_double_control_crispr

  # linear model is with all samples:
  # lm(mean_observed_single_crispr ~ mean_expected_single_crispr)

  # single_target_gi_score = mean_observed_single_crispr -
  #      (intercept + slope * mean_expected_single_crispr)
  # double_target_gi_score = mean_double_crispr_score -
  #      (intercept + slope * mean_expected_double_crispr)

  # Code adapted from
  # https://github.com/FredHutch/GI_mapping/blob/main/workflow/scripts/
  # 04-calculate_GI_scores.Rmd

  if (!is.null(.data)) gimap_dataset <- .data

  op <- options("R_MAX_VSIZE" = 32000000000)
  on.exit(options(op))

  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works",
      " with gimap_dataset objects which can be made with the",
      " setup_data() function."
    )
  }

  if (is.null(gimap_dataset$normalized_log_fc)) {
    stop(
      "This function only works",
      "with already normalized gimap_dataset objects",
      "which can be done with the gimap_normalize() function."
    )
  }
  lfc_adj <- gimap_dataset$normalized_log_fc

  if (!"crispr_score" %in% colnames(lfc_adj)) {
    if (!use_lfc) {
      stop("No CRISPR scores found, you can set use_lfc = TRUE to calculate
         genetic interaction scores using log fold change")
    }
  }
  if (use_lfc) {
    lfc_adj$crispr_score <- lfc_adj$lfc
    message("Calculating Genetic Interaction scores using LFC")
  }

  #### STEP 1 CALCULATE EXPECTED VALUES AND COLLAPSE TO MEANS
  # Get mean control target CRISPR scores -- they will be used for expected
  # calculations
  control_target_df <- lfc_adj %>%
    dplyr::filter(target_type == "ctrl_ctrl") %>%
    tidyr::pivot_longer(
      cols = c(gRNA1_seq, gRNA2_seq),
      names_to = "position",
      values_to = "control_gRNA_seq"
    ) %>%
    # If there's the same control sequence, and rep
    dplyr::group_by(pgRNA_target, control_gRNA_seq, norm_ctrl_flag) %>%
    # Then take the mean for when controls have the same sequence
    dplyr::summarize(
      mean_double_control_crispr =
        mean(crispr_score, na.rm = TRUE),
      .groups = "drop"
    ) %>%
    dplyr::select(
      pgRNA_target,
      control_gRNA_seq,
      mean_double_control_crispr,
      norm_ctrl_flag,
    )
  # This means we have a mean double control crispr for each control sequence

  # Add this check after creating control_target_df
  if (nrow(control_target_df) == 0) {
    message("No ctrl_ctrl controls found. Using mean_double_control_crispr = 0 for calculations.",
            "\n If ctrl_ctrl's are expected then re-evaluate your annotations.")
  }

  # Add this check after creating control_target_df
  if (!use_ntc) {
    message("use_ntc = FALSE so ctrl_ctrl values either do not exist or will be ignored.",
            "Using mean_double_control_crispr = 0 for calculations")
  }

  # Calculate expected and mean CRISPR scores for single targets
  single_crispr_df <- lfc_adj %>%
    dplyr::filter(target_type %in% c("ctrl_gene", "gene_ctrl")) %>%
    # We will be joining things based on the gRNA sequences so
    # we do some recoding here
    mutate(
      targeting_gRNA_seq = case_when(
        target_type == "gene_ctrl" ~ gRNA1_seq,
        target_type == "ctrl_gene" ~ gRNA2_seq
      ),
      control_gRNA_seq = case_when(
        target_type == "gene_ctrl" ~ gRNA2_seq,
        target_type == "ctrl_gene" ~ gRNA1_seq
      ),
      gene_symbol = dplyr::case_when(
        target_type == "gene_ctrl" ~ gene1_symbol,
        target_type == "ctrl_gene" ~ gene2_symbol
      ),
    ) %>%
    dplyr::left_join(control_target_df,
      by = c("control_gRNA_seq" = "control_gRNA_seq"),
      suffix = c("", "_control")
    ) %>%
    group_by(pgRNA_target, targeting_gRNA_seq) %>%
    # Taking the mean of the single target crisprs that have the same
    # targeting sequence
    summarize(
      mean_single_crispr = mean(crispr_score, na.rm = TRUE),
      # Note this is now the mean of all the control sequences for a particular target
      mean_double_control_crispr = mean(mean_double_control_crispr, na.rm = TRUE),
      .groups = "drop"
    ) %>%
    dplyr::select(
      pgRNA_target,
      targeting_gRNA_seq,
      mean_single_crispr,
      mean_double_control_crispr
    ) %>%
    ## calculate expected single-targeting GI score by summing the
    ## single-targeting and the mean_double_control_crispr
    dplyr::mutate(
      ## Replace NA values in mean_double control_crispr with 0 when there are no ctrl_ctrl controls
      mean_double_control_crispr = ifelse(is.na(mean_double_control_crispr), 0, mean_double_control_crispr),
      # OR if use_ntc == FALSE then it will still be set to 0
      mean_double_control_crispr = if_else(rep(use_ntc, times=n()), mean_double_control_crispr, 0),
      expected_single_crispr = mean_single_crispr + mean_double_control_crispr,
    )

  # This will be added on to the double crispr df
  expected_single_crispr_df <- single_crispr_df %>%
    dplyr::select(pgRNA_target, targeting_gRNA_seq, mean_single_crispr) %>%
    dplyr::distinct()

  # Now put it all together into one df
  double_crispr_df <- lfc_adj %>%
    dplyr::filter(target_type == "gene_gene") %>%
    dplyr::select(
      pg_ids,
      rep,
      crispr_score,
      gRNA1_seq,
      gRNA2_seq,
      pgRNA_target
    ) %>%
    dplyr::left_join(expected_single_crispr_df,
      by = c("gRNA1_seq" = "targeting_gRNA_seq"),
      suffix = c("", "_1")
    ) %>%
    dplyr::left_join(expected_single_crispr_df,
      by = c("gRNA2_seq" = "targeting_gRNA_seq"),
      suffix = c("", "_2")
    ) %>%
    dplyr::select(
      pg_ids,
      double_crispr = crispr_score,
      gRNA1_seq,
      gRNA2_seq,
      pgRNA_target,
      mean_single_crispr_1 = mean_single_crispr,
      mean_single_crispr_2
    ) %>%
    dplyr::mutate(
      expected_double_crispr = mean_single_crispr_1 + mean_single_crispr_2
    )

  #### STEP 2 LINEAR MODEL AND GI SCORE CALC

  message("Calculating Genetic Interaction scores")

  # Calculate the linear models from this
  # Previously we had a dplyr::group_by(rep) %>% here because we made linear models
  # by each rep but we found doing a single linear model made more sense for
  # expectation calculations
  gimap_dataset$linear_model <- lm(mean_single_crispr ~ expected_single_crispr,
    data = single_crispr_df
  )

  # Pull out the intercept and slope
  calc_slope <- gimap_dataset$linear_model$coefficients[["expected_single_crispr"]]
  calc_intercept <- gimap_dataset$linear_model$coefficients[["(Intercept)"]]

  # Do the linear model adjustments
  gi_calc_single <- single_crispr_df %>%
    # Previously GI scores were calculated on a per construct basis but then we
    # realized a mean_expected should be used because it is hypothetically closer
    # to the expectation for the target overall.
    dplyr::group_by(pgRNA_target) %>%
    dplyr::summarize(
      mean_expected_cs = mean(expected_single_crispr, na.rm = TRUE),
      mean_observed_cs = mean(mean_single_crispr, na.rm = TRUE)
    ) %>%
    dplyr::mutate(
      gi_score = mean_observed_cs -
        (calc_intercept + calc_slope * mean_expected_cs)
    )

  # Do the linear model adjustments but don't collapse double
  gi_calc_double <- double_crispr_df %>%
    dplyr::group_by(pg_ids, pgRNA_target) %>%
    dplyr::summarize(
      mean_expected_cs = mean(expected_double_crispr, na.rm = TRUE),
      mean_observed_cs = mean(double_crispr, na.rm = TRUE),
      .groups = "drop"
    ) %>%
    # Using the single target's linear model here
    dplyr::mutate(
      gi_score = mean_observed_cs -
        (calc_intercept + calc_slope * mean_expected_cs)
    )
  # TODO: Check with Alice.
  # We need at least more than one data point per target but currently have
  # All 16 constructs going in where each is a mean of the reps


  #### STEP 3: RUN THE TESTS
  target_results_df <- gimap_stats(
    gi_calc_single = gi_calc_single,
    gi_calc_double = gi_calc_double,
    use_ntc = use_ntc
  )

  #### STEP 4: FORMATTING DATA FOR EASY STORING
  gi_calc_double_clean <- gi_calc_double %>%
    # Collapse to just stats and don't care about pg_ids anymore
    dplyr::select(
      pgRNA_target,
      mean_expected_cs,
      mean_observed_cs,
      gi_score
    ) %>%
    dplyr::group_by(
      pgRNA_target
    ) %>%
    # Collapse to one mean per target now that we've tested
    dplyr::summarize(
      mean_observed_cs = mean(mean_observed_cs, na.rm = TRUE),
      mean_expected_cs = mean(mean_expected_cs, na.rm = TRUE),
      gi_score = mean(gi_score, na.rm = TRUE),
      target_type = "gene_gene"
    )

  gi_calc_single_clean <- gi_calc_single %>%
    dplyr::mutate(target_type = dplyr::case_when(
      grepl("^ctrl_*", pgRNA_target) ~ "ctrl_gene",
      grepl("*_ctrl$", pgRNA_target) ~ "gene_ctrl"
    )) %>%
    dplyr::select(
      target_type,
      pgRNA_target,
      mean_expected_cs,
      mean_observed_cs,
      gi_score
    ) %>%
    dplyr::distinct()

  all_gi_scores <- dplyr::bind_rows(gi_calc_double_clean, gi_calc_single_clean)

  # Add on test results
  all_gi_scores <- all_gi_scores %>%
    dplyr::left_join(target_results_df,
      by = c("pgRNA_target")
    )

  # Store the useful bits
  gimap_dataset$gi_scores <- all_gi_scores

  if (!use_lfc) {
    # Save at the target level but call them crispr scores
    gimap_dataset$crispr_score$single_crispr_score <- single_crispr_df
    gimap_dataset$crispr_score$double_crispr_score <- double_crispr_df
    gimap_dataset$crispr_score$neg_control_crispr <- control_target_df
  } else {
    # Save at the target level but they aren't crispr scores
    colnames(single_crispr_df) <- gsub("crispr", "lfc", colnames(single_crispr_df))
    colnames(double_crispr_df) <- gsub("crispr", "lfc", colnames(double_crispr_df))
    colnames(control_target_df) <- gsub("crispr", "lfc", colnames(control_target_df))

    gimap_dataset$lfc$single_lfc <- single_crispr_df
    gimap_dataset$lfc$double_lfc <- double_crispr_df
    gimap_dataset$lfc$neg_control_crispr <- control_target_df

    # The final results also aren't crispr scores
    gimap_dataset$gi_scores <- gimap_dataset$gi_scores %>%
      dplyr::rename(
        mean_expected_lfc = mean_expected_cs,
        mean_observed_lfc = mean_observed_cs
      )
  }

  return(gimap_dataset)
}


#' Do tests --an internal function used by calc_gi() function
#' @description Create results table that has t test p values
#' @param gi_calc_single a data.frame with adjusted single gi scores
#' @param gi_calc_double a data.frame with adjusted double gi scores
#' @param use_ntc Logical. If TRUE, uses a two-sample t-test comparing double
#' target GI scores against single target GI scores. If FALSE, uses a one-sample
#' t-test against mu=0 since single target GI scores have zero variance when
#' negative controls are not used.
#' @param replicate a name of a replicate to filter out from gi_calc_adj Optional
#' @importFrom stats p.adjust t.test wilcox.test
gimap_stats <- function(gi_calc_double, gi_calc_single, use_ntc = TRUE, replicate = NULL) {
  ## get a vector of GI scores for all single-targeting ("control") pgRNAs
  ## for each rep
  ## get double-targeting pgRNAs for this rep, do a t-test to compare the
  ## double-targeting GI scores for each paralog pair to the control vector
  ## adjust for multiple testing using the Benjamini-Hochberg method

  if (use_ntc) {
    # Two-sample t-test: compare double target GI scores against single target GI scores
    gi_scores <- gi_calc_double %>%
      group_by(pgRNA_target) %>%
      mutate(p_val = t.test(
        x = gi_calc_single$gi_score,
        y = gi_score, # all construct guides here
        paired = FALSE
      )$p.value)
  } else {
    # One-sample t-test: test if double target GI scores differ from 0
    # Used when use_ntc=FALSE because single target GI scores are all ~0
    gi_scores <- gi_calc_double %>%
      group_by(pgRNA_target) %>%
      mutate(p_val = t.test(gi_score, mu = 0)$p.value)
  }


  ## adjust for multiple testing using the Benjamini-Hochberg method
  d_p_val <- gi_scores %>%
    dplyr::select(pgRNA_target, p_val) %>%
    arrange(p_val) %>%
    distinct(p_val, .keep_all = TRUE)

  fdr_vals <- p.adjust(d_p_val$p_val, method = "BH")

  d_fdr <- tibble("fdr" = fdr_vals) %>%
    bind_cols(d_p_val) %>%
    dplyr::select(pgRNA_target, p_val, fdr)

  return(d_fdr)
}

utils::globalVariables(c(
  "id", "pg_ids", "plot_theme()", "negative_control", "positive_control", "mean_observed_cs", "timepoints", "value", "timepoint_avg", "target_type",
  "unexpressed_ctrl_flag", "median", "lfc_adj", "median", "gRNA1_seq", "gRNA2_seq",
  "control_gRNA_seq", "crispr_score", "pgRNA_target", "mean_double_control_crispr",
  "pgRNA_target", "targeting_gRNA_seq", "mean_single_crispr", "double_crispr_score",

  "single_crispr_score_1", "single_crispr_score_2", "pgRNA_target_double", "mean_single_crispr_1",
  "mean_single_crispr_2", "mean_double_control_crispr_2",
  "expected_crispr", "term", "estimate", "mean_expected_crispr", "intercept", "slope",
  "p_val_ttest", "p_val_wil", "fde_vals_ttest", "fdr_vals_wil", "double_gi_score",
  "single_gi_score_1", "single_gi_score_2", "gene", "DepMap_ID",
 "gene1_symbol", "gene2_symbol", "expressed_flag", "norm_ctrl_flag", "bool_vals",
  "filter_name", "counts", "numzero", "name", "value", "lfc_plasmid_vs_late", "lfc_adj",
  "double_gi_score", "count_normalized", "construct",
  "filterFlag", "plasmid_log2_cpm", "log2_cpm", "gene_symbol", "gene_symbol_1", "gene_symbol_2",
  "mean_double_control_crispr_1", "expected_crispr_double", "expected_crispr_single_1",
  "expected_crispr_single_2", "fdr_vals_ttest", "read_table", "stripped_cell_line_name",
  "comparison", ".", "col_names", "lfc_adj1", "t.test", "wilcox.test", "p.adjust",
  "cor", "quantile", "var", "browseURL", "single_crispr", "mean_single_crispr_2",
  "mean_single_crispr_1", "expected_single_crispr", "double_crispr", "double_gi_score",
  "fdr", "lfc", "mean_expected_cs", "mean_gi_score", "mean_single_crispr",
  "expected_double_crispr", "p_val", "single_gi_score", "Rank", "broad_target_type",
  "logfdr", "pointColor", "both", "mean_score", "gi_score", "Day05_RepA", "id",
  "depmap_id", "ModelID", "model_id"
))

#' Check if internet/URL is available
#' @description Internal helper function to check if an internet resource is
#' reachable before attempting to download.
#' @param url The URL to check availability for
#' @param timeout_seconds Timeout in seconds for the check
#' @return TRUE if the URL is reachable, FALSE otherwise
#' @keywords internal
check_internet_available <- function(url = "https://api.figshare.com",
                                     timeout_seconds = 5) {
  tryCatch(
    {
      response <- httr::HEAD(url, httr::timeout(timeout_seconds))
      httr::status_code(response) < 400
    },
    error = function(e) {
      FALSE
    }
  )
}

#' Build manual download instructions for Figshare files
#' @description Internal helper to build a warning message for manual downloads.
#' @param item Figshare item ID.
#' @param files Character vector of file names.
#' @return A formatted message string.
#' @keywords internal
figshare_manual_download_message <- function(item, files) {
  files_text <- paste0(files, collapse = ", ")
  paste0(
    "You can manually download these files from Figshare item ", item, ": ",
    files_text, ". ",
    "Use `gimap::get_figshare(file_name = ..., item = \"", item,
    "\", output_dir = <data_dir>)` or see README \"Manual data download (Figshare)\"."
  )
}

#' Resolve and create data directory
#' @description Internal helper to resolve a writable data directory for downloads.
#' @param data_dir Optional data directory. If NULL or empty, uses a user data dir.
#' @return A normalized, existing directory path.
#' @keywords internal
gimap_data_dir <- function(data_dir = NULL) {
  if (is.null(data_dir) || length(data_dir) == 0) {
    data_dir <- tools::R_user_dir("gimap", "data")
  } else if (is.na(data_dir) || !nzchar(data_dir)) {
    data_dir <- tools::R_user_dir("gimap", "data")
  }

  data_dir <- normalizePath(data_dir, winslash = "/", mustWork = FALSE)
  if (!dir.exists(data_dir)) {
    dir.create(data_dir, recursive = TRUE, showWarnings = FALSE)
  }
  return(data_dir)
}

#' Session cache for Figshare example files (under [tempdir()])
#' @description Not exported; avoids persisting example downloads outside the session
#' temp directory unless the caller passes an explicit `data_dir`.
#' @return Normalized path to an existing directory.
#' @keywords internal
.gimap_example_cache_dir <- function() {
  d <- file.path(tempdir(), "gimap-example-data")
  d <- normalizePath(d, winslash = "/", mustWork = FALSE)
  if (!dir.exists(d)) {
    dir.create(d, recursive = TRUE, showWarnings = FALSE)
  }
  normalizePath(d, winslash = "/", mustWork = TRUE)
}

#' Default directory for [get_example_data()] and related helpers
#' @keywords internal
resolve_example_data_dir <- function(data_dir) {
  if (is.null(data_dir) || length(data_dir) == 0L ||
    (length(data_dir) == 1L && (is.na(data_dir) || !nzchar(data_dir)))) {
    .gimap_example_cache_dir()
  } else {
    gimap_data_dir(data_dir)
  }
}

#' Basenames of all files managed by [get_example_data()]
#' @keywords internal
.gimap_example_filenames <- function() {
  c(
    "PP_pgPEN_HeLa_counts.txt",
    "counts_pgPEN_PC9_example.tsv",
    "pgRNA_ID_pgPEN_library_comp.csv",
    "gimap_dataset_timepoint.RDS",
    "gimap_dataset_treatment.RDS",
    "pgPEN_annotations.txt"
  )
}

#' Whether `dir` is allowed as a root for [delete_example_data()]
#' @keywords internal
.is_safe_example_data_delete_root <- function(dir) {
  dir <- normalizePath(dir, winslash = "/", mustWork = FALSE)
  legacy <- normalizePath(
    tools::R_user_dir("gimap", "data"),
    winslash = "/", mustWork = FALSE
  )
  td <- normalizePath(tempdir(), winslash = "/", mustWork = FALSE)
  cache <- normalizePath(
    file.path(tempdir(), "gimap-example-data"),
    winslash = "/", mustWork = FALSE
  )
  dir == legacy ||
    dir == cache ||
    startsWith(dir, paste0(td, .Platform$file.sep))
}


#' Returns example data for package
#' @description This function loads and returns example data for the package.
#' Which dataset is returned must be specified. Data will be downloaded from Figshare
#' the first time it is used.
#' @param which_data One of `"count"`, `"count_treatment"`, `"meta"`, `"gimap"`,
#' `"gimap_treatment"`, or `"annotation"`; specifies which example dataset is returned.
#' @param data_dir Where files are stored. If `NULL` (default), example files are
#' written under `file.path(tempdir(), "gimap-example-data")` for the current R
#' session (they are not kept after the session ends). Pass an explicit directory
#' if you want a persistent copy (for example to mirror files elsewhere).
#' @param refresh_data If `TRUE`, removes known example filenames from `data_dir`
#' (only under safe locations; see [delete_example_data()]) before re-downloading.
#' @export
#' @returns the respective example data either as a data frame or a specialized
#' gimap_dataset depending on what was requested.
#'
#' @examples \dontrun{
#'
#' counts_timepoint <- get_example_data("count")
#' counts_treatment <- get_example_data("count_treatment")
#' gimap_timepoint_dataset <- get_example_data("gimap")
#' gimap_treatment_dataset <- get_example_data("gimap_treatment")
#' metadata <- get_example_data("meta")
#' annotation <- get_example_data("annotation")
#' }
get_example_data <- function(which_data,
                             data_dir = NULL,
                             refresh_data = FALSE) {
  data_dir <- resolve_example_data_dir(data_dir)

  file_name <- switch(which_data,
    "count" = "PP_pgPEN_HeLa_counts.txt",
    "count_treatment" = "counts_pgPEN_PC9_example.tsv",
    "meta" = "pgRNA_ID_pgPEN_library_comp.csv",
    "gimap" = "gimap_dataset_timepoint.RDS",
    "gimap_treatment" = "gimap_dataset_treatment.RDS",
    "annotation" = "pgPEN_annotations.txt"
  )

  if (is.null(file_name)) {
    stop(
      "`which_data` must be one of: count, count_treatment, meta, gimap, ",
      "gimap_treatment, annotation",
      call. = FALSE
    )
  }

  # If data is to be refreshed delete old data
  if (isTRUE(refresh_data)) {
    delete_example_data(data_dir = data_dir)
  }

  file_path <- file.path(data_dir, file_name)

  if (!grepl("RDS$", file_name)) {
    if (!file.exists(file_path)) {
      download_result <- get_figshare(
        file_name = file_name,
        item = "28264271",
        output_dir = data_dir
      )
      # Handle case where download failed
      if (is.null(download_result)) {
        warning(
          "Could not download example data '", which_data, "'. ",
          "The file is not available locally and could not be fetched from Figshare. ",
          figshare_manual_download_message("28264271", file_name)
        )
        return(NULL)
      }
    }
  } else {
    # For RDS files, check if underlying data can be obtained
    result <- tryCatch(
      {
        save_example_timepoint_data(data_dir = data_dir)
        save_example_treatment_data(data_dir = data_dir)
        TRUE
      },
      error = function(e) {
        message(
          "Could not prepare example data '", which_data, "': ", e$message
        )
        return(FALSE)
      }
    )
    if (!isTRUE(result)) {
      return(NULL)
    }
  }

  # Check if file exists before trying to read
  if (!file.exists(file_path)) {
    warning(
      "Example data file not found: ", file_path, "\n",
      "The data could not be downloaded. Please check your internet connection.\n",
      figshare_manual_download_message("28264271", file_name)
    )
    return(NULL)
  }

  dataset <- switch(which_data,
    "count" = readr::read_tsv(file_path,
      show_col_types = FALSE
    ),
    "count_treatment" = readr::read_tsv(file_path,
      show_col_types = FALSE
    ),
    "meta" = readr::read_csv(file_path,
      skip = 1,
      show_col_types = FALSE
    ),
    "gimap" = readr::read_rds(file_path),
    "gimap_treatment" = readr::read_rds(file_path),
    "annotation" = readr::read_tsv(file_path, show_col_types = FALSE)
  )
  return(dataset)
}


#' Folder used for Figshare example data
#' @param data_dir Optional directory. If `NULL`, returns the session example
#' cache under [tempdir()] (same default as [get_example_data()]). Otherwise
#' resolves via [gimap_data_dir()].
#' @export
#' @return Normalized path to the folder where example files are stored.
example_data_folder <- function(data_dir = NULL) {
  resolve_example_data_dir(data_dir)
}

#' Set up example count data
#' @param data_dir Optional data directory override.
#' @export
#' @return Returns the file path to folder where the example data is stored
save_example_timepoint_data <- function(data_dir = NULL) {
  data_dir <- resolve_example_data_dir(data_dir)
  example_data <- get_example_data("count", data_dir = data_dir) %>%
    dplyr::select(!Day05_RepA)

  example_pg_metadata <- get_example_data("meta", data_dir = data_dir)

  example_counts <- example_data %>%
    dplyr::select(c("Day00_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC")) %>%
    as.matrix()

  example_pg_id <- example_data %>%
    dplyr::select("id")

  example_pg_metadata <- example_data %>%
    dplyr::select(c("id", "seq_1", "seq_2"))

  example_sample_metadata <- data.frame(
    col_names = c("Day00_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC"),
    day = as.numeric(c("0", "22", "22", "22")),
    rep = as.factor(c("RepA", "RepA", "RepB", "RepC"))
  )

  gimap_dataset <- setup_data(
    counts = example_counts,
    pg_ids = example_pg_id,
    sample_metadata = example_sample_metadata
  )

  saveRDS(gimap_dataset, file.path(data_dir, "gimap_dataset_timepoint.RDS"))
}

#' Set up example count data
#' @param data_dir Optional data directory override.
#' @export
#' @return Returns the file path to folder where the example data is stored
save_example_treatment_data <- function(data_dir = NULL) {
  data_dir <- resolve_example_data_dir(data_dir)
  example_data <- get_example_data("count_treatment", data_dir = data_dir)

  example_pg_metadata <- get_example_data("meta", data_dir = data_dir)

  example_counts <- example_data %>%
    select(c("pretreatment", "dmsoA", "dmsoB", "drug1A", "drug1B")) %>%
    as.matrix()

  example_pg_id <- example_data %>%
    dplyr::select("id")

  example_pg_metadata <- example_pg_metadata %>%
    dplyr::select(c("pgRNA_ID", "target1_sgRNA_seq", "target1_sgRNA_seq"))

  example_sample_metadata <- data.frame(
    col_names = c("pretreatment", "dmsoA", "dmsoB", "drug1A", "drug1B"),
    drug_treatment = as.factor(c("pretreatment", "dmso", "dmso", "drug", "drug"))
  )

  gimap_dataset <- setup_data(
    counts = example_counts,
    pg_ids = example_pg_id,
    sample_metadata = example_sample_metadata
  )

  saveRDS(gimap_dataset, file.path(data_dir, "gimap_dataset_treatment.RDS"))
}

plot_options <- function() {
  list(theme_bw(base_size = 12))
}

#' Default creds path
#' @param app_name What app set up are you looking for? Supported apps are 'google' 'calendly' and 'github'
encrypt_creds_path <- function(app_name) {
  list.files(
    pattern = paste0("figshare_encrypt.rds"),
    recursive = TRUE,
    system.file("extdata", package = "gimap"),
    full.names = TRUE
  )
}
#' Get file path to an key encryption RDS
key_encrypt_creds_path <- function() {
  list.files(
    pattern = "encrypt_pass.rds",
    recursive = TRUE,
    system.file("extdata", package = "gimap"),
    full.names = TRUE
  )
}
#' Handler function for GET requests from Figshare
#' @param item What is the item we are retrieving?
#' @param file_name Which item are we downloading?
#' @param output_dir Where should the file be saved?
#' @param return_list Should the list of files be returned instead of the file
#' @importFrom utils menu installed.packages
#' @import httr
#' @importFrom jsonlite fromJSON
#' @importFrom openssl aes_cbc_decrypt
#' @returns Downloads necessary annotation files from Figshare and reads them
#' in as data frames.
#' @export
#'
#' @examples \dontrun{
#'
#' get_figshare(
#'   return_list = TRUE,
#'   output_dir = tempdir()
#' )
#'
#' get_figshare(
#'   file_name = "Achilles_common_essentials.csv",
#'   output_dir = tempdir()
#' )
#' }
get_figshare <- function(file_name = NA,
                         item = "19700056",
                         output_dir = tempdir(),
                         return_list = FALSE) {
  if (is.null(output_dir)) output_dir <- system.file("extdata", package = "gimap")

  # Check if Figshare is available before attempting to connect
 if (!check_internet_available("https://api.figshare.com")) {
    message(
      "Cannot connect to Figshare. ",
      "Please check your internet connection and try again later."
    )
    return(NULL)
  }

  decrypted <- tryCatch(
    {
      openssl::aes_cbc_decrypt(
        readRDS(encrypt_creds_path()),
        key = readRDS(key_encrypt_creds_path())
      )
    },
    error = function(e) {
      message("Could not decrypt Figshare credentials: ", e$message)
      return(NULL)
    }
  )

  if (is.null(decrypted)) {
    return(NULL)
  }

  url <- file.path("https://api.figshare.com/v2/articles", item)

  # Figshare API get with error handling
  result <- tryCatch(
    {
      httr::GET(
        url,
        httr::progress(),
        httr::add_headers(
          Authorization = paste0("Bearer ", unserialize(decrypted)$client_secret)
        ),
        httr::accept_json()
      )
    },
    error = function(e) {
      message(
        "Failed to connect to Figshare API: ", e$message, "\n",
        "Please check your internet connection and try again later."
      )
      return(NULL)
    }
  )

  if (is.null(result)) {
    return(NULL)
  }

  if (httr::status_code(result) != 200) {
    message(
      "Figshare API returned an error (HTTP ", httr::status_code(result), ").\n",
      "The resource may be temporarily unavailable. Please try again later."
    )
    return(NULL)
  }

  # Process and return results
  result_content <- httr::content(result, "text",
    encoding = "UTF-8"
  )
  result_list <- jsonlite::fromJSON(result_content)

  if (return_list) {
    return(result_list$files)
  }
  file_id <- result_list$files %>%
    dplyr::filter(name == file_name) %>%
    dplyr::pull(id)

  message("Downloading: ", file_name)
  result <- tryCatch(
    {
      httr::GET(
        file.path("https://api.figshare.com/v2/file/download/", file_id),
        httr::progress(),
        httr::add_headers(
          Authorization = paste0("Bearer ", unserialize(decrypted)$client_secret)
        ),
        httr::accept_json()
      )
    },
    error = function(e) {
      message(
        "Failed to download file from Figshare: ", e$message, "\n",
        "Please check your internet connection and try again later."
      )
      return(NULL)
    }
  )

  if (is.null(result)) {
    return(NULL)
  }

  if (httr::status_code(result) != 200) {
    message(
      "Failed to download file from Figshare (HTTP ", httr::status_code(result), ").\n",
      "The resource may be temporarily unavailable. Please try again later."
    )
    return(NULL)
  }

  result_content <- httr::content(result, "text",
    encoding = "UTF-8"
  )

  writeLines(result_content, file.path(output_dir, file_name))

  return(file.path(output_dir, file_name))
}

#' Pipe operator
#'
#' See \code{magrittr::\link[magrittr:pipe]{\%>\%}} for details.
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
#' @param lhs A value or the magrittr placeholder.
#' @param rhs A function call using the magrittr semantics.
#' @return The result of calling `rhs(lhs)`.
NULL


#' Delete cached Figshare example files
#' @description Removes only the known example filenames produced by
#' [get_example_data()] under `data_dir`. Paths are never taken from [options()],
#' so unrelated files cannot be deleted by mistake. Deletion is only allowed when
#' `data_dir` is the session example cache ([example_data_folder()]), the legacy
#' per-user `tools::R_user_dir("gimap", "data")`, or any directory under
#' [tempdir()] (for example test directories).
#' @param data_dir Directory whose example files should be removed. The default
#' (`NULL`) uses the same session cache as [get_example_data()].
#' @export
#' @return Invisibly, the character vector of file paths that were removed (may
#' be length zero if nothing existed).
#' @examples
#' delete_example_data()
#'
delete_example_data <- function(data_dir = NULL) {
  data_dir <- resolve_example_data_dir(data_dir)
  data_dir <- normalizePath(data_dir, winslash = "/", mustWork = TRUE)

  if (!.is_safe_example_data_delete_root(data_dir)) {
    stop(
      "Refusing to delete: `data_dir` must be the gimap example cache ",
      "(see ?example_data_folder), a directory under tempdir(), or the ",
      "legacy per-user gimap data directory from tools::R_user_dir(\"gimap\", \"data\").",
      call. = FALSE
    )
  }

  files <- file.path(data_dir, .gimap_example_filenames())
  existed <- file.exists(files)
  to_remove <- files[existed]

  if (length(to_remove) == 0L) {
    message("No gimap example data files found under:\n  ", data_dir)
    return(invisible(character(0)))
  }

  unlink(to_remove)
  message(
    "Removed ", length(to_remove), " gimap example file(s) from:\n  ",
    data_dir, "\n",
    paste0("  - ", basename(to_remove), collapse = "\n")
  )
  invisible(to_remove)
}

# Example data for testing
example_counts <- matrix(1:20, nrow = 4, ncol = 5)
example_pg_ids <- data.frame(id = 1:4)
example_sample_metadata <- data.frame(id = 1:5, replicate = factor(c(1, 1, 2, 2, 3)), timepoint = factor(c("T0", "T0", "T1", "T1", "T2")))

# Test elements inside output list
test_that("setup_data() works correctly", {
  skip_if_figshare_unavailable()
  data_dir <- test_example_data_dir()

  result <- setup_data(
    counts = example_counts,
    pg_ids = example_pg_ids,
    sample_metadata = example_sample_metadata
  )

  expect_s3_class(result, "gimap_dataset")
  expect_equal(result$raw_counts, example_counts)
  expect_equal(result$metadata$pg_ids, example_pg_ids)
  expect_equal(result$metadata$sample_metadata, example_sample_metadata)
  expect_equal(result$counts_per_sample, apply(example_counts, 2, sum))


  counts_timepoint <- get_example_data("count", data_dir = data_dir)
  counts_treatment <- get_example_data("count_treatment", data_dir = data_dir)
  gimap_dataset <- get_example_data("gimap", data_dir = data_dir)
  gimap_dataset <- get_example_data("gimap_treatment", data_dir = data_dir)
  metadata <- get_example_data("meta", data_dir = data_dir)
  annotation <- get_example_data("annotation", data_dir = data_dir)
})

test_that("HTML file is created and content is correct", {
  skip_if_figshare_unavailable()
  testthat::skip_if_not(
    rmarkdown::pandoc_available(),
    "Pandoc is required to render the QC report (install pandoc or skip this test)"
  )
  data_dir <- test_example_data_dir()

  # Does the thing run?
  gimap_dataset <- get_example_data("gimap", data_dir = data_dir)

  html_file <- run_qc(gimap_dataset,
    output_file = tempfile(),
    open_results = FALSE,
    overwrite = TRUE
  )

  expect_true(file.exists(html_file))

  # Test the other dataset too
  gimap_dataset <- get_example_data("gimap_treatment", data_dir = data_dir)

  html_file <- run_qc(gimap_dataset,
    output_file = tempfile(),
    open_results = FALSE,
    overwrite = TRUE
  )

  expect_true(file.exists(html_file))
})

test_that("Figshare API download works", {
  skip_if_figshare_unavailable()
  data_dir <- test_example_data_dir()

  data_list <- get_figshare(return_list = TRUE)

  testthat::expect_type(data_list, "list")

  get_figshare(file_name = "Achilles_common_essentials.csv", output_dir = data_dir)

  testthat::expect_true(
    file.exists(
      file.path(data_dir, "Achilles_common_essentials.csv")
    )
  )
})

test_that("delete_example_data only removes known files under safe roots", {
  d <- test_example_data_dir()
  tf <- file.path(d, "PP_pgPEN_HeLa_counts.txt")
  writeLines("x", tf)
  other <- file.path(d, "user_important_file.csv")
  writeLines("keep", other)

  delete_example_data(data_dir = d)

  expect_false(file.exists(tf))
  expect_true(file.exists(other))
  unlink(other)
})

test_that("delete_example_data refuses unsafe directories", {
  expect_error(
    delete_example_data(data_dir = if (.Platform$OS.type == "unix") "/" else "C:/"),
    "Refusing to delete"
  )
})

test_that("get_example_data does not clobber unrelated options() named like datasets", {
  ops <- options()
  on.exit(options(ops), add = TRUE)
  options(count = "user-value-should-not-be-clobbered")
  skip_if_figshare_unavailable()

  get_example_data("count", data_dir = test_example_data_dir())
  expect_equal(getOption("count"), "user-value-should-not-be-clobbered")
})

test_that("Test normalization", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      timepoints = "day",
      missing_ids_file = tempfile()
    )

  # make sure the important columns are there
  testthat::expect_true(
    all(c("target_type", "lfc", "rep", "crispr_score", "unexpressed_ctrl_flag")
    %in% colnames(gimap_dataset$normalized_log_fc))
  )

  neg_controls <- gimap_dataset$normalized_log_fc %>%
    dplyr::filter(norm_ctrl_flag == "negative_control") %>%
    dplyr::group_by(rep) %>%
    dplyr::summarize(neg_ctrl_med = median(crispr_score)) %>%
    dplyr::pull(neg_ctrl_med)

  # We expect negative controls to be now equal to 0
  testthat::expect_equal(neg_controls[1:3], c(0, 0, 0))

  pos_controls <- gimap_dataset$normalized_log_fc %>%
    dplyr::filter(norm_ctrl_flag == "positive_control") %>%
    dplyr::group_by(rep) %>%
    dplyr::summarize(pos_ctrl_med = median(crispr_score)) %>%
    dplyr::pull(pos_ctrl_med)

  # We expect positive controls to be now equal to -1
  testthat::expect_equal(
    round(pos_controls),
    round(c(-1, -1, -1))
  )
})

test_that("Test normalization without expression cutoff", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  gimap_dataset_true <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      timepoints = "day",
      normalize_by_unexpressed = TRUE,
      missing_ids_file = tempfile()
    )

  gimap_dataset_false <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      timepoints = "day",
      normalize_by_unexpressed = FALSE,
      missing_ids_file = tempfile()
    )

  testthat::expect_true(
    all(gimap_dataset_true$normalized_log_fc$lfc[1:6] !=
      gimap_dataset_false$normalized_log_fc$lfc[1:6])
  )
})

#' A function to run filtering
#' @description This function applies filters to the gimap data. By default it
#' runs both the zero count (across all samples) and the low plasmid cpm
#' filters, but users can select a subset of these filters or even adjust the
#' behavior of each filter
#' @param .data Data can be piped in with tidyverse pipes from function to
#' function. But the data must still be a gimap_dataset
#' @param gimap_dataset A special dataset structure that is setup using the
#' `setup_data()` function.
#' @param filter_type Can be one of the following: `zero_count_only`,
#' `low_plasmid_cpm_only` or `both`.
#' Potentially in the future also `rep_variation`, `zero_in_last_time_point`
#'  or a vector that includes multiple of these filters.
#' @param filter_zerocount_target_col default is NULL; Which sample column(s)
#' should be used to check for counts of 0? If NULL and not specified,
#' downstream analysis will select all sample columns
#' @param filter_plasmid_target_col default is NULL, and if NULL, will select
#' the first column only; this parameter specifically should be used to specify
#' the plasmid column(s) that will be selected
#' @param filter_replicates_target_col default is NULL, Which sample columns
#' are the final time point replicates; If NULL, the last 3 sample columns are
#' used. This is only used by this function to save a list of which pgRNA IDs
#' have a zero count for all of these samples.
#' @param cutoff default is NULL, relates to the low_plasmid_cpm filter; the
#' cutoff for low log2 CPM values for the plasmid time period; if not specified,
#' The lower outlier (defined by taking the difference of the lower quartile and
#'  1.5 * interquartile range) is used
#' @param min_n_filters default is 1; this parameter defines at least
#' how many/the minimum number of independent filters have to flag a pgRNA
#' construct before the construct is filtered when using a combination of
#' filters You should decide on the appropriate filter based on the results of
#' your QC report.
#' @importFrom purrr reduce
#' @returns a filtered version of the gimap_dataset returned in the
#' $filtered_data section filter_step_run is a boolean reporting if the filter
#' step was run or not (since it's optional)
#' metadata_pg_ids is a subset the pgRNA IDs such that these are the ones that
#' remain in the dataset following completion of filtering
#' transformed_log2_cpm is a subset the log2_cpm data such that these are the
#' ones that remain in the dataset following completion of filtering
#' removed_pg_ids is a record of which pgRNAs are filtered out once filtering
#' is complete
#' all_reps_zerocount_ids is not actually filtered data necessarily.
#' Instead it's just a record of which pgRNAs have a zero count in all final
#' timepoint replicates
#' @export
#' @import dplyr
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap", data_dir = tempdir()) %>%
#'   gimap_filter()
#'
#' # To see filtered data
#' # gimap_dataset$filtered_data
#'
#' # If you want to only use a single filter or some subset,
#' # specify which using the filter_type parameter
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter(filter_type = "zero_count_only")
#' # or
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter(filter_type = "low_plasmid_cpm_only")
#'
#' # If you want to use multiple filters and more than one to flag a pgRNA
#' # construct before it's filtered out, use the `min_n_filters` argument
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter(
#'     filter_type = "both",
#'     min_n_filters = 2
#'   )
#'
#' # You can also specify which columns the filters will be applied to
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter(
#'     filter_type = "zero_count_only",
#'     filter_zerocount_target_col = c(1, 2)
#'   )
#' }
gimap_filter <- function(.data = NULL,
                         gimap_dataset,
                         filter_type = "both",
                         cutoff = NULL,
                         filter_zerocount_target_col = NULL,
                         filter_plasmid_target_col = NULL,
                         filter_replicates_target_col = NULL,
                         min_n_filters = 1) {
  if (!is.null(.data)) gimap_dataset <- .data

  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works with gimap_dataset objects ",
      "which can be made with the setup_data() function."
    )
  }

  # check filter type input to make sure that it is a supportable input
  if (!(filter_type %in%
    c("both", "zero_count_only", "low_plasmid_cpm_only"))) {
    stop(
      "Specification for `filter_type` not understood;",
      "Need to use 'both', 'zero_count_only', or 'low_plasmid_cpm_only'"
    )
  }

  filter_call <- switch(filter_type,
    "both" = "both",
    "zero_count_only" = "zc_filter",
    "low_plasmid_cpm_only" = "p_filter"
  )

  filter_df <-
    data.frame(
      zc_filter = qc_filter_zerocounts(
        gimap_dataset,
        filter_zerocount_target_col = filter_zerocount_target_col
      )$filter,
      p_filter = qc_filter_plasmid(
        gimap_dataset,
        cutoff = cutoff,
        filter_plasmid_target_col = filter_plasmid_target_col
      )$filter$plasmid_cpm_filter
    ) %>%
    dplyr::mutate(both = rowSums(.) >= min_n_filters)

  ## The final call is based on what they specify
  final_call <- filter_df %>%
    dplyr::pull(filter_call)

  # store some data/results from filtering
  ## adding a way to know if the filter step was run since it's optional
  gimap_dataset$filtered_data$filter_step_run <- TRUE

  ## subset the pgRNA IDs such that these are the ones that remain in the
  ## dataset following completion of filtering
  gimap_dataset$filtered_data$metadata_pg_ids <-
    gimap_dataset$metadata$pg_ids[!final_call, ]

  ## subset the log2_cpm data such that these are the ones that remain in the
  # dataset following completion of filtering
  gimap_dataset$filtered_data$transformed_log2_cpm <-
    gimap_dataset$transformed_data$log2_cpm[!final_call, ]

  ## save a list of which pgRNAs are filtered out once filtering is complete
  gimap_dataset$filtered_data$removed_pg_ids <- cbind(
    gimap_dataset$metadata$pg_ids[final_call, ],
    filter_df[final_call, ]
  )

  gimap_dataset$filtered_data$removed_pg_ids <-
    gimap_dataset$filtered_data$removed_pg_ids %>%
    # add the IDs as a column together with the TRUEs and FALSEs for each
    # filter, focusing only on pgRNAs which are in some way flagged for removal
    pivot_longer(c(-id, -both),
      # pivot longer so that IDs are repeated and filter names are listed in a
      # column and the last column (`boolVals`) are TRUEs and FALSEs
      names_to = "filter_name",
      values_to = "bool_vals"
    ) %>%
    # drop rows where boolVals is false, so this leaves only filters that
    # flagged a pgRNA for removal
    filter(bool_vals == TRUE) %>%
    # drop the boolVals column because don't need it anymore
    select(id, filter_name) %>%
    # group by the pgRNA constructs
    group_by(id) %>%
    # and make a column that comma separates the relevant filters
    summarize(relevantFilters = toString(filter_name))

  ## save a list of which pgRNAs have a zero count in all final timepoint
  ## replicates.
  # NOTE these are NOT necessarily filtered out
  if (is.null(filter_replicates_target_col)) {
    filter_replicates_target_col <-
      c((ncol(gimap_dataset$transformed_data$log2_cpm) - 2
      ):ncol(gimap_dataset$transformed_data$log2_cpm))
  } # last 3 columns of the data

  gimap_dataset$filtered_data$all_reps_zerocount_ids <-
    gimap_dataset$metadata$pg_ids[
      unlist(
        # grab the data from the final timepoint replicate columns
        gimap_dataset$raw_counts[, filter_replicates_target_col] %>%
          as.data.frame() %>%
          mutate(row = row_number()) %>% # add a row number column for reference
          # Use pivot longer to put the sample names of the samples in a `name`
          # column
          tidyr::pivot_longer(
            colnames(gimap_dataset$raw_counts)[filter_replicates_target_col],
            values_to = "counts"
          ) %>% # and the corresponding values in a `counts` column
          group_by(row) %>% # group by the row index/reference
          # summarize the number of replicates that have a count of 0 for each
          # row/pgRNA construct
          summarize(numzero = sum(counts == 0)) %>%
          # filter to include only pgRNA constructs that have a zero count for
          # all replicates
          filter(numzero == length(filter_replicates_target_col)) %>%
          select(row) # select the rows column so we can grab the corresponding
        # pgRNA IDs
      ) # unlist the tibble
      ,
    ] # subselect just the IDs and store

  return(gimap_dataset)
}

#' Filter out samples of zero counts
#' Create a filter for pgRNAs which have a raw count of 0 for any sample/time
#' # point
#' @description This function flags and reports which and how many pgRNAs have a
#' raw count of 0 for any sample/time point
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function
#' which contains the raw count data
#' @param filter_zerocount_target_col default is NULL; Which sample column(s)
#' should be used to check for counts of 0? If NULL and not specified,
#' downstream analysis will select all sample columns
#' @importFrom magrittr %>%
#' @importFrom dplyr mutate
#' @importFrom purrr reduce map
#' @return a named list with the filter `filter` specifying which pgRNA have a
#' count zero for at least one sample/time point and a report df `reportdf` for
#' the number and percent of pgRNA which have a count zero for at least one
#' sample/time point
#' @examples \dontrun{
#' gimap_dataset <- get_example_data("gimap", data_dir = tempdir())
#' qc_filter_zerocounts(gimap_dataset)
#'
#' # or to specify a different column (or set of columns to select)
#' qc_filter_zerocounts(gimap_dataset, filter_zerocount_target_col = 1:2)
#' }
#' @export
qc_filter_zerocounts <- function(gimap_dataset,
                                 filter_zerocount_target_col = NULL) {
  if (is.null(filter_zerocount_target_col)) {
    filter_zerocount_target_col <- c(1:ncol(gimap_dataset$raw_counts))
  }

  if (!all(filter_zerocount_target_col %in% 1:ncol(gimap_dataset$raw_counts))) {
    stop(
      "The columns selected do not exist. `filter_zerocount_target_col`
         needs to correspond to the index of the columns in",
      "`gimap_dataset$raw_counts` that you need to filter by"
    )
  }

  counts_filter <-
    data.frame(gimap_dataset$raw_counts[, filter_zerocount_target_col]) %>%
    map(~ .x %in% c(0)) %>%
    reduce(`|`)

  zerocount_df <- data.frame(
    "RawCount0" = c(FALSE, TRUE),
    n = c(sum(!counts_filter), sum(counts_filter))
  ) %>%
    mutate(percent = round(((n / sum(n)) * 100), 2))

  return(list(
    filter = counts_filter,
    reportdf = zerocount_df
  ))
}

#' Create a filter for pgRNAs which have a low log2 CPM value for the
#' plasmid/Day 0 sample/time point
#' @description This function flags and reports which and how many pgRNAs have
#' low log2 CPM values for the plasmid/Day 0 sample/time point. If more than one
#' column is specified as the plasmid sample,
#' we pool all the replicate samples to find the lower outlier and flag
#' constructs for which any plasmid replicate has a log2 CPM value below the
#' cutoff
#' @param gimap_dataset The special gimap_dataset from the `setup_data` function
#' which contains the log2 CPM transformed data
#' @param cutoff default is NULL, the cutoff for low log2 CPM values for the
#' plasmid time period; if not specified, The lower outlier (defined by taking
#' the difference of the lower quartile and 1.5 * interquartile range) is used
#' @param filter_plasmid_target_col default is NULL, and if NULL, will select
#' the first column only; this parameter specifically should be used to specify
#' the plasmid column(s) that will be selected
#' @importFrom magrittr %>%
#' @importFrom dplyr mutate across if_any
#' @importFrom tidyr pivot_wider pivot_longer
#' @importFrom janitor clean_names
#' @importFrom stats quantile
#' @export
#' @return a named list with the filter `filter` specifying which pgRNAs have
#' low plasmid log2 CPM (column of interest is `plasmid_cpm_filter`) and a
#' report df `reportdf` for the number and percent of pgRNA which have a low
#' plasmid log2 CPM
#' @examples \dontrun{
#' gimap_dataset <- get_example_data("gimap", data_dir = tempdir())
#'
#' qc_filter_plasmid(gimap_dataset)
#'
#' # or to specify a cutoff value to be used in the filter rather than the lower
#' # outlier default
#' qc_filter_plasmid(gimap_dataset, cutoff = 2)
#'
#' # or to specify a different column (or set of columns to select)
#' qc_filter_plasmid(gimap_dataset, filter_plasmid_target_col = 1:2)
#'
#' # or to specify a cutoff value that will be used in the filter rather than
#' # the lower outlier default as well as to specify a different column (or set
#' # of columns) to select
#' qc_filter_plasmid(gimap_dataset,
#'   cutoff = 1.75,
#'   filter_plasmid_target_col = 1:2
#' )
#' }
#'
qc_filter_plasmid <- function(gimap_dataset,
                              cutoff = NULL,
                              filter_plasmid_target_col = NULL) {
  if (is.null(filter_plasmid_target_col)) {
    filter_plasmid_target_col <- c(1)
  }

  if (!all(filter_plasmid_target_col %in%
    1:ncol(gimap_dataset$transformed_data$log2_cpm))) {
    stop(
      "The columns selected do not exist. `filter_plasmid_target_col` needs",
      "to correspond to the index of the columns in",
      "`gimap_dataset$transformed_data$log2_cpm` that you need to filter by"
    )
  }

  plasmid_data <-
    data.frame(
      gimap_dataset$transformed_data$log2_cpm[, filter_plasmid_target_col]
    ) %>%
    `colnames<-`(rep(
      c("plasmid_log2_cpm"),
      length(filter_plasmid_target_col)
    )) %>%
    clean_names()

  if (length(filter_plasmid_target_col > 1)) {
    # if more than one column was selected, collapse all of the columns into
    # the same vector using pivot_longer to store in a df with the name of the
    # rep and number for row/construct
    plasmid_data <- plasmid_data %>%
      mutate(construct = rownames(plasmid_data)) %>%
      pivot_longer(starts_with("plasmid_log2_cpm"),
        values_to = "plasmid_log2_cpm",
        names_to = "rep"
      )
  }

  if (is.null(cutoff)) {
    # if cutoff is null, use lower outlier
    quantile_info <- quantile(plasmid_data$plasmid_log2_cpm)

    cutoff <- quantile_info["25%"] -
      (1.5 * (quantile_info["75%"] - quantile_info["25%"]))
    # later step make a function for this in utils since
    # it's used more than once
  }

  if (length(filter_plasmid_target_col > 1)) {
    # if more than one column was selected, take collapsed/pooled data and
    # compare it to the cutoff
    # then pivot_wider so that the constructs are in the same row and we can use
    # if_any to report if any of the replicates were flagged by the cutoff
    # return just that summary column (reporting if any are TRUE) as the filter
    plasmid_data <- plasmid_data %>%
      mutate(filterFlag = plasmid_log2_cpm < cutoff) %>%
      pivot_wider(
        id_cols = construct, names_from = rep,
        values_from = filterFlag
      )
    plasmid_cpm_filter <- plasmid_data %>%
      mutate(plasmid_cpm_filter = if_any(
        .cols = starts_with("plasmid_log2_cpm")
      )) %>%
      select(plasmid_cpm_filter)
  } else {
    plasmid_cpm_filter <-
      as.data.frame(plasmid_data$plasmid_log2_cpm < cutoff) %>%
      `colnames<-`("plasmid_cpm_filter")
  }

  plasmid_filter_df <- data.frame(
    "Plasmid_log2cpmBelowCutoff" = c(FALSE, TRUE),
    n = c(sum(!plasmid_cpm_filter), sum(plasmid_cpm_filter))
  ) %>%
    mutate(percent = round(((n / sum(n)) * 100), 2))
  # later step make a function for this in utils since it's used more than once

  return(list(
    filter = plasmid_cpm_filter,
    reportdf = plasmid_filter_df
  ))
}

#' Run Quality Control Checks
#' @description This function takes a `gimap_dataset` and creates a QC report
#' @param gimap_dataset A special dataset structure that is setup using the
#' `setup_data()` function.
#' @param output_file Needs to be a string that ends with ".Rmd" What the
#' name of the output QC report file should be.
#' @param overwrite default is FALSE; whether to overwrite the QC Report file
#' @param filter_zerocount_target_col default is NULL; Which sample column(s)
#' should be
#'  used to check for counts of 0?  If NULL and not specified, downstream
#'  analysis will select all sample columns
#' @param filter_plasmid_target_col default is NULL; Which sample columns(s)
#' should be used to look at log2
#' CPM expression for plasmid pgRNA constructs? If NULL and not specified,
#' downstream analysis
#' will select the first sample column only
#' @param filter_replicates_target_col default is NULL; Which sample columns
#' are replicates whose variation you'd like to analyze; If NULL, the last
#' 3 sample columns are used
#' @param open_results default is TRUE but if you don't want the report
#' automatically opened, choose FALSE.
#' @param ... additional parameters are sent to `rmarkdown::render()`
#' @returns a QC report saved locally
#' @export
#' @importFrom utils browseURL
#' @importFrom tidyr pivot_longer
#' @importFrom magrittr %>%
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap")
#'
#' run_qc(
#'   gimap_dataset,
#'   output_file = paste0(tempfile(), "_QC_Report.Rmd")
#' )
#' }
run_qc <- function(gimap_dataset,
                   output_file,
                   overwrite = FALSE,
                   filter_zerocount_target_col = NULL,
                   filter_plasmid_target_col = NULL,
                   filter_replicates_target_col = NULL,
                   open_results = TRUE,
                   ...) {
  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop("This function only works with gimap_dataset objects
       which can be made with the setup_data() function.")
  }

  # Determine the template
  templateFile <- system.file("extdata/gimapQCTemplate.Rmd", package = "gimap")

  # Make sure that the directory exists!
  directory <- dirname(output_file)

  # If not, make the directory
  if (directory != ".") {
    if (!dir.exists(directory)) {
      dir.create(directory, showWarnings = TRUE, recursive = TRUE)
    }
  }

  # Now if the file exists,
  if (file.exists(templateFile)) {
    if (file.exists(output_file) & !overwrite) {
      stop("there is already an output .Rmd file", output_file,
        ". Please remove or rename this file,",
        "or choose another output_file name.",
        call. = FALSE
      )
    } else {
      file.copy(from = templateFile, to = output_file, overwrite = overwrite)
    }
  } else {
    stop("The Rmd template file ", templateFile,
      " does not exist -- did you move it from the package files?",
      call. = FALSE
    )
  }

  # Send the data to render it!
  rmarkdown::render(output_file,
    params = list(
      dataset = gimap_dataset,
      filter_zerocount_target_col = filter_zerocount_target_col,
      filter_plasmid_target_col = filter_plasmid_target_col,
      filter_replicates_target_col = filter_replicates_target_col
    ),
    ...
  )

  # Tell where the output is
  results_file <- gsub("\\.Rmd$", "\\.html", output_file)
  message("Results in: ", results_file)

  if (interactive()) {
    if (open_results) {
      if (results_file != "") browseURL(results_file)
    }
  }
  results_file
}

#' Normalize Log fold changes
#'
#' @description This calculates the log fold change for a gimap dataset based on
#' the annotation and metadata provided.
#' gimap takes in a counts matrix that represents the number of cells that have
#' each type of pgRNA this data needs some normalization before CRISPR scores
#' and Genetic Interaction scores can be calculated.
#'
#' There are three steps of normalization.
#' 1. `Calculate log2CPM` - First we account for different read depths across
#' samples and transforms data to log2 counts per million reads.
#' `log2((counts / total counts for sample)) * 1 million) + 1)`
#' 2. `Calculate log2 fold change` - This is done by subtracting the log2CPM for
#' the pre-treatment from each sample.  control is what is highlighted.
#' The pretreatment is the day 0 of CRISPR treatment, before CRISPR pgRNAs
#' have taken effect.
#' `log2FC = log2CPM for each sample - pretreament log2CPM`
#'
#' 3. `Normalize by negative and positive controls` - Calculate a negative
#' control median for each sample and a positive control median for each sample
#' and divide each log2FC by this value.
#' log2FC adjusted = log2FC /
#' (median negative control for a sample - median positive control for a sample)
#'
#' @param .data Data can be piped in with a tidyverse pipe from function to
#' function. But the data must still be a gimap_dataset
#' @param gimap_dataset A special dataset structure that is setup using the
#' `setup_data()` function.
#' @param normalize_by_unexpressed TRUE/FALSE crispr data should be normalized
#' so that the median of unexpressed controls is 0. For this to happen set this
#' to TRUE but you need to have added TPM data in the gimap_annotate step using
#' cell_line_annotation or custom_tpm. If `unexpressed_ctrl_flag` is not present
#' (for example DepMap download failed), this is coerced to `FALSE` with a message.
#' @param timepoints Specifies the column name of the metadata set up in
#' `$metadata$sample_metadata`
#' that has a factor that represents the timepoints.
#' Timepoints will be made into three categories:
#' plasmid for the earliest time point, early for all middle timepoints and
#' late for the latest timepoints.
#' The late timepoints will be the focus for the calculations. The column used
#' for timepoints must be numeric or at least ordinal.
#' @param treatments Specifies the column name of the metadata set up in
#' `$metadata$sample_metadata`
#' that has a factor that represents column that specifies the treatment applied
#' to each. The replicates will be kept collapsed to an average.
#' @param control_name A name that specifies the data either in the treatments
#' column that should be used as the control. This could be the Day 0 of
#' treatment or an untreated sample.
#' For timepoints testing it will be assumed that the mininmum timepoint
#' is the control.
#' @param adj_method Must be one of three methods as stated by a character string
#' "negative_control_adj" or "no_adjustment". Default is "negative_control_adj"
#' "negative_control_adj" where CRISPR scores will be used for the GI scores
#' "no_adjustment" is where LFC adjusted will be used for the GI scores
#' @param num_ids_wo_annot default is 20; the number of pgRNA IDs to display to
#' console if they don't have corresponding annotation data;
#' ff there are more IDs without annotation data than this number, the output
#' will be sent to a file rather than the console.
#' @param rm_ids_wo_annot default is TRUE; whether or not to filter out pgRNA
#' IDs from the input dataset that don't have corresponding annotation data
#' available
#' @param missing_ids_file If there are missing IDs and a file is saved, where
#' do you want this file to be saved? Provide a file path.
#' @param overwrite Should existing normalized_log_fc data in the gimap_dataset
#' be overwritten?
#' @return A gimap_dataset with normalized log FC as a data frame that can be
#' retrieve by using gimap_dataset$normalized_log_fc. This will contain
#' log2FC adjusted stored in a column named `log_adj` and the CRISPR scores
#' stored in a column named `crispr_score`.
#' genes in the set.
#' @import dplyr
#' @export
#' @examples \dontrun{
#'
#' gimap_dataset_org <- get_example_data("gimap") %>%
#'   gimap_filter() %>%
#'   gimap_annotate(cell_line = "HELA") %>%
#'   gimap_normalize(
#'     timepoints = "day",
#'     missing_ids_file = tempfile()
#'   )
#' }
gimap_normalize <- function(.data = NULL,
                            gimap_dataset,
                            normalize_by_unexpressed = TRUE,
                            timepoints = NULL,
                            treatments = NULL,
                            control_name = NULL,
                            adj_method = "negative_control_adj",
                            num_ids_wo_annot = 20,
                            rm_ids_wo_annot = TRUE,
                            missing_ids_file = "missing_ids_file.csv",
                            overwrite = TRUE) {
  # Code adapted from
  # https://github.com/FredHutch/GI_mapping/blob/main/workflow/
  # scripts/03-filter_and_calculate_LFC.Rmd

  if (!is.null(.data)) gimap_dataset <- .data

  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works with gimap_dataset objects which",
      "can be made with the setup_data() function."
    )
  }
  # Based on log fold change calculations and other handling will go
  # based on the code in:
  # https://github.com/FredHutch/GI_mapping/blob/main/workflow/scripts/
  # 03-filter_and_calculate_LFC.Rmd

  if (!adj_method %in% c("negative_control_adj", "no_negative_control", "no_adjustment")) {
    if (!is.null(gimap_dataset$normalized_log_fc) & !overwrite) {
      stop(
        "Normalization has already been preformed on this dataset.",
        "set overwrite = TRUE if you'd like the existing data to be overwritten."
      )
    }
  }

  if (overwrite) {
    gimap_dataset$normalized_log_fc <- NULL
  }

  if (gimap_dataset$filtered_data$filter_step_run) {
    dataset <- gimap_dataset$filtered_data$transformed_log2_cpm
    pg_ids <- gimap_dataset$filtered_data$metadata_pg_ids$id
  } else {
    dataset <- gimap_dataset$transformed_data$log2_cpm
    pg_ids <- gimap_dataset$metadata$pg_ids
  }

  if (is.null(timepoints) & is.null(treatments)) {
    stop("Either timepoints or treatments must be specified so comparisons can
         be made and stats calculated.")
  }

  ### IF WE HAVE TREATMENTS
  if (!is.null(treatments)) {
    if (!(treatments %in% colnames(gimap_dataset$metadata$sample_metadata))) {
      stop(
        "The column name specified for 'treatments' does not exist in",
        "gimap_dataset$metadata$sample_metadata"
      )
    }

    # Just extract what we are working with
    treatment_vector <- gimap_dataset$metadata$sample_metadata[[treatments]]

    # Check the control is here
    if (!(control_name %in% treatment_vector)) {
      stop(
        "The specified control with the name: '", control_name,
        "' does not exist in specified treatments column: '", treatments,
        "'"
      )
    }

    # What are the comparisons we are doing here?
    treatment_group_names <- setdiff(unique(treatment_vector), control_name)

    # Rename and recode the timepoints variable
    gimap_dataset$metadata$sample_metadata <-
      gimap_dataset$metadata$sample_metadata %>%
      dplyr::mutate(
        comparison = dplyr::case_when(
          treatment_vector == control_name ~ "control",
          TRUE ~ treatment_vector
        ),
        comparison = factor(comparison,
          levels = c("control", treatment_group_names)
        )
      )
  }

  ### IF WE HAVE TIMEPOINTS
  if (!is.null(timepoints)) {
    if (!(timepoints %in% colnames(gimap_dataset$metadata$sample_metadata))) {
      stop(
        "The column name specified for 'timepoints' does not exist in",
        "gimap_dataset$metadata$sample_metadata"
      )
    }

    # Rename and recode the timepoints variable
    gimap_dataset$metadata$sample_metadata <-
      gimap_dataset$metadata$sample_metadata %>%
      # Note that timepoints are extablished as three categories:
      # control, early, or late.
      dplyr::mutate(
        timepoints = !!sym(timepoints),
        comparison = dplyr::case_when(
          timepoints == min(timepoints) ~ "control",
          timepoints == max(timepoints) ~ "late",
          TRUE ~ "early"
        )
      ) %>%
      dplyr::mutate(comparison = factor(comparison,
        levels = c("control", "early", "late")
      ))

    # What are the comparisons we are doing here?
    treatment_group_names <- c("early", "late")
  }

  ### Stop if no annotations
  if (is.null(gimap_dataset$annotation)) {
    stop(
      "No annotations are stored in this gimap_dataset, annotations are needed",
      "so we know what genes should be used as controls.",
      "Please run gimap_annotate() function on your gimap_dataset and then",
      " retry this function."
    )
  }

  if (isTRUE(normalize_by_unexpressed) &&
    !("unexpressed_ctrl_flag" %in% colnames(gimap_dataset$annotation))) {
    message(
      "TPM-based unexpressed flags are not present (DepMap annotation may have been skipped). ",
      "Using normalize_by_unexpressed = FALSE."
    )
    normalize_by_unexpressed <- FALSE
  }

  message("Normalizing Log Fold Change")

  # Doing some reshaping to get one handy data frame
  lfc_df <- dataset %>%
    as.data.frame() %>%
    dplyr::mutate(pg_ids = pg_ids) %>%
    tidyr::pivot_longer(-pg_ids, values_to = "log2_cpm") %>%
    # Adding on metadata
    dplyr::left_join(
      gimap_dataset$metadata$sample_metadata %>%
        dplyr::select(col_names, comparison),
      by = c("name" = "col_names")
    ) %>%
    tidyr::pivot_wider(
      values_from = "log2_cpm",
      names_from = c(name, comparison)
    )

  ##### Checking for missing ids
  missing_ids <- data.frame(
    missing_ids = setdiff(lfc_df$pg_ids, gimap_dataset$annotation$pgRNA_id)
  )

  if ((nrow(missing_ids) > 0) & (nrow(missing_ids) < num_ids_wo_annot)) {
    message(
      "The following ", nrow(missing_ids),
      " IDs were not found in the annotation data: \n",
      paste0(missing_ids, collapse = ", ")
    )

    if (rm_ids_wo_annot) {
      lfc_df <- lfc_df %>%
        filter(!pg_ids %in% missing_ids$missing_ids)
      message(
        "The input data for the IDs which were not found in the annotation",
        "data has been filtered out and will not be included in the analysis",
        "output."
      )
    } else {
      message(
        "The input data for the IDs which were not found in the annotation",
        " data will be kept throughout the analysis, but any data from the",
        " annotation won't be available for them."
      )
    }
  } else {
    missing_ids_file <- file.path(missing_ids_file)
    readr::write_csv(missing_ids, missing_ids_file)
  }

  ############### Subtract the control column (so either day 0 or pretreatment)

  # This is collapsing multiple controls into on should that occur
  ctrl_mean <- lfc_df %>%
    dplyr::select(dplyr::ends_with("_control")) %>%
    apply(., 1., mean, na.rm = TRUE)

  comparison_df <- lfc_df %>%
    dplyr::mutate_at(
      dplyr::vars(!c(pg_ids, dplyr::ends_with("_control"))), ~ .x - ctrl_mean
    ) %>%
    dplyr::select(!dplyr::matches(pg_ids) & !dplyr::ends_with("_control")) %>%
    dplyr::left_join(gimap_dataset$annotation,
      by = c("pg_ids" = "pgRNA_id")
    ) %>%
    tidyr::pivot_longer(dplyr::ends_with(treatment_group_names),
      names_to = "rep",
      values_to = "lfc"
    )

  ########################### Perform adjustments #############################
  if (normalize_by_unexpressed) {
    stopifnot(
      "For normalize_by_unexpressed to be TRUE you need to have added TPM
      data in the annotation step using cell_line_annotation or custom_tpm" =
        "unexpressed_ctrl_flag" %in% colnames(comparison_df)
    )

    comparison_df <- comparison_df %>%
      dplyr::mutate(
        lfc = lfc - median(comparison_df$lfc[comparison_df$unexpressed_ctrl_flag], na.rm = TRUE)
      )
  }

  if (adj_method == "negative_control_adj") {
    medians_df <- comparison_df %>%
      dplyr::group_by(norm_ctrl_flag, rep) %>%
      dplyr::summarize(median = median(lfc, na.rm = TRUE)) %>%
      tidyr::pivot_wider(
        values_from = median,
        names_from = norm_ctrl_flag
      ) %>%
      dplyr::select(rep, negative_control, positive_control)

    # logFC adjusted = (log2FC - log2FC_negctls) / |log2FC_posctls|
    lfc_adj <- comparison_df %>%
      dplyr::left_join(medians_df, by = "rep") %>%
      dplyr::mutate(
        crispr_score = (lfc - negative_control) /
          (negative_control - positive_control)
      )
    # These should equal 0 and -1
    lfc_adj %>%
      dplyr::group_by(rep, norm_ctrl_flag) %>%
      dplyr::summarize(median_crispr = median(crispr_score)) %>%
      dplyr::filter(norm_ctrl_flag %in% c("negative_control", "positive_control"))
  } else {
    lfc_adj <- comparison_df
  }

  # Save this at the construct level
  gimap_dataset$normalized_log_fc <- lfc_adj

  return(gimap_dataset)
}

#' Expected vs Observed CRISPR Scatterplot
#' @description This plot is meant to be functionally equivalent to Fig S5K (for HeLa, equivalent of Fig 3a for PC9).
#' Scatter plot of target-level observed versus expected CRISPR scores in the screen.
#' The solid line is the linear regression line for the negative control (single KO) pgRNAs,
#' while dashed lines indicate the lower and upper quartile residuals.
#' @param gimap_dataset A special dataset structure that is originally setup using `setup_data()` and has had gi scores calculated with `calc_gi()`.
#' @param facet_rep Should the replicates be wrapped with facet_wrap()?
#' @import dplyr
#' @import ggplot2
#' @export
#' @return A ggplot2 scatterplot of the target level observed vs expected
#' CRISPR scores.
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter() %>%
#'   gimap_annotate(cell_line = "HELA") %>%
#'   gimap_normalize(
#'     timepoints = "day",
#'     missing_ids_file = tempfile()
#'   ) %>%
#'   calc_gi()
#'
#' # To plot results
#' plot_exp_v_obs_scatter(gimap_dataset)
#' plot_rank_scatter(gimap_dataset)
#' plot_volcano(gimap_dataset)
#' }
plot_exp_v_obs_scatter <- function(gimap_dataset, facet_rep = FALSE) {
  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works",
      "with gimap_dataset objects which can be made with the setup_data() function."
    )
  }

  if (is.null(gimap_dataset$gi_scores)) {
    stop(
      "This function only works with",
      "gimap_dataset objects which have had gi calculated with calc_gi()"
    )
  }

  expected_col <- ifelse(!"mean_expected_cs" %in% colnames(gimap_dataset$gi_scores),
    "mean_expected_lfc",
    "mean_expected_cs"
  )

  observed_col <- ifelse(!"mean_observed_cs" %in% colnames(gimap_dataset$gi_scores),
    "mean_observed_lfc",
    "mean_observed_cs"
  )

  gplot <- gimap_dataset$gi_scores %>%
    mutate(broad_target_type = case_when(
      target_type == "gene_gene" ~ "DKO",
      target_type == "ctrl_gene" ~ "control",
      target_type == "gene_ctrl" ~ "control"
    )) %>%
    ggplot(aes(
      x = !!sym(expected_col),
      y = !!sym(observed_col),
      color = broad_target_type
    )) +
    geom_point(size = 1, alpha = 0.7) +
    scale_color_manual(values = c(
      "control" = "gray50",
      "DKO" = "mediumpurple3"
    )) +
    theme_classic() +
    theme(legend.title = element_blank()) +
    geom_abline(
      slope = gimap_dataset$linear_model$coefficients[["expected_single_crispr"]],
      intercept = gimap_dataset$linear_model$coefficients[["(Intercept)"]]
    ) +
    geom_abline(
      slope = gimap_dataset$linear_model$coefficients[["expected_single_crispr"]],
      intercept = gimap_dataset$linear_model$coefficients[["(Intercept)"]] +
        quantile(gimap_dataset$linear_model$residuals)["75%"],
      linetype = 3
    ) +
    geom_abline(
      slope = gimap_dataset$linear_model$coefficients[["expected_single_crispr"]],
      intercept = gimap_dataset$linear_model$coefficients[["(Intercept)"]] +
        quantile(gimap_dataset$linear_model$residuals)["25%"],
      linetype = 3
    )

  if (expected_col == "mean_expected_cs") {
    gplot <- gplot +
      xlab("Expected CRISPR score\n(paralog 1 KO + paralog 2 KO)") +
      ylab("Observed CRISPR score\n(paralog 1 & 2 DKO)")
  } else if (expected_col == "mean_expected_lfc") {
    gplot <- gplot +
      xlab("Expected LFC score\n(paralog 1 KO + paralog 2 KO)") +
      ylab("Observed LFC score\n(paralog 1 & 2 DKO)")
  }

  return(gplot)
}
#' Rank plot for target-level GI scores
#' @description This plot is meant to be functionally equivalent to Fig 5a (for HeLa, equivalent of Fig 3c for PC9).
#' Rank plot of target-level GI scores.
#' Dashed horizontal lines are for GI scores of 0.25 and -0.5
#' @param gimap_dataset A special dataset structure that is originally setup using `setup_data()` and has had gi scores calculated with `calc_gi()`.
#' @param reps_to_drop Names of replicates that should be not plotted (Optional)
#' @import dplyr
#' @import ggplot2
#' @return A ggplot2 rankplot of the target level genetic interaction scores.
#' @export
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter() %>%
#'   gimap_annotate(cell_line = "HELA") %>%
#'   gimap_normalize(
#'     timepoints = "day"
#'   ) %>%
#'   calc_gi()
#'
#' # To plot results
#' plot_exp_v_obs_scatter(gimap_dataset)
#' plot_rank_scatter(gimap_dataset)
#' plot_volcano(gimap_dataset)
#' }
plot_rank_scatter <- function(gimap_dataset, reps_to_drop = "") {
  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works",
      "with gimap_dataset objects which can be made with the setup_data() function."
    )
  }

  if (is.null(gimap_dataset$gi_scores)) {
    stop(
      "This function only works with",
      "gimap_dataset objects which have had gi calculated with calc_gi()"
    )
  }

  gplot <- gimap_dataset$gi_scores %>%
    dplyr::ungroup() %>%
    filter(target_type == "gene_gene") %>%
    mutate(Rank = percent_rank(gi_score)) %>%
    ggplot(aes(
      x = Rank,
      y = gi_score
    )) +
    geom_point(size = 1, alpha = 0.7) +
    theme_classic() +
    theme(legend.title = element_blank()) +
    ylab("GI score") +
    geom_hline(yintercept = 0) +
    geom_hline(yintercept = -0.5, linetype = "dashed") +
    geom_hline(yintercept = 0.25, linetype = "dashed")

  return(gplot)
}

#' Volcano plot for GI scores
#' @description This plot is meant to be functionally equivalent to Fig 5b (for HeLa, equivalent of Fig 3d for PC9).
#' Volcano plot of target-level GI scores
#' Blue points are synthetic lethal paralog GIs with GI < 0.5 and FDR < 0.1; red points are buffering paralog GIs with GI > 0.25 and FDR < 0.1.
#' @param gimap_dataset A special dataset structure that is originally setup using `setup_data()` and has had gi scores calculated with `calc_gi()`.
#' @param facet_rep Should the replicates be wrapped with facet_wrap()?
#' @import dplyr
#' @import ggplot2
#' @return A ggplot2 volcano plot of the target level genetic interaction scores.
#' @export
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter() %>%
#'   gimap_annotate(cell_line = "HELA") %>%
#'   gimap_normalize(
#'     timepoints = "day"
#'   ) %>%
#'   calc_gi()
#'
#' # To plot results
#' plot_exp_v_obs_scatter(gimap_dataset)
#' plot_rank_scatter(gimap_dataset)
#' plot_volcano(gimap_dataset)
#' }
plot_volcano <- function(gimap_dataset, facet_rep = FALSE) {
  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works",
      "with gimap_dataset objects which can be made with the setup_data() function."
    )
  }

  if (is.null(gimap_dataset$gi_scores)) {
    stop(
      "This function only works with",
      "gimap_dataset objects which have had gi calculated with calc_gi()"
    )
  }

  gplot <- gimap_dataset$gi_scores %>%
    filter(target_type == "gene_gene") %>% # get only double targeting
    mutate(
      logfdr = -log10(fdr),
      pointColor = case_when(logfdr < 1 ~ "darkgrey",
        ((gi_score < -0.5) & (logfdr > 1)) ~ "dodgerblue3",
        ((gi_score > 0.25) & (logfdr > 1)) ~ "darkred",
        .default = "black"
      )
    ) %>%
    ggplot(aes(
      x = gi_score,
      y = logfdr,
      color = pointColor
    )) +
    geom_point(size = 1, alpha = 0.7) +
    theme_classic() +
    geom_hline(yintercept = 1, linetype = "dashed") +
    geom_vline(xintercept = -0.5, linetype = "dashed") +
    geom_vline(xintercept = 0.25, linetype = "dashed") +
    theme(legend.position = "none") +
    scale_color_manual(values = c(
      "darkgrey" = "darkgrey",
      "dodgerblue3" = "dodgerblue3",
      "darkred" = "darkred",
      "black" = "black"
    )) +
    ylab("-log10(FDR)") +
    xlab("Mean GI score")

  return(gplot)
}
#' Target bar plot for CRISPR scores
#' @description This plot is for when you'd like to examine a target pair
#' specifically -- meant to be functionally equivalent to Fig 3b
#' CRISPR scores for representative synthetic lethal paralog pairs.
#' Data shown are the mean CRISPR score for each single KO or DKO target across
#'  three biological replicates with replicate data shown in overlaid points.
#' @param gimap_dataset A special dataset structure that is originally setup using
#'  `setup_data()` and has had gi scores calculated with `calc_gi()`.
#' @param target1 Name of the first target to be plotted e.g.
#' @param target2 Name of the second target to be plotted e.g.
#' @param reps_to_drop Names of replicates that should be not plotted (Optional)
#' @import dplyr
#' @import ggplot2
#' @export
#' @return A ggplot2 bar plot of the specific target's genetic interaction scores.
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter() %>%
#'   gimap_annotate(cell_line = "HELA") %>%
#'   gimap_normalize(
#'     timepoints = "day"
#'   ) %>%
#'   calc_gi()
#'
#' # To plot results, pick out two targets from the gi_score table
#' head(dplyr::arrange(gimap_dataset$gi_score, fdr))
#'
#' # "TIAL1_TIA1" is top result so let's plot that
#' plot_targets(gimap_dataset, target1 = "TIAL1", target2 = "TIA1")
#' }
plot_targets <- function(gimap_dataset, target1, target2, reps_to_drop = "") {
  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works",
      "with gimap_dataset objects which can be made with the setup_data() function."
    )
  }

  if (is.null(gimap_dataset$normalized_log_fc)) {
    stop(
      "This function only works with",
      "gimap_dataset objects which have had CRISPR scores calculated with gimap_normalize()"
    )
  }

  expected_col <- ifelse("crispr_score" %in% colnames(gimap_dataset$normalized_log_fc),
    "crispr_score",
    "lfc"
  )

  gplot_data <- gimap_dataset$normalized_log_fc %>%
    dplyr::filter(gene1_symbol == target1 | gene2_symbol == target2) %>%
    filter(!(rep %in% reps_to_drop))


  if (nrow(gplot_data) == 0) {
    stop("No targets were found with those gene names")
  }

  gplot <- gplot_data %>%
    ggplot(aes(
      y = !!sym(expected_col),
      x = target_type
    )) +
    geom_boxplot(aes(fill = target_type), outlier.shape = NA) +
    scale_x_discrete(labels = c(
      "ctrl_gene" = paste0(target2, " KO"),
      # this assumes that target2 is the ctrl_{target2} gene
      "gene_ctrl" = paste0(target1, " KO"),
      # this assumes that target1 is the {target1}_ctrl gene
      "gene_gene" = "DKO"
    )) +
    geom_jitter(aes(x = target_type, y = crispr_score, color = pg_ids),
      pch = 21, size = 1, width = .2
    ) +
    theme_bw() +
    xlab("") +
    ggtitle(paste0(target1, "/", target2)) +
    geom_hline(yintercept = 0) +
    theme(
      legend.position = "none",
      panel.background = element_blank(),
      panel.grid = element_blank(),
      axis.text.x = element_text(angle = 45, hjust = 1)
    )

  if (expected_col == "crispr_score") {
    gplot <- gplot +
      ylab("CRISPR score")
  } else if (expected_col == "lfc") {
    gplot <- gplot +
      ylab("Adjusted Log Fold Change")
  }

  return(gplot)
}

test_that("Test Genetic Interaction score calculations", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      timepoints = "day",
      missing_ids_file = tempfile()
    ) %>%
    calc_gi()

  # Test that all expected data structures exist and are correct type
  testthat::expect_s3_class(gimap_dataset$linear_model, "lm")
  testthat::expect_true(is.data.frame(gimap_dataset$gi_scores))
  testthat::expect_true(nrow(gimap_dataset$gi_scores) > 0)

  # Test that required columns exist
  expected_cols <- c("mean_expected_cs", "mean_observed_cs", "gi_score", "p_val", "fdr", "pgRNA_target", "target_type")
  testthat::expect_true(all(expected_cols %in% colnames(gimap_dataset$gi_scores)))

  # Test that values are reasonable (not NA, not infinite)
  testthat::expect_false(any(is.na(gimap_dataset$gi_scores$mean_expected_cs)))
  testthat::expect_false(any(is.na(gimap_dataset$gi_scores$mean_observed_cs)))
  testthat::expect_false(any(is.na(gimap_dataset$gi_scores$gi_score)))
  testthat::expect_false(any(is.infinite(gimap_dataset$gi_scores$gi_score)))

  # Test that we have the expected target types
  expected_target_types <- c("gene_gene", "gene_ctrl", "ctrl_gene")
  testthat::expect_true(all(gimap_dataset$gi_scores$target_type %in% expected_target_types))

  # Test that linear model has reasonable coefficients (not NA or infinite)
  testthat::expect_false(any(is.na(gimap_dataset$linear_model$coefficients)))
  testthat::expect_false(any(is.infinite(gimap_dataset$linear_model$coefficients)))

  # Test that we have both single and double targeting results
  testthat::expect_true("gene_gene" %in% gimap_dataset$gi_scores$target_type)
  testthat::expect_true(any(c("gene_ctrl", "ctrl_gene") %in% gimap_dataset$gi_scores$target_type))
})


test_that("Test Genetic Interaction score calculations using LFC", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      timepoints = "day",
      adj_method = "no_adjustment",
      missing_ids_file = tempfile()
    ) %>%
    calc_gi(use_lfc = TRUE)

  testthat::expect_true(class(gimap_dataset)[1] == "list")
})


test_that("Test Genetic Interaction score without normalization", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  gimap_dataset_wo <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      normalize_by_unexpressed = FALSE,
      timepoints = "day",
      missing_ids_file = tempfile()
    ) %>%
    calc_gi()

  gimap_dataset_w <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      normalize_by_unexpressed = TRUE,
      timepoints = "day",
      missing_ids_file = tempfile()
    ) %>%
    calc_gi()


  # Are the GI scores the same? No
  testthat::expect_false(
    all(gimap_dataset_wo$gi_scores$gi_score == gimap_dataset_w$gi_scores$gi_score)
  )

  # Are the log fold change values the same? No
  testthat::expect_false(
    all(gimap_dataset_wo$normalized_log_fc$lfc == gimap_dataset_w$normalized_log_fc$lfc)
  )

  # Are the CRISPR scores the same? No
  testthat::expect_false(
    all(gimap_dataset_wo$normalized_log_fc$crispr_score == gimap_dataset_w$normalized_log_fc$crispr_score)
  )

  table(
    gimap_dataset_w$normalized_log_fc$unexpressed_ctrl_flag,
    gimap_dataset_w$normalized_log_fc$norm_ctrl_flag
  )
})

test_that("Test Genetic Interaction score calculations without ctrl_ctrl controls", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  # Get the example data and modify it to remove ctrl_ctrl controls
  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA")

  # Remove ctrl_ctrl controls from the annotation to simulate Ito et al. dataset
  gimap_dataset$annotation <- gimap_dataset$annotation %>%
    dplyr::filter(target_type != "ctrl_ctrl")

  # This should work without error when using no_adjustment and use_lfc = TRUE
  gimap_dataset_no_ctrl <- gimap_dataset %>%
    gimap_normalize(
      timepoints = "day",
      adj_method = "no_adjustment",
      missing_ids_file = tempfile()
    ) %>%
    calc_gi(use_lfc = TRUE)

  # Check that the function completed successfully
  testthat::expect_true(class(gimap_dataset_no_ctrl)[1] == "list")
  testthat::expect_false(is.null(gimap_dataset_no_ctrl$gi_scores))
  testthat::expect_false(is.null(gimap_dataset_no_ctrl$linear_model))

  # Check that we have GI scores calculated
  testthat::expect_true(nrow(gimap_dataset_no_ctrl$gi_scores) > 0)

  # Check that the linear model was created successfully
  testthat::expect_true(class(gimap_dataset_no_ctrl$linear_model)[1] == "lm")

  # Check that mean_expected_lfc values are not all NA
  testthat::expect_false(all(is.na(gimap_dataset_no_ctrl$gi_scores$mean_expected_lfc)))

  # Check that gi_score values are not all NA
  testthat::expect_false(all(is.na(gimap_dataset_no_ctrl$gi_scores$gi_score)))

  # Verify that no ctrl_ctrl controls exist in the final results
  testthat::expect_false("ctrl_ctrl" %in% gimap_dataset_no_ctrl$gi_scores$target_type)

  # Check that the LFC data structure exists (since use_lfc = TRUE)
  testthat::expect_false(is.null(gimap_dataset_no_ctrl$lfc))
  testthat::expect_true("single_lfc" %in% names(gimap_dataset_no_ctrl$lfc))
  testthat::expect_true("double_lfc" %in% names(gimap_dataset_no_ctrl$lfc))

  # This should work without error when using use_ntc = TRUE
  gimap_dataset_no_ntc <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA") %>%
    gimap_normalize(
      normalize_by_unexpressed = FALSE,
      timepoints = "day",
      missing_ids_file = tempfile()
    ) %>%
    calc_gi(use_ntc = FALSE)

  plot_exp_v_obs_scatter(gimap_dataset_no_ntc)
  plot_exp_v_obs_scatter(gimap_dataset_no_ctrl)

  plot_rank_scatter(gimap_dataset_no_ntc)
  plot_rank_scatter(gimap_dataset_no_ctrl)

  plot_volcano(gimap_dataset_no_ntc)
  plot_volcano(gimap_dataset_no_ctrl)

  # Check that gi_score values are not all NA
  testthat::expect_false(all(is.na(gimap_dataset_no_ctrl$gi_scores$gi_score)))

  # Verify that no ctrl_ctrl controls exist in the final results
  testthat::expect_false("ctrl_ctrl" %in% gimap_dataset_no_ctrl$gi_scores$target_type)
})

test_that("Test proper error messages for incompatible settings", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  # Get the example data and modify it to remove ctrl_ctrl controls
  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA")

  # Remove ctrl_ctrl controls from the annotation
  gimap_dataset$annotation <- gimap_dataset$annotation %>%
    dplyr::filter(target_type != "ctrl_ctrl")

  # This should fail during normalization when trying to use negative_control_adj without ctrl_ctrl
  # This is expected behavior - user should use "no_adjustment" when no ctrl_ctrl available
  testthat::expect_error(
    gimap_dataset %>%
      gimap_normalize(
        timepoints = "day",
        adj_method = "negative_control_adj",
        missing_ids_file = tempfile()
      ),
    "negative_control.*doesn't exist"
  )

  # This should fail when trying to use CRISPR scores without proper normalization
  # User should set use_lfc = TRUE when using no_adjustment
  testthat::expect_error(
    gimap_dataset %>%
      gimap_normalize(
        timepoints = "day",
        adj_method = "no_adjustment",
        missing_ids_file = tempfile()
      ) %>%
      calc_gi(use_lfc = FALSE),
    "No CRISPR scores found"
  )
})

#' Plot CRISPR scores after normalization
#' @description This plots normalization after CRISPR scores have been calculated
#'
#' @param .data Data can be piped in with tidyverse pipes from function to
#' function. But the data must still be a gimap_dataset
#' @param gimap_dataset A special dataset structure that is setup using the
#' `setup_data()` function.
#' @param output_file A file for the output
#' @export
#' @return A ggplot2 boxplot of the CRISPR scores separated by the type of
#' target. Can be used to determine the normalization has proceeded properly.
#'
#' @import ggplot2
#'
#' @examples \dontrun{
#'
#' gimap_dataset <- get_example_data("gimap") %>%
#'   gimap_filter() %>%
#'   gimap_annotate(cell_line = "HELA") %>%
#'   gimap_normalize(
#'     timepoints = "day",
#'     missing_ids_file = tempfile()
#'   )
#'
#' # Plot:
#' plot_crispr(gimap_dataset)
#' }
plot_crispr <- function(.data = NULL, gimap_dataset, output_file = "crispr_norm_plot.png") {
  if (!is.null(.data)) gimap_dataset <- .data

  if (!("gimap_dataset" %in% class(gimap_dataset))) stop("This function only works with gimap_dataset objects which can be made with the setup_data() function.")

  if (is.null(gimap_dataset$normalized_log_fc)) {
    stop("No normalized data found in this gimap_dataset. Make sure you have run the gimap_normalize() function")
  }

  if (!is.null(gimap_dataset$normalized_log_fc)) {
    source_data <- gimap_dataset$normalized_log_fc
  }

  output_plot <- gimap_dataset$normalized_log_fc %>%
    ggplot2::ggplot(ggplot2::aes(x = norm_ctrl_flag, y = crispr_score, fill = norm_ctrl_flag)) +
    ggplot2::geom_violin() +
    ggplot2::geom_boxplot(outliers = FALSE) +
    ggplot2::theme(axis.text.x = element_text(angle = 90)) +
    ggplot2::facet_wrap(~rep)

  return(output_plot)
}

#' Making a new gimap dataset
#' @description This function allows people to have their data ready to be
#'  processed by gimap
#' @param counts a matrix of data that contains the counts where rows are each
#' paired_guide target and columns are each sample
#' @param pg_ids the pgRNA IDs: metadata associated with the pgRNA constructs
#' that correspond to the rows of the counts data
#' @param sample_metadata metadata associated with the samples of the dataset
#' that correspond to the columns of the counts data.
#' Should include a column that has replicate information as well as a column
#' that contains timepoint information respectively (this will be used for log
#'  fold calculations). These columns should be factors.
#' @return A special gimap_dataset to be used with the other functions in this
#' package.
#' @export
#' @examples \dontrun{
#'
#' counts <- get_example_data("count", data_dir = tempdir()) %>%
#'   dplyr::select(c(
#'     "Day00_RepA", "Day05_RepA", "Day22_RepA", "Day22_RepB",
#'     "Day22_RepC"
#'   )) %>%
#'   as.matrix()
#'
#' pg_ids <- get_example_data("count", data_dir = tempdir()) %>%
#'   dplyr::select("id")
#'
#' sample_metadata <- data.frame(
#'   col_names = c("Day00_RepA", "Day05_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC"),
#'   day = as.numeric(c("0", "5", "22", "22", "22")),
#'   rep = as.factor(c("RepA", "RepA", "RepA", "RepB", "RepC"))
#' )
#'
#' gimap_dataset <- setup_data(
#'   counts = counts,
#'   pg_ids = pg_ids,
#'   sample_metadata = sample_metadata
#' )
#' }
setup_data <- function(counts = NULL,
                       pg_ids = NULL,
                       sample_metadata = NULL) {
  new_data <- gimap_object()
  class(new_data) <- c("list", "gimap_dataset")
  if (is.null(counts)) stop("counts cannot be NULL")
  if (!is.matrix(counts)) stop("counts can only be in the form of a matrix")
  # If they don't give sample metadata, then we will make up a row id
  if (is.null(sample_metadata)) {
    stop("sample metadata will be required in later steps. Please provide it")
  }
  if (!is.data.frame(sample_metadata)) {
    stop("metadata can only be in the form of a data.frame")
  }
  if (nrow(sample_metadata) != ncol(counts)) {
    stop(
      "the number of rows in the sample metadata",
      "is not equal to the number of columns in the counts"
    )
  }
  if (!(length(unique(sample_metadata[, 1])) ==
    length(sample_metadata[, 1]))) {
    stop("The first column in sample metadata must be a unique ID")
  }
  new_data$metadata$sample_metadata <- sample_metadata
  # If they don't give paired guide ids, then we will make up a row id
  if (is.null(pg_ids)) pg_ids <- data.frame(id = 1:nrow(counts))
  if (!is.data.frame(pg_ids)) {
    stop("pgRNA IDs can only be in the form of a data.frame")
  }
  if (nrow(pg_ids) != nrow(counts)) {
    stop(
      "the number of rows in the pg_info is not equal",
      "to the number of rows in the counts"
    )
  }
  # we need the first column to be a unique id
  if (!(length(unique(pg_ids[, 1])) == length(pg_ids[, 1]))) {
    stop("The paired guide IDs must be a unique ID")
  }
  new_data$metadata$pg_ids <- pg_ids
  # Store the raw counts
  new_data$raw_counts <- counts
  # Calculate the counts per sample
  new_data$counts_per_sample <- apply(counts, 2, sum)
  # Transform the data
  new_data$transformed_data$cpm <-
    apply(counts, 2, function(x) (x / sum(x)) * 1e6)
  new_data$transformed_data$log2_cpm <-
    log2(new_data$transformed_data$cpm + 1)
  return(new_data)
}

#' Make an empty gimap dataset object
#' @description This function makes an empty gimap data object
#' @export
#' @return an empty `gimap_dataset` which is a named list which will
#' be filled by various `gimap` functions.
gimap_object <- function() {
  gimap_obj <- list(
    raw_counts = NULL,
    counts_per_sample = NULL,
    transformed_data = list(
      cpm = NULL,
      log2_cpm = NULL
    ),
    metadata = list(
      pg_ids = NULL,
      sample_metadata = NULL
    ),
    filtered_data = list(
      filter_step_run = FALSE,
      metadata_pg_ids = NULL,
      transformed_log2_cpm = NULL,
      removed_pg_ids = NULL,
      all_reps_zerocount_ids = NULL
    ),
    comparisons = NULL,
    annotation = NULL,
    normalized_log_fc = NULL,
    gi_scores = NULL,
    linear_model = NULL
  )

  return(gimap_obj)
}

#' Normalize DepMap metadata for downstream joins
#' @keywords internal
normalize_depmap_metadata <- function(depmap_metadata) {
  if (is.null(depmap_metadata) || nrow(depmap_metadata) < 1L) {
    return(NULL)
  }
  cols <- colnames(depmap_metadata)
  # Strip UTF-8 BOM from the first column name if present (bad CSV export / download)
  if (length(cols) > 0L) {
    cols[1] <- sub("^\ufeff", "", cols[1])
    colnames(depmap_metadata) <- cols
    cols <- colnames(depmap_metadata)
  }
  # Standardize DepMap ID column across releases (Figshare / portal naming)
  if (!("DepMap_ID" %in% cols) && ("depmap_id" %in% cols)) {
    depmap_metadata <- depmap_metadata %>%
      dplyr::rename(DepMap_ID = depmap_id)
  }
  cols <- colnames(depmap_metadata)
  if (!("DepMap_ID" %in% cols) && ("ModelID" %in% cols)) {
    depmap_metadata <- depmap_metadata %>%
      dplyr::rename(DepMap_ID = ModelID)
  }
  cols <- colnames(depmap_metadata)
  if (!("DepMap_ID" %in% cols) && ("model_id" %in% cols)) {
    depmap_metadata <- depmap_metadata %>%
      dplyr::rename(DepMap_ID = model_id)
  }
  cols <- colnames(depmap_metadata)
  # Standardize stripped cell line name column
  if (!("stripped_cell_line_name" %in% cols)) {
    if ("cell_line" %in% cols) {
      depmap_metadata <- depmap_metadata %>%
        dplyr::mutate(stripped_cell_line_name = toupper(gsub("_.*$", "", .data$cell_line)))
    } else if ("cell_line_name" %in% cols) {
      depmap_metadata <- depmap_metadata %>%
        dplyr::mutate(stripped_cell_line_name = toupper(gsub("[^A-Za-z0-9]", "", .data$cell_line_name)))
    } else if ("Cell_Line_Name" %in% cols) {
      depmap_metadata <- depmap_metadata %>%
        dplyr::mutate(stripped_cell_line_name = toupper(gsub("[^A-Za-z0-9]", "", .data$Cell_Line_Name)))
    } else {
      return(NULL)
    }
  }
  depmap_metadata
}

#' Annotate gimap data
#' @description In this function, a `gimap_dataset` is annotated as far as which
#' genes should be used as controls.
#' @param .data Data can be piped in with tidyverse pipes from function to
#' function. But the data must still be a gimap_dataset
#' @param gimap_dataset A special dataset structure that is setup using the
#' `setup_data()` function.
#' @param annotation_file If no file is given, will attempt to use the design
#' file from
#' https://media.addgene.org/cms/filer_public/a9/9a/
#' a99a9328-324b-42ff-8ccc-30c544b899e4/pgrna_library.xlsx
#' @param control_genes A vector of gene symbols (e.g. AAMP) that should be
#' labeled as control genes. These will be used for log fold change
#' calculations. If no list is given then DepMap Public 23Q4
#' Achilles_common_essentials.csv is used
#' https://depmap.org/portal/download/all/
#' @param cell_line_annotate (Optional) TRUE or FALSE you'd also like to have
#' cell_line_annotation from DepMap.
#' @param cell_line which cell line are you using? (e.g., HELA, PC9, etc.).
#' Required argument if cell_line_annotate is TRUE.
#' @param custom_tpm (Optional) You may supply your own data frame of transcript
#' per million expression to be used for this calculation if you can't or don't
#' want to use DepMap data annotation for your cell_line. This data frame needs
#' to have two columns: 'log2_tpm' that has the log2 tpm expression data for
#' this cell line and and 'genes' which needs to be gene symbols that match
#' those in the data. eg. "NDL1".
#' Note that you can use custom_tpm with cell_line_annotate but your custom_tpm
#' will be used instead of the tpm data from DepMap. However other data from
#' DepMap like CN will be added.
#' @param annot_dir Where should the annotation files be saved? Default is a
#' user data directory.
#' @param refresh_annot Should the annotation file paths saved as options be reset
#' and the files redownloaded? TRUE or FALSE.
#' @return A gimap_dataset with annotation data frame that can be retrieve by using
#' gimap_dataset$annotation. This will contain information about your included
#' genes in the set.
#' @importFrom stringr word
#' @import dplyr
#' @importFrom utils download.file unzip
#' @export
#' @examples \dontrun{
#'
#' # By default DepMap annotation will be used to determine genes which are
#' # unexpressed. In the `gimap_normalize` this will by default be used to
#' # normalize to.
#' gimap_dataset <- get_example_data("gimap",
#'   data_dir = tempdir()
#' ) %>%
#'   gimap_filter() %>%
#'   gimap_annotate(
#'     cell_line = "HELA",
#'     missing_ids_file =  tempfile(),
#'     annot_dir = tempdir()
#'   )
#'
#'
#' # You can also say cell_line_annotate = false if you don't want to use DepMap
#' # annotation BUT if you don't also specify that you say you are
#' # `normalize_by_unexpressed = FALSE` in the normalize step you will get a
#' # warning.
#' gimap_dataset <- get_example_data("gimap",
#'   data_dir = tempdir()
#' ) %>%
#'   gimap_filter() %>%
#'   gimap_annotate(
#'     cell_line_annotate = FALSE,
#'     annot_dir = tempdir()
#'   ) %>%
#'   gimap_normalize(
#'     timepoints = "day",
#'     normalize_by_unexpressed = FALSE,
#'     missing_ids_file = tempfile()
#'   )
#'
#' ### CUSTOM TPM example
#' # Lastly, this is also an option:
#' # where custom data is provided to `custom_tpm` is a data frame with
#' # `genes` and `log2_tpm` as the columns.
#' gimap_dataset <- get_example_data("gimap",
#'   data_dir = tempdir()
#' ) %>%
#'   gimap_filter() %>%
#'   gimap_annotate(
#'     cell_line = "HELA",
#'     custom_tpm = custom_tpm,
#'     annot_dir = tempdir()
#'   ) %>%
#'   gimap_normalize(
#'     timepoints = "day",
#'     missing_ids_file = tempfile()
#'   )
#' }
gimap_annotate <- function(.data = NULL,
                           gimap_dataset,
                           annotation_file = NULL,
                           control_genes = NULL,
                           cell_line_annotate = TRUE,
                           custom_tpm = NULL,
                           cell_line = NULL,
                           annot_dir = gimap_data_dir(),
                           refresh_annot = FALSE) {
  if (!is.null(.data)) gimap_dataset <- .data

  if (!("gimap_dataset" %in% class(gimap_dataset))) {
    stop(
      "This function only works with gimap_dataset objects which can be made",
      "with the setup_data() function."
    )
  }

  if (all(cell_line_annotate, is.null(cell_line))) {
    stop(
      "By default `cell_line_annotate` = TRUE which means if you want DepMap",
      " annotation",
      " you must specify which cell line you are using with the `cell_line`",
      " argument",
      " OR set `cell_line_annotate` = FALSE.",
      " However this means you cannot use expression data for normalization",
      " Aka in gimap_normalize(): `normalize_by_unexpressed` must be set to",
      " FALSE",
      " unless you supply your own expression data using the `custom_tpm`",
      " argument"
    )
  }

  # If data is to be refreshed delete old data
  if (refresh_annot) {
    delete_annotation()
  }

  # Get the annotation data based on the pg construct design
  if (!is.null(annotation_file)) {
    if (!file.exists(annotation_file)) {
      stop(
        "The annotation_file specified cannot be found.",
        "Please double check the file path"
      )
    }
    annotation_df <- readr::read_table(annotation_file)
  } else {
    annotation_df <- get_example_data("annotation")
    if (is.null(annotation_df)) {
      warning(
        "Could not retrieve annotation data. Returning dataset without annotation. ",
        figshare_manual_download_message("28264271", "pgPEN_annotations.txt")
      )
      return(gimap_dataset)
    }
  }

  message("Annotating Data")

  ############################ CONTROL GENE ANNOTATION #########################
  # If control genes aren't provided then we get some from DepMap
  if (!is.null(control_genes)) {
    if (!file.exists(control_genes)) {
      stop(
        "The annotation_file specified cannot be found.",
        "Please double check the file path"
      )
    }
    control_genes <- readr::read_table(control_genes)[, 1]
  } else {
    # This file is from https://depmap.org/portal/download/all/ and
    # from DepMap Public 19Q3 All Files
    # Essential gene labeling is from
    # inst/extdata/Achilles_common_essentials.csv
    control_genes <- ctrl_genes()
    if (is.null(control_genes)) {
      warning(
        "Could not retrieve control genes. Returning dataset without annotation. ",
        figshare_manual_download_message("19700056", "Achilles_common_essentials.csv")
      )
      return(gimap_dataset)
    }
  }

  ############################ Get TPM data ####################################
  depmap_annotation_ok <- isTRUE(cell_line_annotate)
  if (cell_line_annotate) {
    metadata_file <- getOption("gimap_depmap_metadata_file")
    if (!is.null(metadata_file) && file.exists(metadata_file)) {
      depmap_metadata <- tryCatch(
        readr::read_csv(metadata_file, show_col_types = FALSE),
        error = function(e) {
          message("Could not read DepMap metadata from ", metadata_file, ": ", e$message)
          return(NULL)
        }
      )
    } else {
      depmap_metadata <- tryCatch(
        {
          readr::read_csv(
            "https://figshare.com/ndownloader/files/35020903",
            show_col_types = FALSE
          )
        },
        error = function(e) {
          message(
            "Could not download DepMap metadata. ",
            "Please check your internet connection and try again later.\n",
            "Error: ", e$message
          )
          return(NULL)
        }
      )
    }

    if (is.null(depmap_metadata)) {
      message(
        "DepMap metadata could not be loaded. ",
        "Continuing with pgPEN design + control-gene flags only (no DepMap expression/CN). ",
        "Use gimap_normalize(..., normalize_by_unexpressed = FALSE) unless you pass custom_tpm to gimap_annotate()."
      )
      depmap_annotation_ok <- FALSE
    } else if (ncol(depmap_metadata) < 2L) {
      message(
        "DepMap metadata appears invalid (too few columns; blocked download or stale URL?). ",
        "Continuing with pgPEN design + control-gene flags only. ",
        "Use gimap_normalize(..., normalize_by_unexpressed = FALSE) unless you pass custom_tpm to gimap_annotate()."
      )
      depmap_annotation_ok <- FALSE
    } else {
      depmap_metadata <- normalize_depmap_metadata(depmap_metadata)
      if (is.null(depmap_metadata) ||
        !("stripped_cell_line_name" %in% colnames(depmap_metadata)) ||
        !("DepMap_ID" %in% colnames(depmap_metadata))) {
        message(
          "DepMap metadata columns were not recognized (need model id + cell line name). ",
          "Continuing with pgPEN design + control-gene flags only. ",
          "Use gimap_normalize(..., normalize_by_unexpressed = FALSE) unless you pass custom_tpm to gimap_annotate()."
        )
        depmap_annotation_ok <- FALSE
      }
    }

    my_depmap_id <- NULL
    tpm_file <- NULL

    if (isTRUE(depmap_annotation_ok)) {
      my_depmap_id <- depmap_metadata %>%
        dplyr::filter(stripped_cell_line_name == toupper(cell_line)) %>%
        dplyr::pull(DepMap_ID)

      if (length(my_depmap_id) == 0) {
        stop(
          "The cell line specified, ",
          cell_line,
          " was not found in the DepMap data. ",
          "Run supported_cell_lines() to see the full list.",
          call. = FALSE
        )
      }

      tpm_file <- getOption("tpm_file")
      if (is.null(tpm_file)) tpm_file <- tpm_setup(data_dir = annot_dir)
      if (is.null(tpm_file) || !file.exists(tpm_file)) {
        message(
          "Could not obtain TPM file for expression annotation. ",
          "Continuing without DepMap expression. ",
          "Use gimap_normalize(..., normalize_by_unexpressed = FALSE)."
        )
        depmap_annotation_ok <- FALSE
      }
    }

    if (isTRUE(depmap_annotation_ok)) {
      op <- options("VROOM_CONNECTION_SIZE" = 500072)
      on.exit(options(op), add = TRUE)

      tpm <- vroom::vroom(tpm_file,
        show_col_types = FALSE,
        col_select = c("genes", !!my_depmap_id)
      ) %>%
        dplyr::rename(log2_tpm = dplyr::all_of(!!my_depmap_id))

      cn_file <- getOption("cn_file")
      if (is.null(cn_file)) cn_file <- cn_setup(data_dir = annot_dir)

      if (is.null(cn_file) || !file.exists(cn_file)) {
        message(
          "Could not obtain copy-number file; continuing without CN columns on annotations."
        )
      } else {
        depmap_cn <- readr::read_csv(cn_file,
          show_col_types = FALSE,
          col_select = c("genes", dplyr::all_of(!!my_depmap_id))
        ) %>%
          dplyr::rename(log2_cn = dplyr::all_of(!!my_depmap_id))

        annotation_df <- annotation_df %>%
          dplyr::left_join(depmap_cn, by = c("gene1_symbol" = "genes")) %>%
          dplyr::left_join(depmap_cn,
            by = c("gene2_symbol" = "genes"),
            suffix = c("_gene1", "_gene2")
          )
      }
    }

    if (!isTRUE(depmap_annotation_ok)) {
      cell_line_annotate <- FALSE
    }
  }

  # If people supply their own tpm file we need to check it for stuff
  if (!is.null(custom_tpm)) {
    stopifnot(
      "custom_tpm must be a data.frame or tibble" =
        is.data.frame(custom_tpm),
      "custom_tpm must contain a column called 'log2_tpm'" = "log2_tpm" %in%
        colnames(custom_tpm),
      "custom_tpm must contain a column called 'genes'" = "genes" %in%
        colnames(custom_tpm)
    )

    annotation_genes <- unique(c(
      annotation_df$gene1_symbol,
      annotation_df$gene2_symbol
    ))
    gene_matches <- sum(!is.na(match(annotation_genes, custom_tpm$genes)))
    percent <- round(gene_matches / length(annotation_genes) * 100)

    message(
      gene_matches,
      " :number of genes have matches in the custom_tpm data \n",
      percent, "% :percent of genes with matches in the custom_tpm data"
    )

    stopifnot(
      "less than half of the genes have custom_tpm matches" =
        percent > 50
    )
  }
  if (!is.null(custom_tpm) | cell_line_annotate) {
    tpm <- tpm %>%
      dplyr::mutate(expressed_flag = dplyr::case_when(
        log2_tpm < 1 ~ FALSE,
        log2_tpm >= 1 ~ TRUE,
        is.na(log2_tpm) ~ NA
      ))
  }
  ############################ ANNOTATION COMBINING ############################
  # This set up is more or less the same as the original
  annotation_df <- annotation_df %>%
    dplyr::mutate(
      gene1_essential_flag = gene1_symbol %in% control_genes,
      gene2_essential_flag = gene2_symbol %in% control_genes,
      pgRNA_target = dplyr::case_when(
        target_type == "gene_gene" ~ paste(gene1_symbol,
          gene2_symbol,
          sep = "_"
        ),
        target_type == "gene_ctrl" ~ paste(gene1_symbol,
          "ctrl",
          sep = "_"
        ),
        target_type == "ctrl_gene" ~ paste("ctrl",
          gene2_symbol,
          sep = "_"
        ),
        TRUE ~ target_type
      )
    ) %>%
    dplyr::mutate(norm_ctrl_flag = dplyr::case_when(
      target_type == "gene_gene" ~ "double_targeting",
      target_type == "gene_ctrl" & gene1_essential_flag == TRUE ~
        "positive_control",
      target_type == "ctrl_gene" & gene2_essential_flag == TRUE ~
        "positive_control",
      target_type == "gene_ctrl" & gene1_essential_flag != TRUE ~
        "single_targeting",
      target_type == "ctrl_gene" & gene2_essential_flag != TRUE ~
        "single_targeting",
      target_type == "ctrl_ctrl" ~ "negative_control"
    )) %>%
    dplyr::mutate(
      norm_ctrl_flag = factor(norm_ctrl_flag, levels = c(
        "negative_control",
        "positive_control",
        "single_targeting",
        "double_targeting"
      ))
    )
  if (cell_line_annotate | !is.null(custom_tpm)) {
    annotation_df <- annotation_df %>%
      dplyr::left_join(tpm, by = c("gene1_symbol" = "genes")) %>%
      dplyr::rename(gene1_expressed_flag = expressed_flag) %>%
      dplyr::left_join(tpm,
        by = c("gene2_symbol" = "genes"),
        suffix = c("_gene1", "_gene2")
      ) %>%
      dplyr::rename(gene2_expressed_flag = expressed_flag) %>%
      dplyr::mutate(
        unexpressed_ctrl_flag = dplyr::case_when(
          norm_ctrl_flag == "double_targeting" &
            gene1_expressed_flag == FALSE &
            gene2_expressed_flag == FALSE ~ TRUE,
          norm_ctrl_flag == "single_targeting" & (
            gene1_expressed_flag == FALSE |
              gene2_expressed_flag == FALSE) ~ TRUE,
          TRUE ~ FALSE
        )
      )
  }
  ################################ STORE IT ####################################

  if (gimap_dataset$filtered_data$filter_step_run) {
    keep_for_annotdf <- annotation_df$pgRNA_id %in%
      unlist(gimap_dataset$filtered_data$metadata_pg_ids)
    annotation_df <- annotation_df[keep_for_annotdf, ]
  }

  gimap_dataset$annotation <- annotation_df

  return(gimap_dataset)
}

#' Download and set up DepMap TPM data
#' @description  This function sets up the tpm data from DepMap is
#' called by the `gimap_annotate()` function
#' @param overwrite should the files be re downloaded
#' @param data_dir What directory should this be saved to? Default is a user data
#' directory.
#' @importFrom utils download.file unzip
tpm_setup <- function(overwrite = TRUE,
                      data_dir = gimap_data_dir()) {
  tpm_file <- file.path(data_dir, "CCLE_expression.csv")

  options("tpm_file" = tpm_file)
  if (!file.exists(tpm_file) | overwrite) {
    if (!file.exists(file.path(data_dir, "CCLE_expression.csv.zip"))) {
      download_result <- get_figshare(
        file_name = "CCLE_expression.csv",
        output_dir = data_dir
      )
      if (is.null(download_result)) {
        warning(
          "Could not download TPM data from Figshare. ",
          figshare_manual_download_message("19700056", "CCLE_expression.csv")
        )
        return(NULL)
      }
    } else {
      unzip(file.path(data_dir, "CCLE_expression.csv.zip"),
        exdir = data_dir, overwrite = TRUE
      )
    }
  }
  if (dir.exists(file.path(data_dir, "__MACOSX"))) {
    file.remove(file.path(data_dir, "__MACOSX"))
  }

  # Check if file exists after download attempt
  if (!file.exists(tpm_file)) {
    warning(
      "TPM file not found after download attempt: ", tpm_file, ". ",
      figshare_manual_download_message("19700056", "CCLE_expression.csv")
    )
    return(NULL)
  }

  data_df <- readr::read_csv(tpm_file,
    show_col_types = FALSE,
    name_repair = make.names
  )

  if ("X" %in% colnames(data_df)) {
    cell_line_ids <- data_df$X

    genes <- stringr::word(colnames(data_df)[-1], sep = "\\.\\.", 1)

    colnames(data_df) <- c("cell_line_ids", genes)

    data_df <- as.data.frame(t(data_df[, -1]))
    colnames(data_df) <- cell_line_ids
    data_df$genes <- genes

    data_df %>%
      dplyr::select(genes, dplyr::everything()) %>%
      readr::write_csv(tpm_file)
  }
  return(tpm_file)
}

#' Download and set up DepMap CN
#' @description This function sets up the tpm data from DepMap is called
#' by the `gimap_annotate()` function if the cn_annotate = TRUE
#' @param overwrite Should the files be redownloaded?
#' @param data_dir What directory should this be saved to? Default is a user data
#' directory.
#' @importFrom utils download.file unzip
cn_setup <- function(overwrite = TRUE,
                     data_dir = gimap_data_dir()) {
  options(timeout = 1000)

  cn_file <- file.path(
    data_dir,
    "CCLE_gene_cn.csv"
  )

  options("cn_file" = cn_file)

  if (!file.exists(cn_file) | overwrite) {
    if (!file.exists(file.path(data_dir, "CCLE_gene_cn.csv.zip"))) {
      download_result <- get_figshare(
        file_name = "CCLE_gene_cn.csv",
        output_dir = data_dir
      )
      if (is.null(download_result)) {
        warning(
          "Could not download CN data from Figshare. ",
          figshare_manual_download_message("19700056", "CCLE_gene_cn.csv")
        )
        return(NULL)
      }
    } else {
      unzip(file.path(data_dir, "CCLE_gene_cn.csv.zip"),
        exdir = data_dir, overwrite = TRUE
      )
    }
  }
  if (dir.exists(file.path(data_dir, "__MACOSX"))) {
    file.remove(file.path(data_dir, "__MACOSX"))
  }

  # Check if file exists after download attempt
  if (!file.exists(cn_file)) {
    warning(
      "CN file not found after download attempt: ", cn_file, ". ",
      figshare_manual_download_message("19700056", "CCLE_gene_cn.csv")
    )
    return(NULL)
  }

  data_df <- readr::read_csv(cn_file,
    show_col_types = FALSE,
    name_repair = make.names
  )
  if ("X" %in% colnames(data_df)) {
    cell_line_ids <- data_df$X

    genes <- stringr::word(colnames(data_df)[-1], sep = "\\.\\.", 1)

    colnames(data_df) <- c("cell_line_ids", genes)

    data_df <- as.data.frame(t(data_df[, -1]))
    colnames(data_df) <- cell_line_ids
    data_df$genes <- genes

    data_df %>%
      dplyr::select(genes, dplyr::everything()) %>%
      readr::write_csv(cn_file)
  }

  return(cn_file)
}

#' Download and set up control genes
#' @description This function sets up the control genes file from DepMap is
#' called by the `gimap_annotate()`
#' @param overwrite Should the file be redownloaded and reset up?
#' @param data_dir What directory should this be saved to? Default is a user data
#' directory.
#' @importFrom utils download.file unzip
ctrl_genes <- function(overwrite = TRUE,
                       data_dir = gimap_data_dir()) {
  ctrl_genes_file <- file.path(data_dir, "Achilles_common_essentials.csv")

  options("ctrl_genes_file" = ctrl_genes_file)

  if (!file.exists(ctrl_genes_file) | overwrite) {
    # Can also be downloaded like this:
    if (!file.exists(file.path(
      data_dir,
      "Achilles_common_essentials.csv.zip"
    ))) {
      download_result <- get_figshare(
        file_name = "Achilles_common_essentials.csv",
        output_dir = data_dir
      )
      if (is.null(download_result)) {
        warning(
          "Could not download control genes data from Figshare. ",
          figshare_manual_download_message("19700056", "Achilles_common_essentials.csv")
        )
        return(NULL)
      }
    } else {
      unzip(file.path(data_dir, "Achilles_common_essentials.csv.zip"),
        exdir = data_dir,
        overwrite = TRUE
      )
      if (dir.exists(file.path(data_dir, "__MACOSX"))) {
        file.remove(file.path(data_dir, "__MACOSX"))
      }
    }
  }

  # Check if file exists after download attempt
  if (!file.exists(ctrl_genes_file)) {
    warning(
      "Control genes file not found after download attempt: ",
      ctrl_genes_file, ". ",
      figshare_manual_download_message("19700056", "Achilles_common_essentials.csv")
    )
    return(NULL)
  }

  ctrl_genes <- readr::read_csv(ctrl_genes_file, show_col_types = FALSE) %>%
    tidyr::separate(
      col = gene, into = c("gene_symbol", "entrez_id"),
      remove = FALSE, extra = "drop"
    )

  return(ctrl_genes$gene_symbol)
}


#' List the supported cell lines
#' @description This function downloads the metadata for DepMap and lists which
#' cell lines are supported.
#' @export
#' @return A list of the cell line names that are available in DepMap for use
#' for annotation in this package.
#' @examples \dontrun{
#'
#' cell_lines <- supported_cell_lines()
#' }
supported_cell_lines <- function() {
  metadata_file <- getOption("gimap_depmap_metadata_file")
  if (!is.null(metadata_file) && file.exists(metadata_file)) {
    depmap_metadata <- tryCatch(
      readr::read_csv(metadata_file, show_col_types = FALSE),
      error = function(e) {
        message("Could not read DepMap metadata from ", metadata_file, ": ", e$message)
        return(NULL)
      }
    )
  } else {
    depmap_metadata <- tryCatch(
      {
        readr::read_csv(
          "https://figshare.com/ndownloader/files/35020903",
          show_col_types = FALSE
        )
      },
      error = function(e) {
        message(
          "Could not download DepMap metadata to retrieve supported cell lines. ",
          "Please check your internet connection and try again later.\n",
          "Error: ", e$message
        )
        return(NULL)
      }
    )
  }

  if (is.null(depmap_metadata)) {
    return(character(0))
  }

  depmap_metadata <- normalize_depmap_metadata(depmap_metadata)
  if (is.null(depmap_metadata) ||
      !("stripped_cell_line_name" %in% colnames(depmap_metadata))) {
    message(
      "DepMap metadata format has changed - expected column 'stripped_cell_line_name' ",
      "(or 'cell_line' / 'cell_line_name') not found."
    )
    return(character(0))
  }

  return(sort(depmap_metadata$stripped_cell_line_name))
}

#' Refresh the annotation files by redownloading them
#' @description This function will set annotation file options to NULL so files
#' will be re-downloaded
#' @export
#' @return options for tpm_file, cn_file, and ctrl_genes_file are set to NULL.
#' @examples
#'
#' delete_annotation()
#'
delete_annotation <- function() {
  message("Deleting original files")
  unlink(options("tpm_file"))
  unlink(options("cn_file"))
  unlink(options("ctrl_genes_file"))

  options("tpm_file" = NULL)
  options("cn_file" = NULL)
  options("ctrl_genes_file" = NULL)
}

test_that("Annotation options", {
  skip_if_figshare_unavailable()
  skip_if_depmap_changed()
  data_dir <- test_example_data_dir()

  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line = "HELA")

  # We should see these columns
  testthat::expect_true(all(c(
    "norm_ctrl_flag",
    "log2_tpm_gene1", "log2_tpm_gene2",
    "log2_cn_gene1", "log2_cn_gene2",
    "gene1_expressed_flag", "gene2_expressed_flag"
  ) %in%
    colnames(gimap_dataset$annotation)))

  # Without DepMap TPM, normalize_by_unexpressed is coerced to FALSE with a message
  testthat::expect_message(
    gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
      gimap_filter() %>%
      gimap_annotate(cell_line_annotate = FALSE) %>%
      gimap_normalize(timepoints = "day"),
    "normalize_by_unexpressed = FALSE"
  )
  testthat::expect_true(!is.null(gimap_dataset$normalized_log_fc))

  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(cell_line_annotate = FALSE) %>%
    gimap_normalize(
      timepoints = "day",
      normalize_by_unexpressed = FALSE
    )

  # We should see these columns
  testthat::expect_true(all(c("lfc", "crispr_score", "norm_ctrl_flag") %in%
    colnames(gimap_dataset$normalized_log_fc)))

  ## Try out using custom TPM data
  tpm_file <- tpm_setup(data_dir = data_dir)
  custom_tpm <- vroom::vroom(
    tpm_file,
    show_col_types = FALSE,
    col_select = c("genes", "ACH-001086")
  ) %>%
    dplyr::rename(log2_tpm = `ACH-001086`)

  gimap_dataset <- get_example_data("gimap", data_dir = data_dir) %>%
    gimap_filter() %>%
    gimap_annotate(
      custom_tpm = custom_tpm,
      cell_line = "HELA"
    )

  gimap_dataset <- gimap_dataset %>%
    gimap_normalize(
      timepoints = "day",
      normalize_by_unexpressed = FALSE,
      overwrite = TRUE
    )

  # We should see these columns
  testthat::expect_true(all(c("lfc", "crispr_score", "norm_ctrl_flag") %in%
    colnames(gimap_dataset$normalized_log_fc)))
})

# Helper function to skip tests when Figshare is unavailable
# This is called within test_that() blocks, not at the top level

skip_if_figshare_unavailable <- function() {
  # Skip on CRAN since these tests require external resources
  testthat::skip_on_cran()

  # Check if Figshare API is reachable
  figshare_available <- tryCatch(
    {
      response <- httr::HEAD("https://api.figshare.com", httr::timeout(5))
      httr::status_code(response) < 400
    },
    error = function(e) FALSE
  )

  if (!figshare_available) {
    testthat::skip("Figshare API is not available (no internet or service down)")
  }
}

# Helper function to create a writable example data directory for tests
test_example_data_dir <- function() {
  data_dir <- file.path(tempdir(), "gimap_example_data")
  if (!dir.exists(data_dir)) {
    dir.create(data_dir, recursive = TRUE, showWarnings = FALSE)
  }
  return(data_dir)
}

# Helper function to skip tests when DepMap metadata format has changed.
# When remote metadata is unavailable or has wrong format, uses bundled fixture
# so tests can run without depending on external DepMap/Figshare format.
skip_if_depmap_changed <- function() {
  # Skip on CRAN since these tests require external resources
  testthat::skip_on_cran()

  # Try remote DepMap metadata first
  depmap_valid <- tryCatch(
    {
      depmap_metadata <- readr::read_csv(
        "https://figshare.com/ndownloader/files/35020903",
        show_col_types = FALSE,
        n_max = 1
      )
      depmap_metadata <- gimap:::normalize_depmap_metadata(depmap_metadata)
      !is.null(depmap_metadata) &&
        nrow(depmap_metadata) > 0 &&
        "stripped_cell_line_name" %in% colnames(depmap_metadata) &&
        "DepMap_ID" %in% colnames(depmap_metadata)
    },
    error = function(e) FALSE
  )

  if (!depmap_valid) {
    # Use bundled fixture so tests still run when external format changes
    fixture_path <- system.file(
      "extdata", "depmap_metadata_fixture.csv",
      package = "gimap", mustWork = FALSE
    )
    if (nzchar(fixture_path) && file.exists(fixture_path)) {
      options(gimap_depmap_metadata_file = fixture_path)
    } else {
      testthat::skip("DepMap metadata format has changed or is unavailable")
    }
  }
}

---
title: "Quick Start"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Quick Start}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Quick Start for gimap

For more background on gimap and the calculations done here, [read here](https://github.com/FredHutch/gimap/blob/main/README.md).

## About this example

This Quick Start walks through a dual-targeting (paired-guide) CRISPR screen
from start to finish using a small example dataset bundled through Figshare.

**Study design of the example data.** The counts come from a paralog genetic
interaction screen in HeLa cells using the pgPEN (paired guide RNAs for
paralog genetic interaction mapping) library. Cells expressing Cas9 are
transduced with a pooled library of paired-guide RNAs (pgRNAs) that knock out
either one gene (single-target constructs, paired with a non-targeting
control) or two paralogous genes simultaneously (double-target constructs).
Cells are collected at a baseline timepoint (Day 0, representing the library
composition before selection) and again after growth selection (Day 22),
with three biological replicates at the later timepoint. Changes in pgRNA
abundance between Day 0 and Day 22 reflect the fitness effect of each
single- or double-knockout, and comparing observed to expected double-knockout
fitness reveals genetic interactions (synthetic lethality or buffering).

The example data are a subset of the screen described in Thompson *et al.*
(2021), *Cell Reports* - [Discovery of synthetic lethal and tumor suppressor
paralog pairs in the human genome](https://pubmed.ncbi.nlm.nih.gov/34469736/).
Refer to that paper for the full experimental protocol, library composition,
and discussion of the biological results.

**How to map your own experiment onto this workflow.** You will need:

- a counts table of pgRNA reads (rows = pgRNA constructs, columns = samples),
- a table of pgRNA construct IDs (matching the row order of the counts), and
- a sample metadata table (matching the column order of the counts) that
  describes each sample - in this example, the timepoint (`day`) and
  replicate (`rep`).

The rest of the tutorial shows how these three pieces are assembled into a
`gimap_dataset` object and run through QC, filtering, annotation,
normalization, and genetic interaction scoring.

## Requirements

Besides installing the gimap package, you will also need to install wget if you do not already have it installed. This will allow you to download the annotation files needed to run `gimap`.

[How to install wget](https://phoenixnap.com/kb/wget-command-with-examples#How_to_Install_wget)

## Data download requirements

Example data are hosted on Figshare and are not bundled with the package to keep
the CRAN tarball small. `get_example_data()` will download the files on first use
to a user data directory. For manual download instructions, see the README section
"Manual data download (Figshare)".

To install the `gimap` package you will need to run:
```
install.packages("gimap")
```
Or you can install the development version from GitHub:
```
install.packages("remotes")
remotes::install_github("FredHutch/gimap")
```

## Loading needed packages

Attach `gimap` and `dplyr` before running any of the code below. We use
`suppressPackageStartupMessages()` to keep the rendered output tidy, but the
calls themselves are essential.

```{r}
suppressPackageStartupMessages({
  library(gimap)
  library(dplyr)
})
```

## Set Up

First we can create a folder we will save files to.

```{r eval = FALSE}
output_dir <- "output_timepoints"
dir.create(output_dir, showWarnings = FALSE)
```

Download the example counts table from Figshare. `get_example_data("count")`
caches the file under `tempdir()` on first use and reads it in as a
[tibble](https://tibble.tidyverse.org/):

```{r   eval = FALSE}
example_data <- get_example_data("count")
```

`example_data` is a tibble where each row is a paired-guide (pgRNA) construct
and each column is either a construct-level identifier or a sample's read
counts. The columns are:

- `id` - unique identifier for the pgRNA construct (e.g. `CCNL1_CCNL2_pg1`).
- `seq_1` - gRNA sequence targeting the first gene ("paralog A") in the pair.
- `seq_2` - gRNA sequence targeting the second gene ("paralog B") in the pair.
- `Day00_RepA` - raw read counts at Day 0 (plasmid / baseline timepoint),
  replicate A.
- `Day05_RepA` - raw read counts at Day 5, replicate A (not used in this
  Quick Start; it is dropped below).
- `Day22_RepA`, `Day22_RepB`, `Day22_RepC` - raw read counts at Day 22
  (post-selection endpoint) for the three biological replicates.

You can inspect the structure and first few rows with `dplyr::glimpse()` or
`head(example_data)`.

## Setting up data

We're going to set up three datasets that we will provide to the `set_up()` function to create a `gimap` dataset object.

- `counts` - the counts generated from pgPEN
- `pg_ids` - the IDs that correspond to the rows of the counts and specify the construct
- `sample_metadata` - metadata that describes the columns of the counts including their timepoints

```{r   eval = FALSE}
counts <- example_data %>%
  select(c("Day00_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC")) %>%
  as.matrix()
```

`pg_id` are just the unique IDs listed in the same order/sorted the same way as the count data.

```{r   eval = FALSE}
pg_ids <- example_data %>%
  dplyr::select("id")
```

Sample metadata is the information that describes the samples and is sorted the same order as the columns in the count data.

```{r   eval = FALSE}
sample_metadata <- data.frame(
  col_names = c("Day00_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC"),
  day = as.numeric(c("0", "22", "22", "22")),
  rep = as.factor(c("RepA", "RepA", "RepB", "RepC"))
)
```

We'll need to provide `example_counts`, `pg_ids` and `sample_metadata` to `setup_data()`.

```{r   eval = FALSE}
gimap_dataset <- setup_data(
  counts = counts,
  pg_ids = pg_ids,
  sample_metadata = sample_metadata
)
```

It's ideal to run quality checks first. The `run_qc()` function will create a report we can look at to assess this.

```{r   eval = FALSE}
run_qc(gimap_dataset,
  output_file = file.path(output_dir, "example_qc_report.Rmd"),
  overwrite = TRUE,
  quiet = TRUE
)
```

You can take a look at an [example QC report here](http://htmlpreview.github.io/?https://raw.githubusercontent.com/FredHutch/gimap/main/inst/example_qc_report.html).

`gimap_annotate()` adds pgPEN design metadata and, when DepMap downloads succeed,
expression and copy-number columns for your `cell_line`. If the bundled DepMap
metadata URL is blocked or the file format changes, annotation still completes
with design + control-gene flags; `gimap_normalize()` then turns off
`normalize_by_unexpressed` automatically when TPM-based flags are missing.

The rest of the pipeline can be chained together:

- `gimap_filter()` removes low-quality pgRNA constructs flagged during QC
  (e.g. zero counts across replicates or very low plasmid abundance).
- `gimap_annotate(cell_line = "HELA")` annotates constructs with DepMap /
  CCLE information (gene expression, copy number, and common-essential
  status) for the specified cell line, which is needed to build a useful
  set of positive/negative controls.
- `gimap_normalize(timepoints = "day")` turns raw counts into log2 CPM,
  computes log2 fold-changes versus the Day 0 timepoint, and converts them
  into CRISPR scores normalized by the positive- and negative-control
  distributions.
- `calc_gi()` computes the genetic interaction (GI) scores and associated
  statistics (see the next section for a detailed description).

```{r   eval = FALSE}
gimap_dataset <- gimap_dataset %>%
  gimap_filter() %>%
  gimap_annotate(cell_line = "HELA") %>%
  gimap_normalize(
    timepoints = "day"
  ) %>%
  calc_gi()
```


## Example output

### How `calc_gi()` computes genetic interaction scores

`calc_gi()` quantifies how much the observed fitness of a double-knockout
deviates from what would be expected if the two genes acted independently.
For each sample it:

1. Builds the *expected* double-target CRISPR score for each pgRNA by summing
   the two corresponding single-target CRISPR scores (i.e. the single
   knockouts that use the same guide sequences paired with a non-targeting
   control).
2. Fits a linear model of observed vs. expected single-target CRISPR scores
   across all constructs - this line represents what "no interaction" looks
   like for that sample.
3. Defines the **genetic interaction (GI) score** for each construct as the
   residual from that line: the distance between the observed CRISPR score
   and the value predicted from the fit. Negative GI scores indicate
   synthetic lethality (the double knockout is worse for cell fitness than
   expected); positive GI scores indicate buffering or cooperativity.
4. For each target pair, runs a t-test comparing the distribution of its
   double-targeting construct GI scores against the overall distribution of
   single-targeting GI scores, then applies Benjamini-Hochberg FDR
   correction across pairs.

For a full description of the underlying math (including the log2 fold-change
and CRISPR-score normalization steps), see the
[About Genetic Interaction Scores](https://github.com/FredHutch/gimap/blob/main/README.md#about-genetic-interaction-scores)
section of the package README and the documentation for `calc_gi()`.

### The `gi_scores` results table

The target-level results are stored in `gimap_dataset$gi_scores`, with one
row per gene pair. The key columns are:

- `pgRNA_target` - the gene(s) targeted by the original pgRNAs for this row
  (e.g. `CNOT8_CNOT7` for a double-targeting pair, or `CNOT8_ctrl` for a
  single-targeting construct).
- `target_type` - the CRISPR design type: `"gene_gene"` (two genes targeted),
  `"gene_ctrl"` (gene in position 1, non-targeting control in position 2),
  or `"ctrl_gene"` (non-targeting control in position 1, gene in position 2).
- `mean_expected_cs` - the mean expected CRISPR score for the pair, computed
  as described above.
- `mean_gi_score` - the mean observed GI score across the constructs that
  target this pair (averaged over replicates when `stat_by_rep = FALSE`).
  Negative values suggest synthetic lethality, positive values suggest
  buffering / cooperativity.
- `p_val` - p-value from the t-test comparing the pair's double-targeting
  GI scores to the overall distribution of single-targeting GI scores.
- `fdr` - Benjamini-Hochberg FDR-adjusted p-value; low values flag pairs
  whose interaction is unlikely to be due to chance.

The block below arranges the table by FDR and shows the top hits:

```{r   eval = FALSE}
gimap_dataset$gi_scores %>%
  dplyr::arrange(fdr) %>%
  head() %>%
  knitr::kable(format = "html")
```

## Plot the results

You can remove any samples from these plots by altering the `reps_to_drop` argument.

```{r   eval = FALSE}
plot_exp_v_obs_scatter(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "exp_v_obs_scatter.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/exp_v_obs_scatter.png")
```

```{r   eval = FALSE}
plot_rank_scatter(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "plot_rank_scatter.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/plot_rank_scatter.png")
```

```{r   eval = FALSE}
plot_volcano(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "volcano_plot.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/volcano_plot.png")
```

### Plot specific target pair

We can pick out a specific pair to plot.

```{r   eval = FALSE}
# "CNOT8_CNOT7" is top result so let's plot that
plot_targets(gimap_dataset, target1 = "CNOT8", target2 = "CNOT7")

# Save it to a file
ggsave(file.path(output_dir, "CNOT8_CNOT7.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/CNOT8_CNOT7.png")
```

## Saving data to a file

We can save all these data as an RDS or the genetic interaction scores themselves to a tsv file.

```
saveRDS(gimap_dataset, "gimap_dataset_final.RDS")
```

```
readr::write_tsv(gimap_dataset$gi_scores, "gi_scores.tsv")
```

## Session Info

This is just for provenance purposes.

```{r}
sessionInfo()
```

---
title: "QC Report"
date: "`r Sys.Date()`"
author: "`r paste0('gimap (v', utils::packageVersion('gimap'), ')')`"
params:
  dataset: NULL
  plots_dir: ./plots
  filter_zerocount_target_col: NULL
  filter_plasmid_target_col: NULL
  filter_replicates_target_col: NULL
output: 
  html_document:
    theme: spacelab
    toc: true
    toc_float: TRUE
    df_print: kable
    code_folding: show
  pdf_document:
    toc: true
  always_allow_html: yes
  editor_options:
    chunk_output_type:console
---

<style>
  tbody tr:nth-child(odd){
    background-color: #D6E0F5;
}
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
knitr::opts_knit$set(progress = TRUE, verbose = FALSE)
library(gimap)
library(ggplot2)
library(purrr)
```

# Unfiltered/raw data

## Counts CDF

```{r}
qc_cdf(gimap_dataset)
```

## Histogram of cpm values by sample

```{r}
qc_sample_hist(gimap_dataset)
```


## Sample Correlation heatmap (unfiltered data)

```{r}
qc_cor_heatmap(gimap_dataset)
```

## Variance within replicates

```{r}
qc_variance_hist(gimap_dataset, filter_replicates_target_col = filter_replicates_target_col)
```

## Plasmid expression (log 2 cpm values for Day 0 sample/time point)

```{r}
qc_plasmid_histogram(gimap_dataset, filter_plasmid_target_col = filter_plasmid_target_col)
```

# Applying potential filters

## Filter pgRNAs where there is a count of 0 for any sample/time point

### Replicates with a pgRNA count of 0 

```{r}
qc_constructs_countzero_bar(gimap_dataset, filter_zerocount_target_col = filter_zerocount_target_col, filter_replicates_target_col = filter_replicates_target_col)
```

If this filter is applied, this is the number of pgRNAs that would be filtered out

```{r}
potentialFilter1 <- qc_filter_zerocounts(gimap_dataset, filter_zerocount_target_col = filter_zerocount_target_col)

potentialFilter1$reportdf
```

### Filter pgRNAs where there is a low log2 CPM value for the plasmid sample/time point

If this filter is applied, this is the number of pgRNAs that would be filtered out

```{r}
potentialFilter2 <- qc_filter_plasmid(gimap_dataset, filter_plasmid_target_col = filter_plasmid_target_col)

potentialFilter2$reportdf
```

### If both filters are applied

```{r}
combined_filters <- reduce(list(potentialFilter1$filter, potentialFilter2$filter), cbind)
```

| Which Filter(s) | Number of pgRNAs flagged for removal | Percent of total pgRNA constructs |
|:---------------|:------------------------------------|:----------|
| Zero count, but not low plasmid CPM | `r sum(combined_filters[,1] == TRUE & combined_filters[,2] == FALSE)` | `r round(sum(combined_filters[,1] == TRUE & combined_filters[,2] == FALSE) / nrow(combined_filters) * 100, 2)`|
| Low plasmid CPM, but not zero count | `r sum(combined_filters[,2] == TRUE & combined_filters[,1] == FALSE)` | `r round(sum(combined_filters[,2] == TRUE & combined_filters[,1] == FALSE) / nrow(combined_filters) * 100, 2)` |
| Either Zero count or Low plasmid CPM or both | `r sum(rowSums(combined_filters) >= 1)`| `r round(sum(rowSums(combined_filters) >= 1) / nrow(combined_filters) * 100, 2)`  |
| Both Zero count and Low plasmid CPM | `r sum(rowSums(combined_filters) == 2)` | `r round(sum(rowSums(combined_filters) == 2) / nrow(combined_filters) * 100, 2)` |
| Remaining pgRNAs flagged by no filters | `r sum(rowSums(combined_filters) == 0)` |  `r round(sum(rowSums(combined_filters) == 0) / nrow(combined_filters) * 100, 2)` |

# Session Info

```{r}
sessionInfo()
```

---
title: "Quick Start"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Quick Start}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Quick Start for gimap

For more background on gimap and the calculations done here, [read here](https://github.com/FredHutch/gimap/blob/main/README.md)

## Requirements

Besides installing the gimap package, you will also need to install wget if you do not already have it installed. This will allow you to download the annotation files needed to run `gimap`.

[How to install wget](https://phoenixnap.com/kb/wget-command-with-examples#How_to_Install_wget)

## Data download requirements

Example data are hosted on Figshare and are not bundled with the package to keep
the CRAN tarball small. `get_example_data()` will download the files on first use
to a user data directory. For manual download instructions, see the README section
"Manual data download (Figshare)".

To install the `gimap` package you will need to run:
```
install.packages("gimap")
```
Or you can install the development version from GitHub:
```
install.packages("remotes")
remotes::install_github("FredHutch/gimap")
```

## Loading needed packages

```{r echo = FALSE, results = 'hide'}
library(gimap)
```

```{r echo = FALSE, results = 'hide'}
library(dplyr)
```

## Set Up

First we can create a folder we will save files to.

```{r eval = FALSE}
output_dir <- "output_timepoints"
dir.create(output_dir, showWarnings = FALSE)
```

```{r   eval = FALSE}
example_data <- get_example_data("count")
```

## Setting up data

We're going to set up three datasets that we will provide to the `set_up()` function to create a `gimap` dataset object.

- `counts` - the counts generated from pgPEN
- `pg_ids` - the IDs that correspond to the rows of the counts and specify the construct
- `sample_metadata` - metadata that describes the columns of the counts including their timepoints

```{r   eval = FALSE}
counts <- example_data %>%
  select(c("Day00_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC")) %>%
  as.matrix()
```

`pg_id` are just the unique IDs listed in the same order/sorted the same way as the count data.

```{r   eval = FALSE}
pg_ids <- example_data %>%
  dplyr::select("id")
```

Sample metadata is the information that describes the samples and is sorted the same order as the columns in the count data.

```{r   eval = FALSE}
sample_metadata <- data.frame(
  col_names = c("Day00_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC"),
  day = as.numeric(c("0", "22", "22", "22")),
  rep = as.factor(c("RepA", "RepA", "RepB", "RepC"))
)
```

We'll need to provide `example_counts`, `pg_ids` and `sample_metadata` to `setup_data()`.

```{r   eval = FALSE}
gimap_dataset <- setup_data(
  counts = counts,
  pg_ids = pg_ids,
  sample_metadata = sample_metadata
)
```

It's ideal to run quality checks first. The `run_qc()` function will create a report we can look at to assess this.

```{r   eval = FALSE}
run_qc(gimap_dataset,
  output_file = file.path(output_dir, "example_qc_report.Rmd"),
  overwrite = TRUE,
  plots_dir = "plots",
  quiet = TRUE
)
```

You can take a look at an [example QC report here](http://htmlpreview.github.io/?https://raw.githubusercontent.com/FredHutch/gimap/main/inst/example_qc_report.html).

```{r   eval = FALSE}
gimap_dataset <- gimap_dataset %>%
  gimap_filter() %>%
  gimap_annotate(cell_line = "HELA") %>%
  gimap_normalize(
    timepoints = "day"
  ) %>%
  calc_gi()
```


## Example output

Genetic interaction is calculated by:

- `pgRNA_target` - what gene(s) were targeted by this the original pgRNAs for these data
- `mean_expected_cs` - the average expected genetic interaction score
- `mean_gi_score` - the average observer genetic interaction score
- `target_type` - describes whether the CRISPR design is targeting two genes ("gene_gene"), or a gene and a non targeting control ("gene_ctrl") or a targeting control and a gene ("ctrl_gene").
- `p_val` - p values from the testing whether a double knockout construct is significantly different in its genetic interaction score from single targets.  
- `fdr` - False discovery rate corrected p values

```{r   eval = FALSE}
gimap_dataset$gi_scores %>%
  dplyr::arrange(fdr) %>%
  head() %>%
  knitr::kable(format = "html")
```

## Plot the results

You can remove any samples from these plots by altering the `reps_to_drop` argument.

```{r   eval = FALSE}
plot_exp_v_obs_scatter(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "exp_v_obs_scatter.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/exp_v_obs_scatter.png")
```

```{r   eval = FALSE}
plot_rank_scatter(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "plot_rank_scatter.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/plot_rank_scatter.png")
```

```{r   eval = FALSE}
plot_volcano(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "volcano_plot.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/volcano_plot.png")
```

### Plot specific target pair

We can pick out a specific pair to plot.

```{r   eval = FALSE}
# "CNOT8_CNOT7" is top result so let's plot that
plot_targets(gimap_dataset, target1 = "CNOT8", target2 = "CNOT7")

# Save it to a file
ggsave(file.path(output_dir, "CNOT8_CNOT7.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_timepoints/CNOT8_CNOT7.png")
```

## Saving data to a file

We can save all these data as an RDS or the genetic interaction scores themselves to a tsv file.

```
saveRDS(gimap_dataset, "gimap_dataset_final.RDS")
```

```
readr::write_tsv(gimap_dataset$gi_scores, "gi_scores.tsv")
```

## Session Info

This is just for provenance purposes.

```{r}
sessionInfo()
```

---
title: "gimap for Treatment Experiment"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{gimap for Treatment Experiment}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# gimap tutorial for treatments

gimap performs analysis of dual-targeting CRISPR screening data, with the goal of aiding the identification of genetic interactions (e.g. cooperativity, synthetic lethality) in models of disease and other biological contexts. gimap analyzes functional genomic data generated by the pgPEN (paired guide RNAs for genetic interaction mapping) approach, quantifying growth effects of single and paired gene knockouts upon application of a CRISPR library. A multitude of CRISPR screen types can be used for this analysis, with helpful descriptions found in this review (https://www.nature.com/articles/s43586-021-00093-4). Use of pgPEN and GI-mapping in a paired gRNA format can be found here (https://pubmed.ncbi.nlm.nih.gov/34469736/).

## Requirements

Besides installing the gimap package, you will also need to install wget if you do not already have it installed. This will allow you to download the annotation files needed to run `gimap`.

[How to install wget](https://phoenixnap.com/kb/wget-command-with-examples#How_to_Install_wget)

## Data download requirements

Example data are hosted on Figshare and are not bundled with the package to keep
the CRAN tarball small. `get_example_data()` will download the files on first use
to a user data directory. For manual download instructions, see the README section
"Manual data download (Figshare)".

To install this package you will need to run:
```
install.packages("gimap")
```
Or you can install the development version from GitHub:
```
install.packages("remotes")
remotes::install_github("FredHutch/gimap")
```

## Loading needed packages

```{r echo = FALSE, results = 'hide'}
library(gimap)
```
```
library(dplyr)
```

```{r eval = FALSE}
output_dir <- "output_treatment"
dir.create(output_dir, showWarnings = FALSE)
```

## Data loading and setup

In this example we are going to examine a dataset with drug treatments and controls.

Let's examine this example pgPEN counts table. It's divided into columns containing:

- `id`: an ID corresponding to the names of paired guides
- `seq_1`: gRNA sequence 1, targeting "paralog A"
- `seq_2`: gRNA sequence 2, targeting "paralog B"
- `pretreatment`: The pretreated control
- `dmsoA`: Vehicle only for Replicate A
- `dmsoB`: Vehicle only for Replicate B
- `drug1A`: Drug treatment for Replicate A
- `drug2B`: Drug treatment for Replicate B

For the purposes of this tutorial, we can download example data from Figshare.

```{r eval = FALSE}
example_data <- get_example_data("count_treatment")
```

The metadata you have may vary slightly from this but you'll want to make sure you have the essential variables and information regarding how you collected your data.

```{r eval = FALSE}
colnames(example_data)
```

## Setting up data

We're going to set up three datasets that we will provide to the `set_up()` function to create a `gimap` dataset object.

- `counts` - the counts generated from pgPEN
- `pg_ids` - the IDs that correspond to the rows of the counts and specify the construct
- `sample_metadata` - metadata that describes the columns of the counts including their timepoints

For this example we are using a treatment dataset where there is a pretreatment sample, two vehicle replicates and two drug treated replicates.

```{r eval = FALSE}
counts <- example_data %>%
  select(c("pretreatment", "dmsoA", "dmsoB", "drug1A", "drug1B")) %>%
  as.matrix()
```

The next datasets are metadata that describe the dimensions of the count data.

- These both need to be data frames.
- The sizes of these metadata must correspond to the dimensions of the counts data.

`pg_id` are just the unique IDs listed in the same order/sorted the same way as the count data and can be used for mapping between the count data and the metadata. This is required and very important because it is necessary to know the IDs and be able to map them to pgRNA constructs and counts data.

```{r eval = FALSE}
pg_ids <- example_data %>%
  dplyr::select("id")
```

Sample metadata is the information that describes the samples and is sorted the same order as the columns in the count data.

You need to have two columns in the metadata you provide. You'll need to specify the names of these columns in the `gimap_annotate()` function.

- `col_names` - Must match the colnames of the counts data being submitted
- `treatments` - A factor variable that describes the treatments for these data. The label for the control specified in this column will need to be supplied to the `gimap_normalize()` function.

```{r eval = FALSE}
sample_metadata <- data.frame(
  col_names = c("pretreatment", "dmsoA", "dmsoB", "drug1A", "drug1B"),
  drug_treatment = as.factor(c("pretreatment", "dmso", "dmso", "drug", "drug"))
)
```

We'll need to provide `counts`, `pg_ids` and `sample_metadata` to `setup_data()`.

Now let's setup our data using `setup_data()`. Optionally we can provide the metadata in this function as well so that it is stored with the data.

```{r eval = FALSE}
gimap_dataset <- setup_data(
  counts = counts,
  pg_ids = pg_ids,
  sample_metadata = sample_metadata
)
```

You'll notice that this set up gives us a list of formatted data. This contains the original counts we gave `setup_data()` function but also normalized counts, and the total counts per sample.

- `raw_counts`: The original counts data that illustrates the number of cells alive in the sample. This data has samples as the columns and the paired guide constructs as rows.
- `counts_per_sample`: Add up all the counts for each sample over all of the paired guide designs.
- Transformed data: This section contains the various types of normalized and adjusted data made from the raw counts data.
- `count_norm` - For each sample, the data is normalized `-log10(( counts +1) /  total counts for the sample over all the pg designs ))`
- `cpm` - For each sample this is calculated by taking the `counts / total counts for the sample over all the pg designs)*1e6`
- `log2cpm`: log-2 transformed counts per million this is calculated by `log2(cpms + 1)`
- pg_metadata: paired guide metadata - information that describes the paired-guided RNA designs. This may include the sequences used in the CRISPR design as well as what genes are targeted.
- `sample_metadata`: Metadata that describes the samples. This likely includes the time point information, replicates, sample IDs, and any other additional information that is needed regarding the experimental setup.


```{r eval = FALSE}
str(gimap_dataset)
```

Let's see how many rows of data we have.

```{r eval = FALSE}
nrow(gimap_dataset$transformed_data$log2_cpm)
```  

## Quality Checks

The first step is running some quality checks on our data. The `run_qc()` function will create a report we can look at to assess this.

The report includes several visualizations of raw/unfiltered data:

- distribution of normalized counts for each sample. The goal of determining this distribution is to identify pgRNA counts that are low at the start of the screen - before any type of phenotypic or growth selection is occurring, either in the T0 or plasmid sample. These low abundance pgRNAs should be removed from the analysis. [the goal section here doesn't fit my expectations/understanding of this plot.]
- histogram of `log2cpm` values for each individual sample: this helps users identify samples that do not have a normal distribution of reads and inform the upcoming filtering steps.
- sample correlation heatmap: generates a heatmap using cpm values for each replicate/sample. This heatmap gives an overview on how similar samples are. Replicates should correlate well, and cluster together, while each timepoint sample should be different from the T0. This analysis will also allow users to identify potential sample swaps, if correlation scores between replicates is poor.
- A histogram that shows the variance within replicates for each pgRNA construct. For each pgRNA construct, the variance among the 3 replicates is found and a distribution is constructed by looking at these variances together.
- A histogram of the log2 CPM values of pgRNA constructs at the plasmid time point. This graph relates to a plasmid filter that can be applied to the data, but the plot displays all of the data prior to a filter being applied.

This report also includes several visualizations after filters are applied:

There is a filter that flags pgRNA constructs where any of the time points have a count of zero.
  - We include a bar plot that shows the number of pgRNA constructs which have counts of zero in either 0, 1, 2, or 3 replicates.
  - We include a table that specifies how many pgRNAs would be filtered out by applying this filter.

There is a filter that flags pgRNA constructs that have low log2 CPM counts for the day 0 or plasmid time point.
  - The histogram of the log2 CPM values of pgRNA constructs at the plasmid time point mentioned earlier does have a dashed line specifying the lower outlier (or a user defined cutoff) and pgRNA constructs with a plasmid log2 CPM lower than that value can be filtered out.
  - We include a table that specifies how many pgRNAs would be filtered out by applying this filter.

There is a filter that flags pgRNA constructs that have low log2 CPM counts for the day 0 or plasmid time point.
  - The histogram of the log2 CPM values of pgRNA constructs at the plasmid time point mentioned earlier does have a dashed line specifying the lower outlier (or a user defined cutoff) and pgRNA constructs with a plasmid log2 CPM lower than that value can be filtered out.
  - We include a table that specifies how many pgRNAs would be filtered out by applying this filter.

```{r eval = FALSE}
run_qc(gimap_dataset,
  output_file = file.path(output_dir, "example_qc_report.Rmd"),
  overwrite = TRUE,
  quiet = TRUE
)
```

You can take a look at an [example QC report here](http://htmlpreview.github.io/?https://raw.githubusercontent.com/FredHutch/gimap/main/inst/example_qc_report.html).

## Filtering the data

After considering the QC report and which filters are appropriate/desired for your data, you can apply filters to the data using the `gimap_filter` function.

```{r eval = FALSE}
gimap_filtered <- gimap_dataset %>%
  gimap_filter()
```

How many rows of data do we have now after filtering?

```{r eval = FALSE}
nrow(gimap_filtered$filtered_data$transformed_log2_cpm)
```

```{r eval = FALSE}
str(gimap_filtered$filtered_data)
```

Let's take a look at how many rows of data we have left:

```{r eval = FALSE}
nrow(gimap_filtered$filtered_data$transformed_log2_cpm)
```

As you can see from the output above, there are fewer pgRNA constructs in the filtered dataset following completion of filtering.

The filtering step also stores two tables of information that you may want to use or report.

- `$filtered_data$removed_pg_ids` is a table that has the pgRNA construct IDs that are removed following completion of filtering in the `id` column and the relevant filter(s) that led to removal as a comma separated list in the `relevantFilters` column
- `$filtered_data$all_reps_zerocount_ids` is a table that lists the IDs of pgRNA constructs which had a count of 0 for all final timepoint replicates. These pgRNA constructs are NOT necessarily filtered out

Now that you've performed QC and filtering, the rest of the pipeline can be run

- First annotating the data set (expression levels, copy number, etc.) with DepMap data. For the annotation step, you *must* specify which `cell_line` your data uses so that the correct corresponding DepMap data is used for annotation. This function is `gimap_annotate()`
- Then the data is normalized with the `gimap_normalize()` function. `timepoints` needs to be specified pointing to the correct column names from the `sample_data` passed to the `setup_data()` function earlier.
- Genetic interaction scores are computed with the `calc_gi()` function.

## Normalization and calculating scores

```{r eval = FALSE}
gimap_dataset <- gimap_filtered %>%
  gimap_annotate(cell_line = "PC9") %>%
  # Whatever is specified for "control_name" is what will be used to normalize other data points
  gimap_normalize(
    treatments = "drug_treatment",
    control_name = "pretreatment"
  ) %>%
  calc_gi()
```

Here's what's included in the GI Scores table:

```{r eval = FALSE}
head(gimap_dataset$gi_scores)
```

Let's check out the top results

```{r eval = FALSE}
head(dplyr::arrange(gimap_dataset$gi_score, fdr))
```


## Plot the results

You can remove any samples from these plots by altering the `reps_to_drop` argument.

```{r eval = FALSE}
plot_exp_v_obs_scatter(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "exp_v_obs_scatter.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_treatment/exp_v_obs_scatter.png")
```

```{r eval = FALSE}
plot_rank_scatter(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "plot_rank_scatter.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_treatment/plot_rank_scatter.png")
```

```{r eval = FALSE}
plot_volcano(gimap_dataset)

# Save it to a file
ggsave(file.path(output_dir, "volcano_plot.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_treatment/plot_volcano.png")
```

### Plot specific target pair

We can pick out a specific pair to plot.

```{r eval = FALSE}
# "DUSP21_DUSP18" is top result so let's plot that
plot_targets(gimap_dataset, target1 = "DUSP21", target2 = "DUSP18")

# Save it to a file
ggsave(file.path(output_dir, "DUSP21_DUSP18.png"))
```

```{r echo = FALSE, out.width = "100%"}
knitr::include_graphics("output_treatment/DUSP21_DUSP18.png")
```

## Saving results to files

We can save the genetic interactions scores like this:

```{r eval = FALSE}
readr::write_tsv(gimap_dataset$gi_scores, file.path(output_dir, "gi_scores.tsv"))
```

We can save all these data as an RDS.

```{r eval = FALSE}
saveRDS(gimap_dataset, file.path(output_dir, "gimap_dataset_final_treatment.RDS"))
```

## Session Info

This is just for provenance purposes.

```{r }
sessionInfo()
```

---
title: "QC Report"
date: "`r Sys.Date()`"
author: "`r paste0('gimap (v', utils::packageVersion('gimap'), ')')`"
params:
  dataset: NULL
  filter_zerocount_target_col: NULL
  filter_plasmid_target_col: NULL
  filter_replicates_target_col: NULL
output: 
  html_document:
    theme: spacelab
    toc: true
    toc_float: TRUE
    df_print: kable
    code_folding: show
  pdf_document:
    toc: true
  always_allow_html: yes
  editor_options:
    chunk_output_type:console
---

<style>
  tbody tr:nth-child(odd){
    background-color: #D6E0F5;
}
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
knitr::opts_knit$set(progress = TRUE, verbose = FALSE)
library(gimap)
library(ggplot2)
library(purrr)
```

# Unfiltered/raw data

## Counts CDF

```{r}
qc_cdf(gimap_dataset)
```

## Histogram of cpm values by sample

```{r}
qc_sample_hist(gimap_dataset)
```


## Sample Correlation heatmap (unfiltered data)

```{r}
qc_cor_heatmap(gimap_dataset)
```

## Variance within replicates

```{r}
qc_variance_hist(gimap_dataset, filter_replicates_target_col = filter_replicates_target_col)
```

## Plasmid expression (log 2 cpm values for Day 0 sample/time point)

```{r}
qc_plasmid_histogram(gimap_dataset, filter_plasmid_target_col = filter_plasmid_target_col)
```

# Applying potential filters

## Filter pgRNAs where there is a count of 0 for any sample/time point

### Replicates with a pgRNA count of 0 

```{r}
qc_constructs_countzero_bar(gimap_dataset, filter_zerocount_target_col = filter_zerocount_target_col, filter_replicates_target_col = filter_replicates_target_col)
```

If this filter is applied, this is the number of pgRNAs that would be filtered out

```{r}
potentialFilter1 <- qc_filter_zerocounts(gimap_dataset, filter_zerocount_target_col = filter_zerocount_target_col)

potentialFilter1$reportdf
```

### Filter pgRNAs where there is a low log2 CPM value for the plasmid sample/time point

If this filter is applied, this is the number of pgRNAs that would be filtered out

```{r}
potentialFilter2 <- qc_filter_plasmid(gimap_dataset, filter_plasmid_target_col = filter_plasmid_target_col)

potentialFilter2$reportdf
```

### If both filters are applied

```{r}
combined_filters <- reduce(list(potentialFilter1$filter, potentialFilter2$filter), cbind)
```

| Which Filter(s) | Number of pgRNAs flagged for removal | Percent of total pgRNA constructs |
|:---------------|:------------------------------------|:----------|
| Zero count, but not low plasmid CPM | `r sum(combined_filters[,1] == TRUE & combined_filters[,2] == FALSE)` | `r round(sum(combined_filters[,1] == TRUE & combined_filters[,2] == FALSE) / nrow(combined_filters) * 100, 2)`|
| Low plasmid CPM, but not zero count | `r sum(combined_filters[,2] == TRUE & combined_filters[,1] == FALSE)` | `r round(sum(combined_filters[,2] == TRUE & combined_filters[,1] == FALSE) / nrow(combined_filters) * 100, 2)` |
| Either Zero count or Low plasmid CPM or both | `r sum(rowSums(combined_filters) >= 1)`| `r round(sum(rowSums(combined_filters) >= 1) / nrow(combined_filters) * 100, 2)`  |
| Both Zero count and Low plasmid CPM | `r sum(rowSums(combined_filters) == 2)` | `r round(sum(rowSums(combined_filters) == 2) / nrow(combined_filters) * 100, 2)` |
| Remaining pgRNAs flagged by no filters | `r sum(rowSums(combined_filters) == 0)` |  `r round(sum(rowSums(combined_filters) == 0) / nrow(combined_filters) * 100, 2)` |

# Session Info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03-annotate.R
\name{ctrl_genes}
\alias{ctrl_genes}
\title{Download and set up control genes}
\usage{
ctrl_genes(overwrite = TRUE, data_dir = gimap_data_dir())
}
\arguments{
\item{overwrite}{Should the file be redownloaded and reset up?}

\item{data_dir}{What directory should this be saved to? Default is a user data
directory.}
}
\description{
This function sets up the control genes file from DepMap is
called by the `gimap_annotate()`
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-qc.R
\name{qc_constructs_countzero_bar}
\alias{qc_constructs_countzero_bar}
\title{Create a bar graph that shows the number of replicates with a zero count for pgRNA constructs flagged by the zero count filter}
\usage{
qc_constructs_countzero_bar(
  gimap_dataset,
  filter_zerocount_target_col = NULL,
  filter_replicates_target_col = NULL,
  wide_ar = 0.75
)
}
\arguments{
\item{gimap_dataset}{The special gimap_dataset from the `setup_data` function
which contains the transformed data}

\item{filter_zerocount_target_col}{default is NULL; Which sample column(s)
should be used to check for counts of 0? If NULL and not specified, downstream analysis will select all sample columns}

\item{filter_replicates_target_col}{default is NULL; Which sample columns are
replicates whose variation you'd like to analyze; If NULL, the last 3 sample columns are used}

\item{wide_ar}{aspect ratio, default is 0.75}
}
\value{
a ggplot barplot
}
\description{
This bar graph first uses the specified `filter_zerocount_target_col` columns to flag pgRNA constructs
 that have a raw count of 0 in any one of those columns/samples of interest.
Then, it looks at the specified columns for the final day/sample replicates
(`filter_replicates_target_col`) to see for pgRNAs that were flagged by the filter,
how many of those replicate samples had raw counts of zeros. And it produces a bar plot reporting on this.
Note, if you select samples/columns to check with the filter that don't have the
 replicate samples, this graph won't be informative. So you want there to be overlap
  between the columns for the two target_col parameters to have an informative graph
}
\examples{
\dontrun{
gimap_dataset <- get_example_data("gimap")
qc_constructs_countzero_bar(gimap_dataset)

# or if you want to select a specific column(s) for
# looking at where/which samples zero counts are present for
qc_constructs_countzero_bar(gimap_dataset, filter_zerocount_target_col = 3:5)

# or if you want to select a specific column(s) for the final day/sample replicates
qc_constructs_countzero_bar(gimap_dataset, filter_replicates_target_col = 3:5)

# or some combination of those
qc_constructs_countzero_bar(gimap_dataset,
  filter_zerocount_target_col = 3:5,
  filter_replicates_target_col = 3:5
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-norm.R
\name{plot_crispr}
\alias{plot_crispr}
\title{Plot CRISPR scores after normalization}
\usage{
plot_crispr(.data = NULL, gimap_dataset, output_file = "crispr_norm_plot.png")
}
\arguments{
\item{.data}{Data can be piped in with tidyverse pipes from function to
function. But the data must still be a gimap_dataset}

\item{gimap_dataset}{A special dataset structure that is setup using the
`setup_data()` function.}

\item{output_file}{A file for the output}
}
\value{
A ggplot2 boxplot of the CRISPR scores separated by the type of
target. Can be used to determine the normalization has proceeded properly.
}
\description{
This plots normalization after CRISPR scores have been calculated
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter() \%>\%
  gimap_annotate(cell_line = "HELA") \%>\%
  gimap_normalize(
    timepoints = "day",
    missing_ids_file = tempfile()
  )

# Plot:
plot_crispr(gimap_dataset)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-qc.R
\name{qc_sample_hist}
\alias{qc_sample_hist}
\title{Create a histogram for the pgRNA log2 CPMs, faceted by sample}
\usage{
qc_sample_hist(gimap_dataset, wide_ar = 0.75)
}
\arguments{
\item{gimap_dataset}{The special gimap_dataset from the `setup_data` function
which contains the transformed data}

\item{wide_ar}{aspect ratio, default is 0.75}
}
\value{
sample_cpm_histogram a ggplot
}
\description{
This function uses pivot_longer to rearrange the data for plotting and then
plots sample specific histograms of the pgRNA cpm's
}
\examples{
\dontrun{
gimap_dataset <- get_example_data("gimap")
qc_sample_hist(gimap_dataset)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03-annotate.R
\name{cn_setup}
\alias{cn_setup}
\title{Download and set up DepMap CN}
\usage{
cn_setup(overwrite = TRUE, data_dir = gimap_data_dir())
}
\arguments{
\item{overwrite}{Should the files be redownloaded?}

\item{data_dir}{What directory should this be saved to? Default is a user data
directory.}
}
\description{
This function sets up the tpm data from DepMap is called
by the `gimap_annotate()` function if the cn_annotate = TRUE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{encrypt_creds_path}
\alias{encrypt_creds_path}
\title{Default creds path}
\usage{
encrypt_creds_path(app_name)
}
\arguments{
\item{app_name}{What app set up are you looking for? Supported apps are 'google' 'calendly' and 'github'}
}
\description{
Default creds path
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03-annotate.R
\name{gimap_annotate}
\alias{gimap_annotate}
\title{Annotate gimap data}
\usage{
gimap_annotate(
  .data = NULL,
  gimap_dataset,
  annotation_file = NULL,
  control_genes = NULL,
  cell_line_annotate = TRUE,
  custom_tpm = NULL,
  cell_line = NULL,
  annot_dir = gimap_data_dir(),
  refresh_annot = FALSE
)
}
\arguments{
\item{.data}{Data can be piped in with tidyverse pipes from function to
function. But the data must still be a gimap_dataset}

\item{gimap_dataset}{A special dataset structure that is setup using the
`setup_data()` function.}

\item{annotation_file}{If no file is given, will attempt to use the design
file from
https://media.addgene.org/cms/filer_public/a9/9a/
a99a9328-324b-42ff-8ccc-30c544b899e4/pgrna_library.xlsx}

\item{control_genes}{A vector of gene symbols (e.g. AAMP) that should be
labeled as control genes. These will be used for log fold change
calculations. If no list is given then DepMap Public 23Q4
Achilles_common_essentials.csv is used
https://depmap.org/portal/download/all/}

\item{cell_line_annotate}{(Optional) TRUE or FALSE you'd also like to have
cell_line_annotation from DepMap.}

\item{custom_tpm}{(Optional) You may supply your own data frame of transcript
per million expression to be used for this calculation if you can't or don't
want to use DepMap data annotation for your cell_line. This data frame needs
to have two columns: 'log2_tpm' that has the log2 tpm expression data for
this cell line and and 'genes' which needs to be gene symbols that match
those in the data. eg. "NDL1".
Note that you can use custom_tpm with cell_line_annotate but your custom_tpm
will be used instead of the tpm data from DepMap. However other data from
DepMap like CN will be added.}

\item{cell_line}{which cell line are you using? (e.g., HELA, PC9, etc.).
Required argument if cell_line_annotate is TRUE.}

\item{annot_dir}{Where should the annotation files be saved? Default is a
user data directory.}

\item{refresh_annot}{Should the annotation file paths saved as options be reset
and the files redownloaded? TRUE or FALSE.}
}
\value{
A gimap_dataset with annotation data frame that can be retrieve by using
gimap_dataset$annotation. This will contain information about your included
genes in the set.
}
\description{
In this function, a `gimap_dataset` is annotated as far as which
genes should be used as controls.
}
\examples{
\dontrun{

# By default DepMap annotation will be used to determine genes which are
# unexpressed. In the `gimap_normalize` this will by default be used to
# normalize to.
gimap_dataset <- get_example_data("gimap",
  data_dir = tempdir()
) \%>\%
  gimap_filter() \%>\%
  gimap_annotate(
    cell_line = "HELA",
    missing_ids_file =  tempfile(),
    annot_dir = tempdir()
  )


# You can also say cell_line_annotate = false if you don't want to use DepMap
# annotation BUT if you don't also specify that you say you are
# `normalize_by_unexpressed = FALSE` in the normalize step you will get a
# warning.
gimap_dataset <- get_example_data("gimap",
  data_dir = tempdir()
) \%>\%
  gimap_filter() \%>\%
  gimap_annotate(
    cell_line_annotate = FALSE,
    annot_dir = tempdir()
  ) \%>\%
  gimap_normalize(
    timepoints = "day",
    normalize_by_unexpressed = FALSE,
    missing_ids_file = tempfile()
  )

### CUSTOM TPM example
# Lastly, this is also an option:
# where custom data is provided to `custom_tpm` is a data frame with
# `genes` and `log2_tpm` as the columns.
gimap_dataset <- get_example_data("gimap",
  data_dir = tempdir()
) \%>\%
  gimap_filter() \%>\%
  gimap_annotate(
    cell_line = "HELA",
    custom_tpm = custom_tpm,
    annot_dir = tempdir()
  ) \%>\%
  gimap_normalize(
    timepoints = "day",
    missing_ids_file = tempfile()
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03-annotate.R
\name{tpm_setup}
\alias{tpm_setup}
\title{Download and set up DepMap TPM data}
\usage{
tpm_setup(overwrite = TRUE, data_dir = gimap_data_dir())
}
\arguments{
\item{overwrite}{should the files be re downloaded}

\item{data_dir}{What directory should this be saved to? Default is a user data
directory.}
}
\description{
This function sets up the tpm data from DepMap is
called by the `gimap_annotate()` function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{resolve_example_data_dir}
\alias{resolve_example_data_dir}
\title{Default directory for [get_example_data()] and related helpers}
\usage{
resolve_example_data_dir(data_dir)
}
\description{
Default directory for [get_example_data()] and related helpers
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-gi.R
\name{plot_targets}
\alias{plot_targets}
\title{Target bar plot for CRISPR scores}
\usage{
plot_targets(gimap_dataset, target1, target2, reps_to_drop = "")
}
\arguments{
\item{gimap_dataset}{A special dataset structure that is originally setup using
`setup_data()` and has had gi scores calculated with `calc_gi()`.}

\item{target1}{Name of the first target to be plotted e.g.}

\item{target2}{Name of the second target to be plotted e.g.}

\item{reps_to_drop}{Names of replicates that should be not plotted (Optional)}
}
\value{
A ggplot2 bar plot of the specific target's genetic interaction scores.
}
\description{
This plot is for when you'd like to examine a target pair
specifically -- meant to be functionally equivalent to Fig 3b
CRISPR scores for representative synthetic lethal paralog pairs.
Data shown are the mean CRISPR score for each single KO or DKO target across
 three biological replicates with replicate data shown in overlaid points.
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter() \%>\%
  gimap_annotate(cell_line = "HELA") \%>\%
  gimap_normalize(
    timepoints = "day"
  ) \%>\%
  calc_gi()

# To plot results, pick out two targets from the gi_score table
head(dplyr::arrange(gimap_dataset$gi_score, fdr))

# "TIAL1_TIA1" is top result so let's plot that
plot_targets(gimap_dataset, target1 = "TIAL1", target2 = "TIA1")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{figshare_manual_download_message}
\alias{figshare_manual_download_message}
\title{Build manual download instructions for Figshare files}
\usage{
figshare_manual_download_message(item, files)
}
\arguments{
\item{item}{Figshare item ID.}

\item{files}{Character vector of file names.}
}
\value{
A formatted message string.
}
\description{
Internal helper to build a warning message for manual downloads.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-gi.R
\name{plot_exp_v_obs_scatter}
\alias{plot_exp_v_obs_scatter}
\title{Expected vs Observed CRISPR Scatterplot}
\usage{
plot_exp_v_obs_scatter(gimap_dataset, facet_rep = FALSE)
}
\arguments{
\item{gimap_dataset}{A special dataset structure that is originally setup using `setup_data()` and has had gi scores calculated with `calc_gi()`.}

\item{facet_rep}{Should the replicates be wrapped with facet_wrap()?}
}
\value{
A ggplot2 scatterplot of the target level observed vs expected
CRISPR scores.
}
\description{
This plot is meant to be functionally equivalent to Fig S5K (for HeLa, equivalent of Fig 3a for PC9).
Scatter plot of target-level observed versus expected CRISPR scores in the screen.
The solid line is the linear regression line for the negative control (single KO) pgRNAs,
while dashed lines indicate the lower and upper quartile residuals.
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter() \%>\%
  gimap_annotate(cell_line = "HELA") \%>\%
  gimap_normalize(
    timepoints = "day",
    missing_ids_file = tempfile()
  ) \%>\%
  calc_gi()

# To plot results
plot_exp_v_obs_scatter(gimap_dataset)
plot_rank_scatter(gimap_dataset)
plot_volcano(gimap_dataset)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{save_example_treatment_data}
\alias{save_example_treatment_data}
\title{Set up example count data}
\usage{
save_example_treatment_data(data_dir = NULL)
}
\arguments{
\item{data_dir}{Optional data directory override.}
}
\value{
Returns the file path to folder where the example data is stored
}
\description{
Set up example count data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-qc.R
\name{plot_theme}
\alias{plot_theme}
\title{Standardized plot theme}
\usage{
plot_theme()
}
\value{
A ggplot2 theme that can be used on the plots.
}
\description{
this is a ggplot2 standardized plot theme for this package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{delete_example_data}
\alias{delete_example_data}
\title{Delete cached Figshare example files}
\usage{
delete_example_data(data_dir = NULL)
}
\arguments{
\item{data_dir}{Directory whose example files should be removed. The default
(`NULL`) uses the same session cache as [get_example_data()].}
}
\value{
Invisibly, the character vector of file paths that were removed (may
be length zero if nothing existed).
}
\description{
Removes only the known example filenames produced by
[get_example_data()] under `data_dir`. Paths are never taken from [options()],
so unrelated files cannot be deleted by mistake. Deletion is only allowed when
`data_dir` is the session example cache ([example_data_folder()]), the legacy
per-user `tools::R_user_dir("gimap", "data")`, or any directory under
[tempdir()] (for example test directories).
}
\examples{
delete_example_data()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00-setup_data.R
\name{setup_data}
\alias{setup_data}
\title{Making a new gimap dataset}
\usage{
setup_data(counts = NULL, pg_ids = NULL, sample_metadata = NULL)
}
\arguments{
\item{counts}{a matrix of data that contains the counts where rows are each
paired_guide target and columns are each sample}

\item{pg_ids}{the pgRNA IDs: metadata associated with the pgRNA constructs
that correspond to the rows of the counts data}

\item{sample_metadata}{metadata associated with the samples of the dataset
that correspond to the columns of the counts data.
Should include a column that has replicate information as well as a column
that contains timepoint information respectively (this will be used for log
 fold calculations). These columns should be factors.}
}
\value{
A special gimap_dataset to be used with the other functions in this
package.
}
\description{
This function allows people to have their data ready to be
 processed by gimap
}
\examples{
\dontrun{

counts <- get_example_data("count", data_dir = tempdir()) \%>\%
  dplyr::select(c(
    "Day00_RepA", "Day05_RepA", "Day22_RepA", "Day22_RepB",
    "Day22_RepC"
  )) \%>\%
  as.matrix()

pg_ids <- get_example_data("count", data_dir = tempdir()) \%>\%
  dplyr::select("id")

sample_metadata <- data.frame(
  col_names = c("Day00_RepA", "Day05_RepA", "Day22_RepA", "Day22_RepB", "Day22_RepC"),
  day = as.numeric(c("0", "5", "22", "22", "22")),
  rep = as.factor(c("RepA", "RepA", "RepA", "RepB", "RepC"))
)

gimap_dataset <- setup_data(
  counts = counts,
  pg_ids = pg_ids,
  sample_metadata = sample_metadata
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05-calculate_gi.R
\name{gimap_stats}
\alias{gimap_stats}
\title{Do tests --an internal function used by calc_gi() function}
\usage{
gimap_stats(gi_calc_double, gi_calc_single, use_ntc = TRUE, replicate = NULL)
}
\arguments{
\item{gi_calc_double}{a data.frame with adjusted double gi scores}

\item{gi_calc_single}{a data.frame with adjusted single gi scores}

\item{use_ntc}{Logical. If TRUE, uses a two-sample t-test comparing double
target GI scores against single target GI scores. If FALSE, uses a one-sample
t-test against mu=0 since single target GI scores have zero variance when
negative controls are not used.}

\item{replicate}{a name of a replicate to filter out from gi_calc_adj Optional}
}
\description{
Create results table that has t test p values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-qc.R
\name{qc_variance_hist}
\alias{qc_variance_hist}
\title{Create a histogram for the variance within replicates for each pgRNA}
\usage{
qc_variance_hist(
  gimap_dataset,
  filter_replicates_target_col = NULL,
  wide_ar = 0.75
)
}
\arguments{
\item{gimap_dataset}{The special gimap_dataset from the `setup_data` function
which contains the transformed data}

\item{filter_replicates_target_col}{default is NULL; Which sample columns are
replicates whose variation you'd like to analyze; If NULL, the last 3 sample columns are used}

\item{wide_ar}{aspect ratio, default is 0.75}
}
\value{
a ggplot histogram
}
\description{
This function uses pivot_longer to rearrange the data for plotting,
 finds the variance for each pgRNA construct (using row number as a proxy) and then
 plots a histogram of these variances
}
\examples{
\dontrun{
gimap_dataset <- get_example_data("gimap")
qc_variance_hist(gimap_dataset)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_example_data}
\alias{get_example_data}
\title{Returns example data for package}
\usage{
get_example_data(which_data, data_dir = NULL, refresh_data = FALSE)
}
\arguments{
\item{which_data}{One of `"count"`, `"count_treatment"`, `"meta"`, `"gimap"`,
`"gimap_treatment"`, or `"annotation"`; specifies which example dataset is returned.}

\item{data_dir}{Where files are stored. If `NULL` (default), example files are
written under `file.path(tempdir(), "gimap-example-data")` for the current R
session (they are not kept after the session ends). Pass an explicit directory
if you want a persistent copy (for example to mirror files elsewhere).}

\item{refresh_data}{If `TRUE`, removes known example filenames from `data_dir`
(only under safe locations; see [delete_example_data()]) before re-downloading.}
}
\value{
the respective example data either as a data frame or a specialized
gimap_dataset depending on what was requested.
}
\description{
This function loads and returns example data for the package.
Which dataset is returned must be specified. Data will be downloaded from Figshare
the first time it is used.
}
\examples{
\dontrun{

counts_timepoint <- get_example_data("count")
counts_treatment <- get_example_data("count_treatment")
gimap_timepoint_dataset <- get_example_data("gimap")
gimap_treatment_dataset <- get_example_data("gimap_treatment")
metadata <- get_example_data("meta")
annotation <- get_example_data("annotation")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03-annotate.R
\name{normalize_depmap_metadata}
\alias{normalize_depmap_metadata}
\title{Normalize DepMap metadata for downstream joins}
\usage{
normalize_depmap_metadata(depmap_metadata)
}
\description{
Normalize DepMap metadata for downstream joins
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_internet_available}
\alias{check_internet_available}
\title{Check if internet/URL is available}
\usage{
check_internet_available(url = "https://api.figshare.com", timeout_seconds = 5)
}
\arguments{
\item{url}{The URL to check availability for}

\item{timeout_seconds}{Timeout in seconds for the check}
}
\value{
TRUE if the URL is reachable, FALSE otherwise
}
\description{
Internal helper function to check if an internet resource is
reachable before attempting to download.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02-gimap_filter.R
\name{gimap_filter}
\alias{gimap_filter}
\title{A function to run filtering}
\usage{
gimap_filter(
  .data = NULL,
  gimap_dataset,
  filter_type = "both",
  cutoff = NULL,
  filter_zerocount_target_col = NULL,
  filter_plasmid_target_col = NULL,
  filter_replicates_target_col = NULL,
  min_n_filters = 1
)
}
\arguments{
\item{.data}{Data can be piped in with tidyverse pipes from function to
function. But the data must still be a gimap_dataset}

\item{gimap_dataset}{A special dataset structure that is setup using the
`setup_data()` function.}

\item{filter_type}{Can be one of the following: `zero_count_only`,
`low_plasmid_cpm_only` or `both`.
Potentially in the future also `rep_variation`, `zero_in_last_time_point`
 or a vector that includes multiple of these filters.}

\item{cutoff}{default is NULL, relates to the low_plasmid_cpm filter; the
cutoff for low log2 CPM values for the plasmid time period; if not specified,
The lower outlier (defined by taking the difference of the lower quartile and
 1.5 * interquartile range) is used}

\item{filter_zerocount_target_col}{default is NULL; Which sample column(s)
should be used to check for counts of 0? If NULL and not specified,
downstream analysis will select all sample columns}

\item{filter_plasmid_target_col}{default is NULL, and if NULL, will select
the first column only; this parameter specifically should be used to specify
the plasmid column(s) that will be selected}

\item{filter_replicates_target_col}{default is NULL, Which sample columns
are the final time point replicates; If NULL, the last 3 sample columns are
used. This is only used by this function to save a list of which pgRNA IDs
have a zero count for all of these samples.}

\item{min_n_filters}{default is 1; this parameter defines at least
how many/the minimum number of independent filters have to flag a pgRNA
construct before the construct is filtered when using a combination of
filters You should decide on the appropriate filter based on the results of
your QC report.}
}
\value{
a filtered version of the gimap_dataset returned in the
$filtered_data section filter_step_run is a boolean reporting if the filter
step was run or not (since it's optional)
metadata_pg_ids is a subset the pgRNA IDs such that these are the ones that
remain in the dataset following completion of filtering
transformed_log2_cpm is a subset the log2_cpm data such that these are the
ones that remain in the dataset following completion of filtering
removed_pg_ids is a record of which pgRNAs are filtered out once filtering
is complete
all_reps_zerocount_ids is not actually filtered data necessarily.
Instead it's just a record of which pgRNAs have a zero count in all final
timepoint replicates
}
\description{
This function applies filters to the gimap data. By default it
runs both the zero count (across all samples) and the low plasmid cpm
filters, but users can select a subset of these filters or even adjust the
behavior of each filter
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap", data_dir = tempdir()) \%>\%
  gimap_filter()

# To see filtered data
# gimap_dataset$filtered_data

# If you want to only use a single filter or some subset,
# specify which using the filter_type parameter
gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter(filter_type = "zero_count_only")
# or
gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter(filter_type = "low_plasmid_cpm_only")

# If you want to use multiple filters and more than one to flag a pgRNA
# construct before it's filtered out, use the `min_n_filters` argument
gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter(
    filter_type = "both",
    min_n_filters = 2
  )

# You can also specify which columns the filters will be applied to
gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter(
    filter_type = "zero_count_only",
    filter_zerocount_target_col = c(1, 2)
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-gi.R
\name{plot_volcano}
\alias{plot_volcano}
\title{Volcano plot for GI scores}
\usage{
plot_volcano(gimap_dataset, facet_rep = FALSE)
}
\arguments{
\item{gimap_dataset}{A special dataset structure that is originally setup using `setup_data()` and has had gi scores calculated with `calc_gi()`.}

\item{facet_rep}{Should the replicates be wrapped with facet_wrap()?}
}
\value{
A ggplot2 volcano plot of the target level genetic interaction scores.
}
\description{
This plot is meant to be functionally equivalent to Fig 5b (for HeLa, equivalent of Fig 3d for PC9).
Volcano plot of target-level GI scores
Blue points are synthetic lethal paralog GIs with GI < 0.5 and FDR < 0.1; red points are buffering paralog GIs with GI > 0.25 and FDR < 0.1.
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter() \%>\%
  gimap_annotate(cell_line = "HELA") \%>\%
  gimap_normalize(
    timepoints = "day"
  ) \%>\%
  calc_gi()

# To plot results
plot_exp_v_obs_scatter(gimap_dataset)
plot_rank_scatter(gimap_dataset)
plot_volcano(gimap_dataset)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{example_data_folder}
\alias{example_data_folder}
\title{Folder used for Figshare example data}
\usage{
example_data_folder(data_dir = NULL)
}
\arguments{
\item{data_dir}{Optional directory. If `NULL`, returns the session example
cache under [tempdir()] (same default as [get_example_data()]). Otherwise
resolves via [gimap_data_dir()].}
}
\value{
Normalized path to the folder where example files are stored.
}
\description{
Folder used for Figshare example data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00-setup_data.R
\name{gimap_object}
\alias{gimap_object}
\title{Make an empty gimap dataset object}
\usage{
gimap_object()
}
\value{
an empty `gimap_dataset` which is a named list which will
be filled by various `gimap` functions.
}
\description{
This function makes an empty gimap data object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03-annotate.R
\name{delete_annotation}
\alias{delete_annotation}
\title{Refresh the annotation files by redownloading them}
\usage{
delete_annotation()
}
\value{
options for tpm_file, cn_file, and ctrl_genes_file are set to NULL.
}
\description{
This function will set annotation file options to NULL so files
will be re-downloaded
}
\examples{

delete_annotation()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05-calculate_gi.R
\name{calc_gi}
\alias{calc_gi}
\title{Calculate Genetic Interaction scores}
\usage{
calc_gi(.data = NULL, gimap_dataset, use_lfc = FALSE, use_ntc = TRUE)
}
\arguments{
\item{.data}{Data can be piped in with tidyverse pipes from function to
function. But the data must still be a gimap_dataset}

\item{gimap_dataset}{A special dataset structure that is setup using the
`setup_data()` function.}

\item{use_lfc}{Should Log fold change be used to calculate GI scores instead
of CRISPR scores? If you do not have negative controls or CRISPR scores you
will need to set this to TRUE.}

\item{use_ntc}{If you do not have negative controls or do not want them to be
used set this to FALSE and then 0 will be used as the comparison value. Default
is that use_ntc = TRUE and looks for negative control genes.}
}
\value{
A gimap dataset with statistics and genetic interaction scores
calculated. Overall results in the returned object can be obtained using
gimap_dataset$overall_results Whereas target level genetic interaction
scores can be retrieved using `gimap_dataset$gi_scores`.
}
\description{
Create results table that has CRISPR scores, Wilcoxon rank-sum
test and t tests.
The output of the `gimap` package is genetic interaction scores which _is the
distance between the observed CRISPR score and the expected CRISPR score._
The expected CRISPR scores are what we expect for the CRISPR values should
two genes be unrelated to each other. The further away an observed CRISPR
scoreis from its expected the more we suspect genetic interaction.
This can be true in a positive way (a CRISPR knockout pair caused more cell
proliferation than expected) or in a negative way (a CRISPR knockout pair
caused more cell lethality than expected).

The genetic interaction scores are based on a linear model calculated for
each sample where `observed_crispr_single` is the outcome variable and
`expected_crispr_single` is the predictor variable.
For each sample: lm(observed_crispr_single ~ expected_crispr_single)

Using `y = mx+b`, we can fill in the following values:
* `y` = observed CRISPR score
* `x` = expected CRISPR score
* `m` = slope from linear model for this sample
* `b` = intercept from linear model for this sample

The intercept and slope from this linear model are used to adjust the CRISPR
scores for each sample:
single target gi score =
  observed single crispr - (intercept + slope * expected single crispr)
double_target_gi_score =
  double crispr score - (intercept + slope * expected double crispr)
These single and double target genetic interaction scores are calculated at
the construct level and are then summarized using a t-test to see if the the
distribution of the set of double targeting constructs is significantly
different than the overall distribution single targeting constructs.
After multiple testing correction, FDR values are reported.
Low FDR value for a double construct means high suspicion of paralogs.
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap",
  data_dir = tempdir()
) \%>\%
  gimap_filter() \%>\%
  gimap_annotate(
    cell_line = "HELA",
    annot_dir = tempdir()
  ) \%>\%
  gimap_normalize(
    timepoints = "day",
    missing_ids_file = tempfile()
  ) \%>\%
  calc_gi()

saveRDS(gimap_dataset, file.path(tempdir(), "gimap_dataset_final.RDS"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{key_encrypt_creds_path}
\alias{key_encrypt_creds_path}
\title{Get file path to an key encryption RDS}
\usage{
key_encrypt_creds_path()
}
\description{
Get file path to an key encryption RDS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{gimap_data_dir}
\alias{gimap_data_dir}
\title{Resolve and create data directory}
\usage{
gimap_data_dir(data_dir = NULL)
}
\arguments{
\item{data_dir}{Optional data directory. If NULL or empty, uses a user data dir.}
}
\value{
A normalized, existing directory path.
}
\description{
Internal helper to resolve a writable data directory for downloads.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-gi.R
\name{plot_rank_scatter}
\alias{plot_rank_scatter}
\title{Rank plot for target-level GI scores}
\usage{
plot_rank_scatter(gimap_dataset, reps_to_drop = "")
}
\arguments{
\item{gimap_dataset}{A special dataset structure that is originally setup using `setup_data()` and has had gi scores calculated with `calc_gi()`.}

\item{reps_to_drop}{Names of replicates that should be not plotted (Optional)}
}
\value{
A ggplot2 rankplot of the target level genetic interaction scores.
}
\description{
This plot is meant to be functionally equivalent to Fig 5a (for HeLa, equivalent of Fig 3c for PC9).
Rank plot of target-level GI scores.
Dashed horizontal lines are for GI scores of 0.25 and -0.5
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap") \%>\%
  gimap_filter() \%>\%
  gimap_annotate(cell_line = "HELA") \%>\%
  gimap_normalize(
    timepoints = "day"
  ) \%>\%
  calc_gi()

# To plot results
plot_exp_v_obs_scatter(gimap_dataset)
plot_rank_scatter(gimap_dataset)
plot_volcano(gimap_dataset)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04-normalize.R
\name{gimap_normalize}
\alias{gimap_normalize}
\title{Normalize Log fold changes}
\usage{
gimap_normalize(
  .data = NULL,
  gimap_dataset,
  normalize_by_unexpressed = TRUE,
  timepoints = NULL,
  treatments = NULL,
  control_name = NULL,
  adj_method = "negative_control_adj",
  num_ids_wo_annot = 20,
  rm_ids_wo_annot = TRUE,
  missing_ids_file = "missing_ids_file.csv",
  overwrite = TRUE
)
}
\arguments{
\item{.data}{Data can be piped in with a tidyverse pipe from function to
function. But the data must still be a gimap_dataset}

\item{gimap_dataset}{A special dataset structure that is setup using the
`setup_data()` function.}

\item{normalize_by_unexpressed}{TRUE/FALSE crispr data should be normalized
so that the median of unexpressed controls is 0. For this to happen set this
to TRUE but you need to have added TPM data in the gimap_annotate step using
cell_line_annotation or custom_tpm. If `unexpressed_ctrl_flag` is not present
(for example DepMap download failed), this is coerced to `FALSE` with a message.}

\item{timepoints}{Specifies the column name of the metadata set up in
`$metadata$sample_metadata`
that has a factor that represents the timepoints.
Timepoints will be made into three categories:
plasmid for the earliest time point, early for all middle timepoints and
late for the latest timepoints.
The late timepoints will be the focus for the calculations. The column used
for timepoints must be numeric or at least ordinal.}

\item{treatments}{Specifies the column name of the metadata set up in
`$metadata$sample_metadata`
that has a factor that represents column that specifies the treatment applied
to each. The replicates will be kept collapsed to an average.}

\item{control_name}{A name that specifies the data either in the treatments
column that should be used as the control. This could be the Day 0 of
treatment or an untreated sample.
For timepoints testing it will be assumed that the mininmum timepoint
is the control.}

\item{adj_method}{Must be one of three methods as stated by a character string
"negative_control_adj" or "no_adjustment". Default is "negative_control_adj"
"negative_control_adj" where CRISPR scores will be used for the GI scores
"no_adjustment" is where LFC adjusted will be used for the GI scores}

\item{num_ids_wo_annot}{default is 20; the number of pgRNA IDs to display to
console if they don't have corresponding annotation data;
ff there are more IDs without annotation data than this number, the output
will be sent to a file rather than the console.}

\item{rm_ids_wo_annot}{default is TRUE; whether or not to filter out pgRNA
IDs from the input dataset that don't have corresponding annotation data
available}

\item{missing_ids_file}{If there are missing IDs and a file is saved, where
do you want this file to be saved? Provide a file path.}

\item{overwrite}{Should existing normalized_log_fc data in the gimap_dataset
be overwritten?}
}
\value{
A gimap_dataset with normalized log FC as a data frame that can be
retrieve by using gimap_dataset$normalized_log_fc. This will contain
log2FC adjusted stored in a column named `log_adj` and the CRISPR scores
stored in a column named `crispr_score`.
genes in the set.
}
\description{
This calculates the log fold change for a gimap dataset based on
the annotation and metadata provided.
gimap takes in a counts matrix that represents the number of cells that have
each type of pgRNA this data needs some normalization before CRISPR scores
and Genetic Interaction scores can be calculated.

There are three steps of normalization.
1. `Calculate log2CPM` - First we account for different read depths across
samples and transforms data to log2 counts per million reads.
`log2((counts / total counts for sample)) * 1 million) + 1)`
2. `Calculate log2 fold change` - This is done by subtracting the log2CPM for
the pre-treatment from each sample.  control is what is highlighted.
The pretreatment is the day 0 of CRISPR treatment, before CRISPR pgRNAs
have taken effect.
`log2FC = log2CPM for each sample - pretreament log2CPM`

3. `Normalize by negative and positive controls` - Calculate a negative
control median for each sample and a positive control median for each sample
and divide each log2FC by this value.
log2FC adjusted = log2FC /
(median negative control for a sample - median positive control for a sample)
}
\examples{
\dontrun{

gimap_dataset_org <- get_example_data("gimap") \%>\%
  gimap_filter() \%>\%
  gimap_annotate(cell_line = "HELA") \%>\%
  gimap_normalize(
    timepoints = "day",
    missing_ids_file = tempfile()
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01-qc.R
\name{run_qc}
\alias{run_qc}
\title{Run Quality Control Checks}
\usage{
run_qc(
  gimap_dataset,
  output_file,
  overwrite = FALSE,
  filter_zerocount_target_col = NULL,
  filter_plasmid_target_col = NULL,
  filter_replicates_target_col = NULL,
  open_results = TRUE,
  ...
)
}
\arguments{
\item{gimap_dataset}{A special dataset structure that is setup using the
`setup_data()` function.}

\item{output_file}{Needs to be a string that ends with ".Rmd" What the
name of the output QC report file should be.}

\item{overwrite}{default is FALSE; whether to overwrite the QC Report file}

\item{filter_zerocount_target_col}{default is NULL; Which sample column(s)
should be
 used to check for counts of 0?  If NULL and not specified, downstream
 analysis will select all sample columns}

\item{filter_plasmid_target_col}{default is NULL; Which sample columns(s)
should be used to look at log2
CPM expression for plasmid pgRNA constructs? If NULL and not specified,
downstream analysis
will select the first sample column only}

\item{filter_replicates_target_col}{default is NULL; Which sample columns
are replicates whose variation you'd like to analyze; If NULL, the last
3 sample columns are used}

\item{open_results}{default is TRUE but if you don't want the report
automatically opened, choose FALSE.}

\item{...}{additional parameters are sent to `rmarkdown::render()`}
}
\value{
a QC report saved locally
}
\description{
This function takes a `gimap_dataset` and creates a QC report
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap")

run_qc(
  gimap_dataset,
  output_file = paste0(tempfile(), "_QC_Report.Rmd")
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02-gimap_filter.R
\name{qc_filter_plasmid}
\alias{qc_filter_plasmid}
\title{Create a filter for pgRNAs which have a low log2 CPM value for the
plasmid/Day 0 sample/time point}
\usage{
qc_filter_plasmid(
  gimap_dataset,
  cutoff = NULL,
  filter_plasmid_target_col = NULL
)
}
\arguments{
\item{gimap_dataset}{The special gimap_dataset from the `setup_data` function
which contains the log2 CPM transformed data}

\item{cutoff}{default is NULL, the cutoff for low log2 CPM values for the
plasmid time period; if not specified, The lower outlier (defined by taking
the difference of the lower quartile and 1.5 * interquartile range) is used}

\item{filter_plasmid_target_col}{default is NULL, and if NULL, will select
the first column only; this parameter specifically should be used to specify
the plasmid column(s) that will be selected}
}
\value{
a named list with the filter `filter` specifying which pgRNAs have
low plasmid log2 CPM (column of interest is `plasmid_cpm_filter`) and a
report df `reportdf` for the number and percent of pgRNA which have a low
plasmid log2 CPM
}
\description{
This function flags and reports which and how many pgRNAs have
low log2 CPM values for the plasmid/Day 0 sample/time point. If more than one
column is specified as the plasmid sample,
we pool all the replicate samples to find the lower outlier and flag
constructs for which any plasmid replicate has a log2 CPM value below the
cutoff
}
\examples{
\dontrun{
gimap_dataset <- get_example_data("gimap", data_dir = tempdir())

qc_filter_plasmid(gimap_dataset)

# or to specify a cutoff value to be used in the filter rather than the lower
# outlier default
qc_filter_plasmid(gimap_dataset, cutoff = 2)

# or to specify a different column (or set of columns to select)
qc_filter_plasmid(gimap_dataset, filter_plasmid_target_col = 1:2)

# or to specify a cutoff value that will be used in the filter rather than
# the lower outlier default as well as to specify a different column (or set
# of columns) to select
qc_filter_plasmid(gimap_dataset,
  cutoff = 1.75,
  filter_plasmid_target_col = 1:2
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_figshare}
\alias{get_figshare}
\title{Handler function for GET requests from Figshare}
\usage{
get_figshare(
  file_name = NA,
  item = "19700056",
  output_dir = tempdir(),
  return_list = FALSE
)
}
\arguments{
\item{file_name}{Which item are we downloading?}

\item{item}{What is the item we are retrieving?}

\item{output_dir}{Where should the file be saved?}

\item{return_list}{Should the list of files be returned instead of the file}
}
\value{
Downloads necessary annotation files from Figshare and reads them
in as data frames.
}
\description{
Handler function for GET requests from Figshare
}
\examples{
\dontrun{

get_figshare(
  return_list = TRUE,
  output_dir = tempdir()
)

get_figshare(
  file_name = "Achilles_common_essentials.csv",
  output_dir = tempdir()
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-qc.R
\name{qc_plasmid_histogram}
\alias{qc_plasmid_histogram}
\title{Create a histogram with plasmid log2 CPM values and ascertain a cutoff for low values}
\usage{
qc_plasmid_histogram(
  gimap_dataset,
  cutoff = NULL,
  filter_plasmid_target_col = NULL,
  wide_ar = 0.75
)
}
\arguments{
\item{gimap_dataset}{The special gimap_dataset from the `setup_data` function which contains the transformed data}

\item{cutoff}{default is NULL, the cutoff for low log2 CPM values for the plasmid time period; if not specified, The lower outlier (defined by taking the difference of the lower quartile and 1.5 * interquartile range) is used}

\item{filter_plasmid_target_col}{default is NULL, and if NULL, will select the first column only; this parameter specifically should be used to specify the plasmid column(s) that will be selected}

\item{wide_ar}{aspect ratio, default is 0.75}
}
\value{
a ggplot histogram
}
\description{
Find the distribution of plasmid (day0 data) pgRNA log2 CPM values, and ascertain a cutoff or filter for low log2 CPM values.
Assumes the first column of the dataset is the day0 data; do I need a better
method to tell, especially if there are reps?
}
\examples{
\dontrun{

gimap_dataset <- get_example_data("gimap")

qc_plasmid_histogram(gimap_dataset)

# or to specify a "cutoff" value that will be displayed as a dashed vertical line
qc_plasmid_histogram(gimap_dataset, cutoff = 1.75)

# or to specify a different column (or set of columns) to select
qc_plasmid_histogram(gimap_dataset, filter_plasmid_target_col = 1:2)

# or to specify a "cutoff" value that will be displayed as a dashed vertical
# line as well as to specify a different column (or set of columns) to select
qc_plasmid_histogram(gimap_dataset, cutoff = 2, filter_plasmid_target_col = 1:2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02-gimap_filter.R
\name{qc_filter_zerocounts}
\alias{qc_filter_zerocounts}
\title{Filter out samples of zero counts
Create a filter for pgRNAs which have a raw count of 0 for any sample/time
# point}
\usage{
qc_filter_zerocounts(gimap_dataset, filter_zerocount_target_col = NULL)
}
\arguments{
\item{gimap_dataset}{The special gimap_dataset from the `setup_data` function
which contains the raw count data}

\item{filter_zerocount_target_col}{default is NULL; Which sample column(s)
should be used to check for counts of 0? If NULL and not specified,
downstream analysis will select all sample columns}
}
\value{
a named list with the filter `filter` specifying which pgRNA have a
count zero for at least one sample/time point and a report df `reportdf` for
the number and percent of pgRNA which have a count zero for at least one
sample/time point
}
\description{
This function flags and reports which and how many pgRNAs have a
raw count of 0 for any sample/time point
}
\examples{
\dontrun{
gimap_dataset <- get_example_data("gimap", data_dir = tempdir())
qc_filter_zerocounts(gimap_dataset)

# or to specify a different column (or set of columns to select)
qc_filter_zerocounts(gimap_dataset, filter_zerocount_target_col = 1:2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03-annotate.R
\name{supported_cell_lines}
\alias{supported_cell_lines}
\title{List the supported cell lines}
\usage{
supported_cell_lines()
}
\value{
A list of the cell line names that are available in DepMap for use
for annotation in this package.
}
\description{
This function downloads the metadata for DepMap and lists which
cell lines are supported.
}
\examples{
\dontrun{

cell_lines <- supported_cell_lines()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots-qc.R
\name{qc_cor_heatmap}
\alias{qc_cor_heatmap}
\title{Create a correlation heatmap for the pgRNA CPMs}
\usage{
qc_cor_heatmap(gimap_dataset)
}
\arguments{
\item{gimap_dataset}{The special gimap_dataset from the `setup_data` function which contains the transformed data}
}
\value{
`sample_cor_heatmap` a pheatmap
}
\description{
This function uses the `cor` function to find correlations between the
 sample CPM's and then plots a heatmap of these
}
\examples{
\dontrun{
gimap_dataset <- get_example_data("gimap")
qc_cor_heatmap(gimap_dataset)
}
}
